! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.


!=======================================================================
      integer function dp__
     &  (inputfile,il,kssfile,kl,gwfile,gl,vkbfile,vl)
!=======================================================================
!     phantom function to avoid underscore problem in linking C-Fortran
!     call the function dp without underscore
!-----------------------------------------------------------------------
      implicit none
      integer il, kl, gl, vl
      character (len=il) :: inputfile
      character (len=kl) :: kssfile
      character (len=gl) :: gwfile
      character (len=vl) :: vkbfile
      integer, external :: dp
      dp__ = dp(inputfile,il,kssfile,kl,gwfile,gl,vkbfile,vl)
      end

!=======================================================================
      integer function dp_(inputfile,il,kssfile,kl,gwfile,gl,vkbfile,vl)
!=======================================================================
!     phantom function to avoid underscore problem in linking C-Fortran
!     call the function dp without underscore
!-----------------------------------------------------------------------
      implicit none
      integer il, kl, gl, vl
      character (len=il) :: inputfile
      character (len=kl) :: kssfile
      character (len=gl) :: gwfile
      character (len=vl) :: vkbfile
      integer, external :: dp
      dp_ = dp(inputfile,il,kssfile,kl,gwfile,gl,vkbfile,vl)
      end

!=======================================================================
      integer function dp(inputfile,il,kssfile,kl,gwfile,gl,vkbfile,vl)
!=======================================================================
!     main fortran program
!-----------------------------------------------------------------------
      use ggwdef
      use dpdef
      implicit none
      integer il, kl, gl, vl
      character (len=il) :: inputfile
      character (len=kl) :: kssfile
      character (len=gl) :: gwfile
      character (len=vl) :: vkbfile

      type(dpparam) p
      integer, parameter :: NQX = 6
      real, parameter :: deltapt = 0.01    ! 10^-2
      real, parameter :: delta   = 0.00001 ! 10^-5

      real q(3,NQX)

      integer nbv
      integer nkbzx
      integer nopr, ngr, nbr, nkr, nshr, nelr
      character (len=80) :: title(2)
      integer ntype, natom, mpsang, nsppol
      real a1(3), a2(3), a3(3)
      real b1(3), b2(3), b3(3)
      real :: op(3,3,96)
      integer, allocatable :: gvec(:,:)
      real, allocatable :: kibz(:,:)
      real, allocatable :: energy(:,:)
      real, allocatable :: occ(:,:)
      complex, allocatable :: wfg(:,:,:)
      complex, allocatable :: wfr(:,:,:)
      real ucvol, bzvol
      real ula, tpula
      integer, allocatable :: type(:)
      double precision, allocatable :: xred(:,:), xcart(:,:)
      real, allocatable :: vkbsign(:,:), vkb(:,:,:,:), vkbd(:,:,:,:)
!      complex, allocatable :: gradvnl(:,:,:,:)
      logical :: inclvkb = .false.
      type(kb_potential) kb
      type(symmetry) symstr
      type(bzstructure) k

      complex, allocatable :: gwshift(:,:)
      real, allocatable :: gwenergy(:,:)

      real brl2cc(3,3), bcc2rl(3,3)

      real, allocatable :: rho2(:)
      real omegaplasma
      real efermi, ofermi
      double precision tve, bce, mge, dge
      real dsffact
      real omegaev

      integer nr
      integer nfft1, nfft2, nfft3, nfft1a, nfft2a, nfft3a
      integer, allocatable :: igfft(:,:,:,:), irottb(:,:)
      integer, allocatable :: grottb(:,:,:), grottbm1(:,:,:)
      type(mesh) fftmesh
      integer addshell

      real, allocatable :: wtk(:)
      real, allocatable :: kbz(:,:)
      integer, allocatable :: ktab(:), ktabi(:), ktabo(:), ktabr(:,:)
      complex, allocatable :: ktabp(:)

      real, allocatable :: qplusG(:,:)
      real, allocatable :: Vc(:,:)

      complex, allocatable :: fxc(:,:), fxcnlda(:,:,:) ! fxc in G,G'
      complex, allocatable :: chi0(:,:,:), lwing(:,:,:), uwing(:,:,:)
      complex fxcchi
      complex, allocatable :: mtmp(:,:)
      complex, allocatable :: epsilon(:,:)
      complex, allocatable :: epsilonm1_g(:)
      complex :: epsilonm1_gg
      complex, allocatable :: deltav(:)
      complex, allocatable :: chi_g(:,:)

      complex, allocatable :: epsmlf(:,:)
      complex, allocatable :: epsmnlf(:,:)
      real :: epsmlf0(8), epsmnlf0(8)


      real rtemp
      integer i, j, io, ig, i1, i2, i3, ib, ik, iq, ia
      integer ig1, ig2
      integer istat

!     kernel (Vc+fxc) related variables
      logical ltemp
      complex, allocatable :: epsm1q(:,:)
      complex, allocatable :: kernel(:,:)

      integer irecl
      real, external :: vlngth
      complex, external :: cdotu
      double precision, external :: fermi_energy, fermi_occupation
      integer, external :: is_insulator
      
      complex (kind(0.0d0)), allocatable :: Tkernel(:,:,:)
      complex (kind(0.0d0)), allocatable :: mtmpd(:,:), chi2d(:,:)
      complex (kind(0.0d0)), allocatable :: chi_gd(:,:),fxcw(:,:,:)
      complex (kind(0.0d0)), allocatable :: chi(:),chi3(:)
      
      integer, external :: test_wfn_etsf
      type(crystal) :: crystr
      type(transposed_space) :: transpa
      type(electronic_structure) :: elecstr
      integer :: symrel(3,3,96)
      double precision :: tnons(3,96)
      
      character (len=3) :: kdepg, kdepb
      integer :: mnp = 1
      double precision, allocatable :: kweight(:)
      double precision, allocatable :: kb_ff(:,:,:,:,:)
      double precision, allocatable :: kb_ffd(:,:,:,:,:)
      double precision :: spacegroup
      
      include 'version.f'

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      print *, 'program DP, version ', version
      print *, 'calculate Dielectric Properties'
      print *, 'TDDFT code in Frequency-Reciprocal space ',
     &         'on a PlaneWaves basis'
      print *, 'Copyright 1998-2007, Valerio Olevano, ',
     &         'Lucia Reining, Francesco Sottile, CNRS.'
      print *, 'All rights reserved'
      print *, 'contributors: Fabien Bruneval '
      print *, 'DP is distributed in Academic Institutions ',
     &         'under the terms of the'
      print *, 'Scientific Software Open Source Academic for Free ',
     &         'License'
      print *, '(see license.txt or http://www.dp-code.org/license)'
      print *

!     start clock
      call pclock(0)

!     write file used in the calculation
      if(il/=0) print *, 'input file: ', inputfile
      if(kl/=0) then
        print *, 'wavefunctions file: ', kssfile(1:kl)
      else
        stop 'missing kss file'
      endif
      if(gl/=0) print *, 'gw file: ', gwfile(1:gl)
      if(vl/=0) print *, 'vkb file: ', vkbfile(1:vl)
      print *

!     read parameters of the qps, kss (qplda) file
      i2 = test_wfn_etsf(kssfile,kl,nopr,ngr,nbr,nkr,
     &  nsppol,nsppol,nsppol,ntype,natom,mpsang,mnp,kdepg,kdepb,nelr)
      if(i2==-1) then
        call testlda(kssfile,kl,nopr,nkr,nbr,ngr,nshr,i1,i2,
     &             ntype,natom,mpsang,nelr)
        if(i2/=0.and.i2/=1.and.i2/=2.and.i2/=502.and.i2/=602) then
          stop 'unknown file form'
        endif
      else if(i2/=100) then
        stop 'wrong NetCDF file'
      endif
      if(mnp/=1) stop 'max number of projectors'

!     read in parameters of the calcuation
      call rddpin(inputfile,il,p)

!     now test input parameters from input and kss file
!     and assume some defults
      if(p%nkibz == -1) p%nkibz = nkr
      if(nkr < p%nkibz) then
        print *, 'ERROR: kss file contains only ', nkr, ' k-points'
        print *, '       instead of ', p%nkibz, ' required;'
        p%nkibz = nkr
        print *, '       the calculation will be done with NKIBZ = ',
     &           p%nkibz
        print *
      else if(nkr > p%nkibz) then
        print *, 'WARNING: kss file contains ', nkr, ' k-points;'
        print *, '         assuming expert user;'
        print *, '         the calculation will be done with the first',
     &           p%nkibz, ' k-points of the file.'
        print *
      endif

      if(p%npwwfn == -1) p%npwwfn = ngr
!     calculate NPWVEC as the biggest between NPWMAT and NPWWFN
      if(p%npwwfn > p%npwmat) then
        p%npwvec = p%npwwfn
      else
        p%npwvec = p%npwmat
      endif
      if(ngr < p%npwvec) then
        print *, 'ERROR: kss file contains only ', ngr, ' planewaves'
        p%npwvec = ngr
        if(p%npwwfn > ngr) then
          p%npwwfn = ngr
          print *, '       the calculation will be done with NPWWFN = ',
     &             p%npwwfn
        endif
        if(p%npwmat > ngr) then
          p%npwmat = ngr
          print *, '       the calculation will be done with NPWMAT = ',
     &             p%npwmat
        endif
        print *
      endif

      if(p%nbnds == -1) then
        if(nbr < 100) then
          p%nbnds = nbr
        else
          p%nbnds = 100
        endif
      endif
      if(nbr < p%nbnds) then
        print *, 'ERROR: kss file contains only ', nbr, ' levels'
        print *, '       instead of ', p%nbnds, ' required;'
        p%nbnds = nbr
        print *, '       the calculation will be done with NBNDS = ',
     &           p%nbnds
        print *
      endif

      if(p%nop == -1) p%nop = nopr
      if(nopr < p%nop) then
        print *, 'ERROR: kss file contains only ', nopr, ' symmetries'
        print *, '       instead of ', p%nop, ' required;'
        p%nop = nopr
        print *, '       the calculation will be done with NOP = ',
     &           p%nop
        print *
      else if(nopr > p%nop) then
        print *, 'WARNING: kss file contains ', nopr, ' symmetries;'
        print *, '         assuming expert user;'
        print *, '         the calculation will be done with the first',
     &           p%nop, ' symmetries of the file.'
        print *
      endif
      p%nopq = nopr
      p%ninvq = 2

      if(p%nel == -1) p%nel = nelr
      if(p%nel == -1) stop 'error: must specify nel'
      if(p%nel /= nelr) then
        print *, 'WARNING: kss file contains ', nelr, ' electrons;'
        print *, '         assuming expert user;'
        print *, '         the calculation will be done with ', p%nel
      endif 

!     print parameters of the calculation
      call print_parameters(p)
      
!     allocate frequencies
      allocate(p%omega(p%nomega))
      do io = 1, p%nomega
        p%omega(io) = cmplx(p%omegai + (io-1)*p%domega , p%broad)
      enddo

!     allocate LDA electronic structure variables
      allocate(kibz(3,p%nkibz),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(gvec(3,p%npwvec),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(energy(p%nkibz,p%nbnds),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(occ(p%nkibz,p%nbnds),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(wfg(p%npwwfn,p%nbnds,p%nkibz),stat=istat)
      if(istat.ne.0) stop 'wavefunctions(G) out of memory'
      allocate(type(natom),xred(3,natom),xcart(3,natom))
          allocate(vkbsign(mpsang,ntype))
          allocate(vkb(p%npwwfn,ntype,mpsang,p%nkibz),stat=istat)
          if(istat.ne.0) stop 'out of memory'
          allocate(vkbd(p%npwwfn,ntype,mpsang,p%nkibz),stat=istat)
          if(istat.ne.0) stop 'out of memory'

!     read in LDA band structure and information about material
      if(i2==100) then
        allocate(transpa%k(3,p%nkibz))
        allocate(kweight(p%nkibz))
        allocate(transpa%sh(nshr))
        allocate(elecstr%oc(p%nbnds,p%nkibz),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        allocate(elecstr%en(p%nbnds,p%nkibz),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        allocate(elecstr%qps(p%nbnds,p%nkibz),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        allocate(elecstr%wfg(p%npwwfn,p%nbnds,p%nkibz),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        allocate(kb%vkbsign(mpsang,ntype))
        allocate(kb_ff(p%npwwfn,p%nkibz,mnp,mpsang,ntype))
        if(istat.ne.0) stop 'out of memory'
        allocate(kb_ffd(p%npwwfn,p%nkibz,mnp,mpsang,ntype))
        if(istat.ne.0) stop 'out of memory'
        call read_wfn_etsf(kssfile,kl,p%nop,p%npwvec,p%npwwfn,
     &       p%nbnds,p%nkibz,ntype,natom,mpsang,mnp,spacegroup,
     &       crystr%rprimd,p%nel,type,xred,symrel,tnons,transpa%k,
     &       kweight,gvec,elecstr%fermie,
     &      elecstr%oc,elecstr%en,elecstr%wfg,elecstr%qps,
     &       kb%vkbsign,kb_ff,kb_ffd)
        allocate(kb%vkb(p%npwwfn,mpsang,p%nkibz,ntype))
        allocate(kb%vkbd(p%npwwfn,mpsang,p%nkibz,ntype))
        a1(:) = crystr%rprimd(:,1)
        a2(:) = crystr%rprimd(:,2)
        a3(:) = crystr%rprimd(:,3)
        do ia = 1, natom
          xcart(:,ia) = xred(1,ia)*crystr%rprimd(:,1) +
     &                  xred(2,ia)*crystr%rprimd(:,2) +
     &                  xred(3,ia)*crystr%rprimd(:,3)
        enddo
        symstr%nsym = p%nop
         do i = 1, symstr%nsym
          symstr%sym(i)%r(:,:) = symrel(:,:,i)
          symstr%sym(i)%t(:)   = tnons(:,i)
          symstr%sym(i)%tr = 1
        enddo
        call setup_symmetry_structure(symstr)
        kibz(:,:) = transpa%k(:,:)
        vkbsign(:,:) = kb%vkbsign(:,:)
        kb%vkb = 0
        kb%vkbd = 0
        vkb = 0
        vkbd = 0
        do i = 1, ntype
          do il = 1, mpsang
            do ik = 1, p%nkibz
              kb%vkb(:,il,ik,i) = kb_ff(:,ik,1,il,i)
              kb%vkbd(:,il,ik,i) = kb_ffd(:,ik,1,il,i)
              vkb(:,i,il,ik) = kb%vkb(:,il,ik,i)
              vkbd(:,i,il,ik) = kb%vkbd(:,il,ik,i)
            enddo
          enddo
        enddo
        deallocate(kb_ff,kb_ffd)
        
        wfg(:,:,:) = elecstr%wfg(:,:,:)
        do ik = 1, p%nkibz
          do ib = 1, p%nbnds
            energy(ik,ib) = elecstr%en(ib,ik)
            occ(ik,ib) = elecstr%oc(ib,ik)
          enddo
        enddo
        deallocate(elecstr%wfg)
        inclvkb = .true.
        inquire(file='vkbfalse',exist=ltemp)
        if(ltemp) inclvkb=.false.
        if(.not.inclvkb) print *, 'WARNING: excluding Vnl term'

      endif
      if(i2==0.or.i2==1) then
!       file unformatted -> i1=1
        i1 = 1
        call rdlda(kssfile,kl,nopr,p%nbnds,p%nkibz,p%npwvec,p%npwwfn,
     &             nopr,nbr,nkr,ngr,
     &             title,i1,i2,i3,a1,a2,a3,op,gvec,kibz,energy,occ,wfg)
      else if(i2==502.or.i2==602) then
        call rdkss(kssfile,kl,nopr,p%nbnds,p%nkibz,p%npwvec,p%npwwfn,
     &           title,a1,a2,a3,symstr,op,gvec,kibz,energy,occ,wfg,
     &           ntype,natom,mpsang,type,xred,vkbsign,vkb,vkbd)
        do ia = 1, natom
          xcart(:,ia) = xred(1,ia)*a1(:) +
     &                  xred(2,ia)*a2(:) +
     &                  xred(3,ia)*a3(:)
        enddo
        inclvkb = .true.
        inquire(file='vkbfalse',exist=ltemp)
        if(ltemp) inclvkb=.false.
        if(.not.inclvkb) print *, 'WARNING: excluding Vnl term'
      endif
      if(i2==0.or.i2==1.or.i2==2) then
        inquire(file=vkbfile,exist=inclvkb)
        if(inclvkb) then
          call testvkb(vkbfile,vl,p%npwwfn,p%nkibz,ntype,natom,mpsang)
           call rdvkb(vkbfile,vl,p%npwwfn,p%nkibz,ntype,natom,mpsang,
     &               type,xcart,vkbsign,vkb,vkbd)
        endif
      endif
      
      if(p%nop /= 1) then
        p%nop = symstr%nsym
      else
        symstr%nsym = 1
        print *, 'using only identity '
        print *
      endif
      
      do i = 1, p%nop
        op(:,:,i) = symstr%itsym(i)%r(:,:)
      enddo

      call pclock(10)
      
      call lattice(a1,a2,a3,b1,b2,b3,ucvol,bzvol)
!     transformation matrices rl -> cc and cc -> rl
      brl2cc(:,1) = b1(:) ; brl2cc(:,2) = b2(:) ; brl2cc(:,3) = b3(:)
      bcc2rl(:,:) = brl2cc(:,:)
      call rginv(3,bcc2rl,3)
!     note that ULA is not guaranteed to be the same as that used by
!     RJN; it is chosen to be sqrt(2) * |a1|
      ula = sqrt(2.0) * sqrt(a1(1)**2+a1(2)**2+a1(3)**2)
!     ula should be the lattice parameter, and tpula = 2pi/a
      tpula = 2.0 * PI / ula
      
      call reduceto1bz(p%qpg,p%q(:,1),p%g,b1,b2,b3)
      call identifyg(p%g,p%ig,gvec,p%npwvec)
      if(p%ig > p%npwmat) then
        print *, 'ERROR: G vector out of limits:'
        print *, '       G = ', p%g
        print *, '       increase NPWMAT.'
        if(p%ig<=p%npwvec) then
          print *, '       NPWMAT should contains at least ', p%ig
        endif
        stop 'dp, error: g vector out of limits'
      endif
      if(all(abs(p%q(:,1))<deltapt)) p%QZERO = .true.
      if(all(abs(p%q(:,1))<delta)) then
        p%nq = 6
        p%q(1,1) = 1.0 ; p%q(2,1) = 0.0 ; p%q(3,1) = 0.0       ! (100)
        p%q(1,2) = 0.0 ; p%q(2,2) = 1.0 ; p%q(3,2) = 0.0       ! (010)
        p%q(1,3) = 0.0 ; p%q(2,3) = 0.0 ; p%q(3,3) = 1.0       ! (001)
        p%qcc(1,4) = 1.0 ; p%qcc(2,4) = 0.0 ; p%qcc(3,4) = 0.0 ! (x)
        p%qcc(1,5) = 0.0 ; p%qcc(2,5) = 1.0 ; p%qcc(3,5) = 0.0 ! (y)
        p%qcc(1,6) = 0.0 ; p%qcc(2,6) = 0.0 ; p%qcc(3,6) = 1.0 ! (z)
        p%q(:,4) = matmul(bcc2rl,p%qcc(:,4))
        p%q(:,5) = matmul(bcc2rl,p%qcc(:,5))
        p%q(:,6) = matmul(bcc2rl,p%qcc(:,6))
        do j = 1, p%nq
!         normalization
          rtemp = sqrt(dot_product(p%q(:,j),p%q(:,j)))
          p%q(:,j) = p%q(:,j) / rtemp
        enddo
        
        do j = 1, p%nq
          p%qcc(:,j) = matmul(brl2cc,p%q(:,j))
        enddo
        p%qcc2pioa(:,:) = p%qcc(:,:) / tpula
        print *, 'calculating at q=0'
        print *, 'along the following polarization directions:' 
        do j = 1, p%nq
          write(6,120) (p%q(i,j),i=1,3), (p%qcc(i,j),i=1,3),
     &                 (p%qcc2pioa(i,j),i=1,3), tpula
120       format(' q = (',f9.6,',',f9.6,',',f9.6,') [r.l.u.]',/,
     &           ' q = (',f9.6,',',f9.6,',',f9.6,') [c.c.]'/
     &           ' q = (',f9.6,',',f9.6,',',f9.6,') ',
     &           '[c.c. units of 2pi/a = ',f9.6,' a.u.]'/)
        enddo
        p%q(:,:) = p%q(:,:) * delta
        p%qcc(:,:) = p%qcc(:,:) * delta
        p%qcc2pioa(:,:) = p%qcc2pioa(:,:) * delta
      else
        p%nq = 1
        p%qcc(:,1) = matmul(brl2cc,p%q(:,1))
        p%qcc2pioa(:,1) = p%qcc(:,1) / tpula
        print *, 'calculating at the following q:'
        write(6,120) (p%q(i,1),i=1,3), (p%qcc(i,1),i=1,3),
     &               (p%qcc2pioa(i,1),i=1,3), tpula
      endif

      p%gcc(:) = real(p%g(:))
      p%gcc(:) = matmul(brl2cc,p%gcc(:))
      p%gcc2pioa(:) = p%gcc(:) / tpula
      print *, 'calculating at the following G:'
      write(6,125) p%g(:), p%gcc(:), p%gcc2pioa(:), tpula
125   format(' G = (',i2,',',i2,',',i2,') [r.l.u.]',/,
     &       ' G = (',f9.6,',',f9.6,',',f9.6,') [c.c.]'/
     &       ' G = (',f9.6,',',f9.6,',',f9.6,') ',
     &       '[c.c. units of 2pi/a = ',f9.6,' a.u.]'/)
     
     
      call findshells(b1,b2,b3,p%npwvec,p%npwmat,gvec)

!     setup FFT real space mesh and tables for FFT igfft
      allocate(igfft(p%npwvec,5,5,5),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      addshell = 2
      if(p%QZERO) addshell=0
      call select_divisor_mesh(symstr,fftmesh%nrr(:))
      call setmesh(p%npwvec,p%npwwfn,p%npwmat,gvec,p%alias,addshell,
     &   fftmesh%nrr,igfft)
      fftmesh%nr = fftmesh%nrr(1)*fftmesh%nrr(2)*fftmesh%nrr(3)
      nfft1 = fftmesh%nrr(1)
      nfft1a = nfft1
      nfft2 = fftmesh%nrr(2)
      nfft3 = fftmesh%nrr(3)
      nr = fftmesh%nr
      
!     set up table indicating rotations of r-points
      allocate(irottb(nr,96),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(grottb(p%npwvec,2,96),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      !allocate(grottbm1(p%npwvec,2,nopr),stat=istat)
      !if(istat.ne.0) stop 'out of memory'
      !call surot(op,nopr,nfft1,nfft1a,nfft2,nfft3,nr,p%npwvec,
      !&           gvec,grottb,irottb,grottbm1)
      
      call setup_rrotation_table(symstr,fftmesh,irottb)
      call setup_grotation_table(symstr,p%npwvec,gvec,grottb)
      
!     nkbzx is the maximum possible number of k-point in the whole BZ
      nkbzx = p%nkibz * 96
      allocate(kbz(3,nkbzx),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(ktab(nkbzx),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(ktabi(nkbzx),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(ktabo(nkbzx),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(ktabr(nr,nkbzx),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(ktabp(nkbzx),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(wtk(p%nkibz),stat=istat)
      if(istat.ne.0) stop 'out of memory'
!      call identk(kibz,p%nkibz,nkbzx,nr,p%nop,p%ninv,irottb,
!     &            op,kbz,ktab,ktabr,ktabi,ktabo,p%nkbz,wtk)
      
      call setup_ibz(p%nkibz,kibz,k)
      call expand_ibz(symstr,k,p%ninv,ktabi)
      p%nkbz = k%nbz
      do ik = 1, p%nkbz
        kbz(:,ik) = k%bz(ik)%rc(:)
        ktab(ik) = k%bz(ik)%ibz
        ktabo(ik) = k%bz(ik)%isym
        ktabr(:,ik) = irottb(:,k%bz(ik)%isym)
        ktabp(ik) = exp( (0,1) * 2 * pi * 
     &  dot_product(kibz(:,ktab(ik)),symstr%isym(ktabo(ik))%t(:)) )
!        print *, ktabp(ik)
      enddo
      do ik = 1, p%nkibz
        wtk(ik) = k%ibz(ik)%w
      enddo
      
      allocate(gwenergy(p%nkibz,p%nbnds),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      gwenergy(:,:) = energy(:,:)
      nbv = p%nel / 2
      p%homo = p%nel / 2
      p%lumo = p%homo+1
      p%nbndv = p%homo - p%lomo + 1
      p%nbndc = p%nbnds - p%lumo + 1
      p%nt = p%nkbz * p%nbndv * p%nbndc

      if(p%GW) then

!       read gw energies
        allocate(gwshift(p%nkibz,p%nbnds),stat=istat)
        if(istat.ne.0) stop 'out of memory'

        call rdgw(gwfile,gl,p%nkibz,p%nkbz,p%nbnds,p%lomo,
     &            kbz,ktab,gwshift,p%nkibz)

        gwenergy(:,:) = energy(:,:) + gwshift(:,:)
        print *, ' k       GW energies [eV]'
        do ik = 1, p%nkibz
          write(*,'(i3,7x,10f7.2/50(10x,10f7.2/))')
     &         ik, (HARTREE*real(gwenergy(ik,ib)),ib=1,p%nbnds)
        enddo
        print *
!        print *, ' k       Im GW energies [eV]'
!        do ik = 1, p%nkibz
!          write(*,'(i3,7x,10f7.2/50(10x,10f7.2/))')
!     &         ik, (HARTREE*aimag(gwenergy(ik,ib)),ib=1,p%nbnds)
!        enddo
!        print *
!       use more refined gw energy
        deallocate(gwshift)

      else if(p%SO) then
        if(((p%nel/2)*2) /= p%nel) then
          print *, 'Warning, metallic system'
        endif
        nbv = p%nel / 2
        do ik = 1, p%nkibz
          do ib = nbv+1, p%nbnds
            gwenergy(ik,ib) = gwenergy(ik,ib) + p%soenergy
          enddo
        enddo

      endif

!     calculate the Fermi level and occupation numbers
      if(is_insulator(p%ns,p%nbnds,p%nkibz,p%nel,gwenergy,
     &   tve,bce,mge,dge) == 1) then
        print *, 'insulator'
        write(*,'(" top of valence       ",f6.2)') tve*HARTREE
        write(*,'(" bottom of conduction ",f6.2)') bce*HARTREE
        write(*,'(" minimum band gap     ",f6.2)') mge*HARTREE
        write(*,'(" direct band gap      ",f6.2)') dge*HARTREE
        print *
        do ik = 1, p%nkibz
          do ib = 1, nbv
            occ(ik,ib) = 2.0
          enddo
          do ib = nbv+1, p%nbnds
            occ(ik,ib) = 0.0
          enddo
        enddo
      else 
       print *, 'metal or semimetal'
       efermi = fermi_energy(p%ns,p%nbnds,p%nkibz,p%nel,gwenergy,wtk)
      ofermi = fermi_occupation(p%ns,p%nbnds,p%nkibz,p%nel,gwenergy,wtk)
       write(*,'(" Fermi energy     ",f6.2)') efermi*HARTREE
       write(*,'(" Fermi occupation ",f8.4)') ofermi
       print *
       do ik = 1, p%nkibz
         do ib = 1, p%nbnds
          if(gwenergy(ik,ib) < efermi) occ(ik,ib) = 2.0
          if(gwenergy(ik,ib) > efermi) occ(ik,ib) = 0.0
          if(gwenergy(ik,ib) == efermi) occ(ik,ib) = ofermi
          print *, ' k=', ik, ' b=', ib, ' occ=', occ(ik,ib)
         enddo
       enddo
      endif


!     find q-points q = k - kfirst for info
      call checkq(p%nkbz,kbz,p%nop,op,p%nq,p%q,b1,b2,b3,tpula)
      call pclock(60)

!     calculate LDA wavefunctions in real space using FFT
      allocate(wfr(nr,p%nbnds,p%nkibz),stat=istat)
      if(istat.ne.0) stop 'wavefunctions(r) out of memory'
      wfr(:,:,:) = 0.0
      call fftwfn(p%npwwfn,p%nbnds,p%nkibz,nr,wfg,wfr,
     &            igfft(:,3,3,3),nfft1,nfft1a,nfft2,nfft3)
      if(.not. p%QZERO) deallocate(wfg)
      call pclock(70)

!     calculate electron density
      allocate(rho2(nr),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      call get_density(p%nbnds,symstr%nsym,p%ninv,fftmesh%nr,ucvol,
     &                 k,irottb,occ,wfr,rho2,omegaplasma)
      call pclock(80)

      if(p%LDA) then
!       calculate exchange-correlation kernel fxc lda
        allocate(fxc(p%npwmat,p%npwmat),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        call ckxclda(nfft1,nfft1a,nfft2,nfft3,
     &               nr,rho2,fxc,p%npwmat,gvec)
        print *, 'fxc_LDA(G,G")'
        call printcm(fxc,p%npwmat,p%npwmat)
      endif

!     calculates qplusG(G) = |q+G| table for coulombian potential
      allocate(qplusG(p%npwmat,p%nq),stat=istat)
      if(istat.ne.0) stop 'out of memory, qplus'
      allocate(Vc(p%npwmat,p%nq),stat=istat)
      if(istat.ne.0) stop 'out of memory, qplus'
      call cVc(p%nq,p%q,b1,b2,b3,p%npwmat,gvec,qplusG,Vc)

      if(p%FXCMT) allocate(Tkernel(p%npwmat,p%npwmat,p%nomega))
      if(p%FXCMT .and. p%nq/=1) stop 'only one q'
      
      if(p%NLDA) then
!       calculate fxcNLDA(q,G,G') and put it into fxcnlda
        allocate(fxcnlda(p%npwmat,p%npwmat,p%nq),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        do iq = 1, p%nq
        call ckxcnlda(p%kxct,p%npwmat,gvec,
     &                nfft1,nfft1a,nfft2,nfft3,
     &                nr,rho2,qplusG(:,iq),
     &                fxcnlda(:,:,iq))
        enddo
!        print *, 'fxc_NLDA(q,G,G")'
!        call printcm(fxcnlda,p%npwmat,p%npwmat)
        stop 'NLDA not implemented'
      endif ! p%NLDA
      print *, 'end of initialization'
      print *
      call pclock(90)

      allocate(chi0(p%npwmat,p%npwmat,p%nomega),stat=istat)
      if(istat.ne.0) stop 'out of memory, chi0'
      allocate(lwing(p%npwmat,p%nomega,p%nq),stat=istat)
      if(istat.ne.0) stop 'out of memory, lwing'
      allocate(uwing(p%npwmat,p%nomega,p%nq),stat=istat)
      if(istat.ne.0) stop 'out of memory, uwing'


!     calculates chi0
      if(p%QZERO) then
!       different treatment of the case q=0
        if(inclvkb) then
          kb%natom = natom
          kb%ntype = ntype
          kb%mpsang = mpsang
          kb%ng = p%npwwfn
          kb%nk = p%nkibz
          call cfnl(p%npwwfn,p%nkibz,a1,a2,a3,
     &              b1,b2,b3,gvec,kibz,ntype,
     &        natom,mpsang,type,xcart,vkbsign,vkb,vkbd,kb)
!          allocate(gradvnl(3,p%npwwfn,p%npwwfn,p%nkibz),stat=istat)
!          call ccgradvnl(p%npwwfn,p%nkibz,b1,b2,b3,gvec,kibz,ntype,
!     &                 natom,mpsang,type,xcart,vkbsign,vkb,vkbd,gradvnl)
          deallocate(vkbsign,vkb,vkbd)
          call pclock(95)
        endif
        call cchi0q0(p,gvec,
     &      op,kibz,
     &      nbv,occ,ktab,ktabr,ktabp,ktabi,ktabo,kbz,
     &      irottb,nfft1,nfft1a,nfft2,nfft3,igfft,nr,fftmesh,
     &      energy,gwenergy,wfg,wfr,chi0,lwing,uwing,
     &      b1,b2,b3,ucvol,wtk,grottb,
     &      inclvkb,kb,Tkernel)
        deallocate(wfg)
      else
        call cchi0(p,p%q(:,1),
     &      nbv,occ,ktab,ktabr,ktabp,ktabi,kbz,
     &      nfft1,nfft1a,nfft2,nfft3,igfft,nr,fftmesh,
     &      energy,gwenergy,wfr,chi0,ucvol)
      endif

      print *, 'chi0(G,G") at the first omega'
      call printcm(chi0(:,:,1),p%npwmat,p%npwmat)
      call pclock(100)
      
      deallocate(wfr)
      deallocate(energy,gwenergy,occ,ktabr,rho2,irottb,grottb)

      allocate(epsmlf(p%nomega,p%nq+2),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(epsmnlf(p%nomega,p%nq+2),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      
      if(p%NOFXC) then
        allocate(epsilon(p%npwmat,p%npwmat),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        allocate(epsilonm1_g(p%npwmat))
        allocate(deltav(p%npwmat))
      else 
        allocate(chi_g(p%npwmat,p%nomega),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        allocate(mtmp(p%npwmat,p%npwmat),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        allocate(kernel(p%npwmat,p%npwmat))
        if(istat.ne.0) stop 'out of memory'
      endif
      if(p%FXCMT) then
        allocate(mtmpd(p%npwmat,p%npwmat),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        allocate(chi2d(p%npwmat,p%npwmat),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        allocate(fxcw(p%npwmat,p%npwmat,p%nomega),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        allocate(chi_gd(p%npwmat,p%nomega),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        allocate(chi3(p%npwmat),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        allocate(chi(p%nomega),stat=istat)
        if(istat.ne.0) stop 'out of memory'
      endif
      
      
      if(p%FXCEPS .or. p%FXCEPSDIAGO) then
        print *, 'calculate screening to be used in the kernel'
        allocate(epsm1q(p%npwmat,p%npwmat),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        iq = 1 ! using the first q
        io = 1 ! using the first omega (omega = 0?)
        chi0(:,1,:) = lwing(:,:,iq)
        chi0(1,:,:) = uwing(:,:,iq)
!       epsilon = 1 - Vc * chi0
        do ig2 = 1, p%npwmat
          epsm1q(:,ig2) = - Vc(:,iq)*chi0(:,ig2,io)
          epsm1q(ig2,ig2) = 1 + epsm1q(ig2,ig2)
        enddo
!       invert epsilon to epsilon^-1
        call cginv(p%npwmat,epsm1q(:,:),p%npwmat)
        print *, 'screening calculated'
      endif 

      do iq = 1, p%nq
        if(p%QZERO) then
          print *, 'q-point number ', iq
          print *
          chi0(:,1,:) = lwing(:,:,iq)
          chi0(1,:,:) = uwing(:,:,iq)
        endif

        if(p%NOFXC) then ! RPA-like calculation

          print *, 'calculating epsilon = 1 - Vc chi0'
          print *, 'solving epsilon * epsilon^-1 = Delta for epsilon^1'
          print *, '(only one column)'
          deltav(:) = (0.0,0.0)
          deltav(p%ig) = (1.0,0.0)
!         calculate epsilon = 1 - Vc chi0
          do io = 1, p%nomega
            do ig2 = 1, p%npwmat
              epsilon(:,ig2) = - Vc(:,iq)*chi0(:,ig2,io)
              epsilon(ig2,ig2) = 1 + epsilon(ig2,ig2)
            enddo
!           calculate macroscopic epsilon without local fields
            epsmnlf(io,iq) = epsilon(p%ig,p%ig)
!           invert epsilon for only one column
            call cgle(p%npwmat,epsilon,p%npwmat,deltav,epsilonm1_g)
!           calculate macroscopic epsilon with local fields
            epsmlf(io,iq) = 1.0 / epsilonm1_g(p%ig)
          enddo

        else if(p%LDA) then

          print *, 'calculating chi = (1 - chi0*Vc - chi0*fxc)^-1 chi0'
          print *, 'calculating (1 - chi0*Vc - chi0*fxc)'
          print *, 'solving (1 - chi0*Vc - chi0*fxc)*chi=chi0 for chi'
          print *, '(only one column)'
          do io = 1, p%nomega
!           calculating - chi0 * fxc
            mtmp(:,:) = - matmul(chi0(:,:,io),fxc)
!           calculating 1 - chi0*Vc - chi0*fxc
            do ig1 = 1, p%npwmat
              mtmp(ig1,:) = mtmp(ig1,:) - Vc(:,iq) * chi0(ig1,:,io)
              mtmp(ig1,ig1) = 1.0 + mtmp(ig1,ig1)
            enddo
!           solving (1 - chi0*Vc - chi0*fxc)*chi=chi0 for chi
           call cgle(p%npwmat,mtmp,p%npwmat,chi0(:,p%ig,io),chi_g(:,io))
          enddo
!         calculate epsilon^-1
          if(p%TESTELECTRON) then
            print *, 'calculating epsilon^-1(G,G") = 1 + (Vc+fxc)*chi'
            do io=1,p%nomega
              fxcchi=cdotu(p%npwmat,fxc(p%ig,:),1,chi_g(:,io),1)
              epsilonm1_gg = 1.0 + Vc(p%ig,iq)*chi_g(p%ig,io)+fxcchi
              epsmlf(io,iq) = 1.0 / epsilonm1_gg
            enddo
            epsmnlf(:,iq) = epsmlf(:,iq)
          else ! p%TESTPARTICLE
            print *, 'calculating epsilon^-1(G,G") = 1 + Vc*chi'
            epsmlf(:,iq) = 1.0 / ( 1 + Vc(p%ig,iq)*chi_g(p%ig,:) )
            epsmnlf(:,iq) = epsmlf(:,iq)
          endif
        
       else if(p%FXCMT) then

          if(p%kernel) then
           print *,'calculating'
            print *, 'K(g,g'',w) -->  fxc(g,g'',w)'
            print *,'chi=(1-chi0*(v+fxc))^-1*chi0'
            
            do io=1,p%nomega
              mtmpd(:,:)=chi0(:,:,io)
              call zginv(p%npwmat,mtmpd,p%npwmat)
              chi2d(:,:)=matmul(Tkernel(:,:,io),mtmpd)
              fxcw(:,:,io)=-matmul(mtmpd,chi2d)
            enddo
            print *,'fxc obtained'
            open(56,file='out.fxc',status='unknown')
            do io=1,p%nomega
              omegaev=real(p%omega(io))*HARTREE
             write(56,*)omegaev,real(fxcw(1,1,io)),aimag(fxcw(1,1,io))
            enddo
            close(56)
            do io=1,p%nomega
              mtmpd(:,:)=chi0(:,:,io)
              chi2d(:,:)=fxcw(:,:,io)
              do ig1=1,p%npwmat
                chi2d(ig1,ig1)= Vc(ig1,iq) + chi2d(ig1,ig1)
              enddo
              chi2d(:,:) = - matmul(mtmpd,chi2d)
              do ig1 = 1, p%npwmat
                chi2d(ig1,ig1) = 1.d0 + chi2d(ig1,ig1)
              enddo
!              call zginv(p%npwmat,chi2d,p%npwmat)
!              kernelomega(:,:,io)=matmul(chi2d,mtmpd)
!             solving (1 - chi0*Vc - chi0*fxc) * chi = chi0  for chi
              call zgle(p%npwmat,chi2d,p%npwmat,mtmpd(:,p%ig),
     &                  chi_gd(:,io))
            enddo
            print *,'chi obtained'
!           calculate epsilon-twiddle^-1 = 1 + Vc chi
!           calculate macroscopic epsilon
            epsmlf(:,iq)= 1.0 / ( 1 + Vc(p%ig,iq)*chi_gd(p%ig,:) )
!            epsmlf(:,iq)= 1.0/(1+Vc(p%ig,iq)*kernelomega(p%ig,p%ig,:))
            epsmnlf(:,iq)=epsmlf(:,iq)
            print *,'epsilon^-1 obtained'

          else
            print *,'calculating '
            print *,'chi=chi0*(chi0-chi0*v*chi0 +K(g,g'',w))^-1*chi0'
            print *,'with K(g,g'',w) = chi0(g,g'',w)*W*chi0(g,g'',w)' 
            chi(:)=(0.0,0.0)
            do io=1,p%nomega
!            pay attention:this is chi0*Vc not the contrary as it could seem 
              do ig2=1,p%npwmat
               do ig1=1,p%npwmat
                mtmpd(ig1,ig2) = Vc(ig2,iq) * chi0(ig1,ig2,io)
               enddo
              enddo
              mtmpd(:,:) = matmul(mtmpd,chi0(:,:,io))
              mtmpd(:,:) = chi0(:,:,io)-mtmpd(:,:)+Tkernel(:,:,io)
              chi3(:) = chi0(:,p%ig,io)
              call zgle(p%npwmat,mtmpd,p%npwmat,chi3,chi3)
              do ig=1,p%npwmat
                chi(io) = chi(io) + chi0(p%ig,ig,io) * chi3(ig)
              enddo
            enddo
            print *, 'creato chi'
!           calculate epsilon-twiddle^-1 = 1 + Vc chi
!           calculate macroscopic epsilon
             
            epsmlf(:,iq)= 1.0 / ( 1 + Vc(p%ig,iq)*chi(:) )
            epsmnlf(:,iq)=epsmlf(:,iq)
            print *, 'creata epsilon'
          endif
          
       else ! experimental fxc

          kernel(:,:) = (0.0,0.0)     
!         calculate fxc
          if (p%FXCALPHA) then
            print *,'chi=(1 - chi0(alpha/q^2 + Vc))^-1 chi0'
!           do ig1=1,p%npwmat
!             kernel(ig1,ig1)= p%alphadiv / (4*PI) * Vc(ig1,iq)
!           enddo
!           the next gives the same result as the previous
            kernel(1,1)= p%alphadiv / (4*PI) * Vc(1,iq)
          else if (p%FXCEPS) then
            print *,'chi=(1 - chi0(eps^-1_gg''*Vc + Vc))^-1 chi0'
            do ig2=1,p%npwmat
              kernel(:,ig2)=-epsm1q(:,ig2)*Vc(ig2,iq)
            enddo
          else if (p%FXCEPSDIAGO) then
            print *,'chi=(1 - chi0(eps^-1_gg*Vc + Vc))^-1 chi0'
            do ig1=1,p%npwmat
              kernel(ig1,ig1)=-epsm1q(ig1,ig1)*Vc(ig1,iq)
            enddo
          else if (p%FXCVBAR) then
            print *,'chi=(1 - chi0(-vbar + Vc))^-1 chi0'
            do ig1=2,p%npwmat
              kernel(ig1,ig1)=-Vc(ig1,iq)
            enddo
          else if (p%CUTOFF) then
            kernel(:,:)=(0.0,0.0)
            do ig1=1,p%npwmat
              kernel(ig1,ig1)=-Vc(ig1,iq)*(1.d0-cos(qplusG(ig1,iq)
     &                         *p%rcut))
            enddo
          endif

!         now (v + fxc)
          do ig1=1,p%npwmat
            kernel(ig1,ig1)= Vc(ig1,iq) + kernel(ig1,ig1)
          enddo
!         now chi = (1-chi0(fxc + v))^-1 chi0
!         calculating (1 - chi0*Vc - chi0*fxc))
          do io = 1, p%nomega
            mtmp(:,:) = - matmul(chi0(:,:,io),kernel)
            do ig1 = 1, p%npwmat
              mtmp(ig1,ig1) = 1.0 + mtmp(ig1,ig1)
            enddo
!           solving (1 - chi0*Vc - chi0*fxc) * chi = chi0  for chi
           call cgle(p%npwmat,mtmp,p%npwmat,chi0(:,p%ig,io),chi_g(:,io))
          enddo
!         calculate epsilon-twiddle^-1 = 1 + Vc chi
!         calculate macroscopic epsilon
          epsmlf(:,iq)= 1.0 / ( 1 + Vc(p%ig,iq)*chi_g(p%ig,:) )
          epsmnlf(:,iq)=epsmlf(:,iq)

        endif

      
      call pclock(100+iq*10)
      enddo ! iq


      dsffact = vlngth(p%qpg,b1,b2,b3)**2 / 
     &          (4 * PI**2 * p%nel/ucvol)

!     write out dielectric macroscopic functions (with and without
!     local fields), refraction functions and dynamic structure factors
      call wreps(p,tpula,epsmlf,epsmnlf,dsffact)


      print *, 'check Kramers Kronig on Macroscopic Epsilon NLF'
      call check_kramerskronig(p%nomega,real(p%omega(:)),epsmnlf(:,1))

      print *, 'check Kramers Kronig on Macroscopic Epsilon LF'
      call check_kramerskronig(p%nomega,real(p%omega(:)),epsmlf(:,1))

      print *, 'check f-sum rule on Macroscopic Epsilon NLF'
      call check_fsumrule(
     &  p%nomega,real(p%omega(:)),aimag(epsmnlf(:,1)),omegaplasma)

      print *, 'check f-sum rule on Macroscopic Epsilon LF'
      print *, 'warning: f-sum rule should be checked without LF'
      call check_fsumrule(
     &  p%nomega,real(p%omega(:)),aimag(epsmlf(:,1)),omegaplasma)

      deallocate(epsmlf,epsmnlf)

      call pclock(200)


      dp = 1
      print *, 'dp ended'
      end
