subroutine get_density(nb,nsym,ninv,nr,ucvol,k,rrottab,occ,wfr,rho,omegaplasma)
  use ggwdef
  implicit none
  integer nb, nsym, nr, ninv
  type(bzstructure) k
  integer rrottab(nr,nsym)
  real ucvol, occ(k%nibz,nb)
  complex wfr(nr,nb,k%nibz)
  real rho(nr)
  real omegaplasma
  real tnepuc, rhoav, rs
  real rhot(nr)
  integer ikibz, ib, ir, isym
  real fact

  print *, 'calculating charge density'

  rhot(:) = 0
  do ikibz = 1, k%nibz
    do ib = 1, nb
      if (occ(ikibz,ib)< 1.e-5) exit
      rhot(:) = rhot(:) + occ(ikibz,ib) * k%ibz(ikibz)%w * &
        &       conjg(wfr(:,ib,ikibz)) * wfr(:,ib,ikibz)
    enddo
  enddo

! loop over symmetry operations, symmetrising rho
  rho(:) = 0
  do isym = 1, nsym
    do ir = 1, nr
      rho(ir) =  rho(ir) + rhot(rrottab(ir,isym))
    enddo
  enddo

  if(ninv == 1) then
    fact = 1.0 / (k%nbz * ucvol)
  else
    ! factor of 2 is for time reversal if present
    fact = 2.0 / (k%nbz * ucvol)
  endif
  rho(:) = fact * rho(:)

! calculate total number of electrons as a check
  tnepuc = 0
  do ir = 1, nr
    tnepuc = tnepuc + rho(ir)
  enddo
  tnepuc = tnepuc * ucvol / nr
  write(*,'(" total number of electrons per unit cell = ",f7.4)') tnepuc
  rhoav = tnepuc / ucvol
  write(*,'(" average of density, n = ",f7.4)') rhoav
  rs = (3.0/(4.0*(4.0*atan(1.0))*rhoav))**(1.0/3.0)
  write(*,'(" r_s = ",f7.4)') rs
  omegaplasma = sqrt(4.0 * (4.0*atan(1.0)) * rhoav)
  write(*,'(" omega_plasma = ",f7.4, "[eV]")') omegaplasma * H2eV
  print *
  return
end


subroutine get_rhotwg(ng,fftmesh,symstr,rrottab,igfftg0,&
&                     is1,inv1,kibz1,wfrkibz1,is2,inv2,kibz2,wfrkibz2,rhotwg)
  use ggwdef
  implicit none
  type(mesh) fftmesh
  type(symmetry) symstr
  integer ng
  integer rrottab(fftmesh%nr,symstr%nsym)
  integer igfftg0(ng)
  integer is1, is2, inv1, inv2
  real kibz1(3), kibz2(3)
  complex wfrkibz1(fftmesh%nr), wfrkibz2(fftmesh%nr)
  complex rhotwg(ng), rhotwr(fftmesh%nr)
  complex wfrkbz1(fftmesh%nr), wfrkbz2(fftmesh%nr)

  call get_wfrkbz(fftmesh%nr,symstr,is1,inv1,kibz1,wfrkibz1,rrottab(:,is1),wfrkbz1)
  call get_wfrkbz(fftmesh%nr,symstr,is2,inv2,kibz2,wfrkibz2,rrottab(:,is2),wfrkbz2)
  rhotwr(:) = conjg(wfrkbz1(:)) * wfrkbz2(:)
! FFT to rho-twiddle (G)
  call c3dfft(rhotwr,fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3),&
&             fftmesh%nrr(1),fftmesh%nrr(2),fftmesh%nrr(3),-1)
!     Set up rho-twiddle(G-G0) for matrix in PW order, normalising for FFT
  rhotwg(:) = rhotwr(igfftg0(:)) / fftmesh%nr
  return
end


subroutine get_wfrkbz(nr,symstr,is,inv,kibz,wfrkibz,irp,wfrkbz)
  use ggwdef
  implicit none
  integer nr, is, inv, ir
  type(symmetry) symstr
  real kibz(3)
  complex wfrkibz(nr), wfrkbz(nr)
  integer irp(nr)
  double precision :: phasefact

  phasefact = exp( (0,1)*2*PI * dot_product(kibz(:),symstr%isym(is)%t(:)) )
  do ir = 1, nr
    wfrkbz(ir) = phasefact * wfrkibz(irp(ir))
  enddo
  if(inv /= 1) wfrkbz(:) = conjg(wfrkbz(:))
  return
end


