! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

module ggwdef
implicit none

type crystal
  integer :: spgroup, pgroup                 ! space, point group
  integer :: bravaislatt, crystsys           ! bravais lattice, crystal system
  double precision :: rprim(3,3)             ! dimensionless
  double precision :: gprim(3,3)             ! dimensionless
  double precision :: acell(3)               ! cell dimension in Bohr
  double precision :: angdeg(3)              ! angle degrees among rprim
  double precision :: rprimd(3,3)            ! in Bohr
  double precision :: gprimd(3,3)            ! a.u. dimensioned primitives
  double precision :: ucvol, bzvol           ! unit cell, brillouin zone volume
  double precision :: gprimdm1(3,3)          ! transf from cc to rl
end type

type symmetry_operation                      ! single symmetry operation
  integer :: r(3,3)                          ! rotation
  double precision :: t(3)                   ! translation
  integer :: tr                              ! time reversal (-1 = time rev)
end type

type symmetry                                ! symmetry structure of the system
  integer :: nsym                            ! maximum symmetries 96
  type(symmetry_operation) :: sym(96)        ! direct symmetries
  type(symmetry_operation) :: isym(96)       ! inverse symmetries
  type(symmetry_operation) :: itsym(96)      ! inverse transpose symmetries for k-points
end type

type pseudoatom                              ! single pseudoatom
  double precision :: znucl                  ! atom Z
  double precision :: zion                   ! pseudoatom Z
  integer :: lmax                            ! maximum angular momentum compon.
  integer :: lloc                            ! local angular momentum compon.
  integer :: mmax                            ! mesh dimension
  integer :: pspso,pspdat,pspcod,pspxc       ! spin orbit ....
  character :: title(132)
end type

type chemistry                               ! chemistry structure of the system
  integer :: ntype                           ! number of atom types
  double precision, pointer :: znucl(:)      ! atoms Z (redundant)
  integer :: natom                           ! number of atoms
  integer, pointer :: type(:)                ! type of the atoms
  double precision, pointer :: xred(:,:)     ! atom reduced coordinates
  integer :: npsa                            ! number of pseudoatoms ( = ntype)
  type(pseudoatom), pointer :: psa(:)        ! pseudoatoms
  integer :: nel                             ! number of electrons in the unit cell
end type

type mesh
  integer ng                                 ! number of G's (of planewaves)
  integer nr                                 ! number of r's (real space mesh)
  integer ngg(3)                             ! reciprocal space mesh
  integer nrr(3)                             ! real space mesh
end type

type transposed_space                        ! transposed space structure
  integer ng                                 ! number of G's (redundant)
  integer nsh                                ! number of shells (to add: number of stars)
  integer nk                                 ! number of k-points in IBZ
  integer, pointer :: g(:,:)                 ! G-vectors
  integer, pointer :: sh(:)                  ! shells limits
  double precision, pointer :: k(:,:)        ! k-points in IBZ (may be better a pointer to ibzvector object)
end type

type ibzvector                               ! IBZ vector
  double precision :: rc(3)                  ! reduced coordinates
  double precision :: w                      ! weight
end type

type bzvector                                ! BZ vector
  double precision :: rc(3)                  ! reduced coordinates
  integer :: ibz                             ! corresponding IBZ-vector (better a pointer to ibzvector object)
  integer :: isym                            ! IBZ -> BZ originating symmetry (better a pointer to symmetry_operation object)
end type

type bzstructure                             ! BZ structure of the system
  integer nibz                               ! number of IBZ k-points
  integer nbz                                ! number of BZ k-points
  type(ibzvector), pointer :: ibz(:)         ! array of IBZ k-points
  type(bzvector), pointer :: bz(:)           ! array of BZ k-points
end type

type electronic_structure                    ! electronic structure of the system
  integer nb                                 ! number of bands
  integer nk                                 ! number of k-points (IBZ)
  integer ng                                 ! number of planewaves (of G's)
  integer nr                                 ! number of r-points
  double precision, pointer :: oc(:,:)            ! occupations oc(n,k) Bloch order
  double precision, pointer :: en(:,:)            ! energies en(n,k) Bloch order
  complex (kind(0.0d0)), pointer :: qps(:,:)    ! qps(n,k) QP shift
  complex (kind(0.0d0)), pointer :: wfg(:,:,:)    ! wavefunctions in rec. space wfg(g,n,k) order
  complex (kind(0.0d0)), pointer :: wfr(:,:,:)    ! wavefunctions in real space wfr(r,n,k) order
  double precision :: fermie                      ! Fermi energy
  double precision :: etotal, residm, ecut_eff    ! 
  integer :: xc                                   ! xc functional
end type

type kb_potential
  integer :: ntype
  integer :: natom
  integer :: mpsang
  integer :: ng
  integer :: nk
  integer, pointer :: vkbsign(:,:)
  double precision, pointer :: vkb(:,:,:,:), vkbd(:,:,:,:)
  complex, pointer :: gradvnl(:,:,:,:)
  complex, pointer :: fnl(:,:,:,:), fnld(:,:,:,:,:)
end type        


!      type rlv
!        integer rlu(3)                          ! gvec
!        integer jgfft                           ! igfft(G0=0)
!        integer igfft(-3:3,-3:3,-3:3)           ! igfft
!        type(rlv), pointer :: grottb(:,:)       ! grottb(ninv,nop)
!      endtype

!      type ibzpoint
!        real rlu(3)                             ! kibz, qibz
!        real w                                  ! weight
!        real, pointer :: en(:)                  ! energy
!      endtype
      
!      type bzpoint
!        real rlu(3)                             ! kbz, qbz
!        type(ibzpoint), pointer :: ibz          ! ktab, qtab
!        type(symmop), pointer :: o              ! ktabo, qtabo
!        logical :: i                            ! ktabi, qtabi
!      endtype

!      type dlv
!        integer, pointer :: irottb(:)           ! irottb(nop)
!      endtype
!     ktabr(nr,nkbz) -> ktabr(:,kbz) = irottb(:,iop)
      

      type lattstruct                           ! deprecated: use crystal
        integer lattype
        real acell(3)                           ! cell dimension in Bohr
        real rprim(3,3)                         ! dimensionless
        real r(3,3)                             ! in Bohr
        real a1(3), a2(3), a3(3)
        real b1(3), b2(3), b3(3)
        real ucvol, bzvol
        real brl2cc(3,3)
        real bcc2rl(3,3)
      endtype
      
      type transition
!       temporary an integer but bzpoint in the future
        integer k
        integer v
        integer c
        complex en
        real factocc
      endtype

      type transition2
!       temporary an integer but bzpoint in the future
        integer k
        integer n
        integer m
        integer l
        real en_nm
        real en_nl
        real en_lm
        real factocc_nl
        real factocc_ml
      endtype

      type transgen
       complex, pointer :: rhotwgen(:,:)
       complex, pointer :: rhotwxgen(:,:)
       real, pointer :: qrotgen(:,:)
       real, pointer :: opinvgen(:,:,:)
       complex, pointer :: aux(:,:)
       complex, pointer :: aux2(:,:)
      endtype

      type response
        complex, pointer :: chi0(:,:,:)
        complex, pointer :: lwing(:,:,:)
        complex, pointer :: uwing(:,:,:)
        complex, pointer :: epsmlf(:,:)
        complex, pointer :: epsmnlf(:,:)
        complex, pointer :: chi0X(:,:,:)
      endtype

      type momentum
        real qplusg(3)
        real qpplusg(3)
        real qplusqpplusg(3)
        real qcc(3)
        real qpcc(3)
        real q(3)
        integer gq(3)
        integer iqg
        real qp(3)
        integer gqp(3)
        integer iqpg
        real qplusqp(3)
        integer gqplusqp(3)
        integer iqplusqpg
      endtype

double precision, parameter :: PI = 3.14159265358979323846
double precision, parameter :: zero = 0.0d0
double precision, parameter :: H2eV = 27.2113845d0 ! standard uncertainty 0.0000023 (NIST)
double precision, parameter :: eV2H = 1.0d0 / H2eV
real, parameter :: HARTREE = 27.2116

end module
