! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

!     ==================================================================
!     interface to the CXML library
!     ==================================================================

!     ==================================================================
      integer function allowed_fftsize(n)
!     ==================================================================
!     calculate the smallest allowed FFT size that encompass n
!     in CXML all the values are allowed
      implicit none
      integer n
      allowed_fftsize = n
      return
      end

!     ==================================================================
      subroutine nfft(m,n)
!     ==================================================================
!     calculate N, the size of the smallest allowed FFT that will encompass
!     the 2M+1 points -M..+M.  (Ideally, N=2M+1)
!     ------------------------------------------------------------------
      implicit none
      integer n, m
      integer, external :: allowed_fftsize
      n = 2 * allowed_fftsize(m) + 1
      return
      end

!     ==================================================================
      subroutine n1dfft(n1,n1d)
!     ==================================================================
      implicit none
      integer n1, n1d
      n1d = n1
      return
      end      


!     ==================================================================
      subroutine c3dfft(c,id,ie,if,nl,nm,nn,isig)
!     ==================================================================
!     == computes the 3d fast fourier transform of a complex function ==
!     ==             isig <0  inverse                                 ==
!     ==             isig >0  direct                                  ==
!     ==--------------------------------------------------------------==
      implicit none
      include 'cxmldef.for'
      integer id, ie, if
      integer nl, nm, nn
      integer isig
      complex c(id,ie,if)

      integer status
      character*1 csig
!     real scale
      if(isig < 0) then
        csig = 'b'
      else
        csig = 'f'
      endif
      
      status = cfft_3d('C','C',csig,c,c,nl,nm,nn,id,ie,1,1,1)
      
      if(isig < 0) c(:,:,:) = c(:,:,:) * nl*nm*nn ! the cxml rescale automatical
              

      return
      end

