! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

!     ==================================================================
!     Interface to the ESSL library
!     ==================================================================

!     ==================================================================
!     Eigensystem analysis
!     ==================================================================


!     ==================================================================
      subroutine chevc(n,a,lda,w,z,ldz)
!     ==================================================================
!     Compute the eigenvalues and the eigenvectors
!     of a complex Hermitian matrix
!     only the upper triangle of a is used
!     ------------------------------------------------------------------
      
      integer n, lda, ldz
      complex a(lda,n)
      complex z(ldz,n)
      real w(n)
      
      integer iopt
!     upper-packed storage mode and eigenvectors (iopt=21)
!     lower-packed storage mode and eigenvectors (iopt=1)
      parameter(iopt=21)
      
      integer np
      complex, allocatable :: ap(:)
      integer naux
      real, allocatable :: aux(:)

      integer istat
      
      np = n*(n+1)/2
      allocate(ap(np),stat=istat)
      if(istat.ne.0) then
        print *, 'error: out of memory'
        print *, 'memory-space requested', np,' sizeof(complex)'
        stop 'chevc 1: out of memory!'
      endif
      
      naux = 4*n
      allocate(aux(naux),stat=istat)
      if(istat.ne.0) stop 'chevc 2: out of memory!'

!     upper-packed storage mode    
      do i = 1, n
        do j = i, n
          ap(i+j*(j-1)/2) = a(i,j)
        enddo
      enddo

!     lower-packed storage mode    
!     do i = 1, n
!       do j = 1, i
!         ap(i+(2*n-j)*(j-1)/2) = a(i,j)
!       enddo
!     enddo
     
      call chpev(iopt,ap,w,z,ldz,n,aux,naux)
      
      deallocate(ap)
      deallocate(aux)
      return
      end


!     ==================================================================
      subroutine zhevc(n,a,lda,w,z,ldz)
!     ==================================================================
!     Compute the eigenvalues and the eigenvectors
!     of a complex double precision Hermitian matrix
!     only the upper triangle of a is used
!     ------------------------------------------------------------------
      
      integer n, lda, ldz
      complex (kind(0.0d0)) a(lda,n)
      complex (kind(0.0d0)) z(ldz,n)
      real (kind(0.0d0)) w(n)
      
      integer iopt
!     upper-packed storage mode and eigenvectors (iopt=21)
!     lower-packed storage mode and eigenvectors (iopt=1)
      parameter(iopt=21)
      
      integer np
      complex (kind(0.0d0)), allocatable :: ap(:)
      integer naux
      real (kind(0.0d0)), allocatable :: aux(:)

      integer istat
      
      np = n*(n+1)/2
      allocate(ap(np),stat=istat)
      if(istat.ne.0) stop 'zhevc 1: out of memory!'
      
      naux = 4*n
      allocate(aux(naux),stat=istat)
      if(istat.ne.0) stop 'zhevc 2: out of memory!'

!     upper-packed storage mode    
      do i = 1, n
        do j = i, n
          ap(i+j*(j-1)/2) = a(i,j)
        enddo
      enddo

!     lower-packed storage mode    
!     do i = 1, n
!       do j = 1, i
!         ap(i+(2*n-j)*(j-1)/2) = a(i,j)
!       enddo
!     enddo
     
      call zhpev(iopt,ap,w,z,ldz,n,aux,naux)
      
      deallocate(ap)
      deallocate(aux)
      return
      end


!     ==================================================================
      subroutine chevcp(n,m,a,lda,small,w,z,ldz)
!     ==================================================================
!     compute the largest or smallest eigenvalues and the
!     corresponding eigenvectors of a complex Hermitian matrix
!     
!     small = .true. the lowest eigenvalues are computed
!     only the upper triangle of a is used
!     ------------------------------------------------------------------
      
      integer n, m, lda, ldz
      logical small
      complex a(lda,n)
      complex z(ldz,m)
      real w(n)
      
      integer iopt
!     upper-packed storage mode and eigenvectors (iopt=21, 31)
!     lower-packed storage mode and eigenvectors (iopt=1, 11)
      
      integer np
      complex, allocatable :: ap(:)
      integer naux
      real, allocatable :: aux(:)

      integer istat

!     upper-packed storage mode and eigenvectors    
      if(small) then
        iopt = 21
      else
        iopt = 31
      endif
      
      np = n*(n+1)/2
      allocate(ap(np),stat=istat)
      if(istat.ne.0) stop 'chevcp 1: out of memory!'
      
      naux = 11*n
      allocate(aux(naux),stat=istat)
      if(istat.ne.0) stop 'chevcp 2: out of memory!'

!     upper-packed storage mode    
      do i = 1, n
        do j = i, n
          ap(i+j*(j-1)/2) = a(i,j)
        enddo
      enddo

!     lower-packed storage mode    
!     do i = 1, n
!       do j = 1, i
!         ap(i+(2*n-j)*(j-1)/2) = a(i,j)
!       enddo
!     enddo
     
      call chpsv(iopt,ap,w,z,ldz,n,m,aux,naux)
      
      deallocate(ap)
      deallocate(aux)
      return
      end


!     ==================================================================
      subroutine zhevcp(n,m,a,lda,small,w,z,ldz)
!     ==================================================================
!     compute the largest or smallest eigenvalues and the
!     corresponding eigenvectors of a complex double prec. Hermitian matrix
!     
!     small = .true. the lowest eigenvalues are computed
!     only the upper triangle of a is used
!     ------------------------------------------------------------------
      
      integer n, m, lda, ldz
      logical small
      complex (kind(0.0d0)) a(lda,n)
      complex (kind(0.0d0)) z(ldz,m)
      real (kind(0.0d0)) w(n)
      
      integer iopt
!     upper-packed storage mode and eigenvectors (iopt=21, 31)
!     lower-packed storage mode and eigenvectors (iopt=1, 11)
      
      integer np
      complex (kind(0.0d0)), allocatable :: ap(:)
      integer naux
      real (kind(0.0d0)), allocatable :: aux(:)

      integer istat

!     upper-packed storage mode and eigenvectors    
      if(small) then
        iopt = 21
      else
        iopt = 31
      endif
      
      np = n*(n+1)/2
      allocate(ap(np),stat=istat)
      if(istat.ne.0) stop 'zhevcp 1: out of memory!'
      
      naux = 11*n
      allocate(aux(naux),stat=istat)
      if(istat.ne.0) stop 'zhevcp 2: out of memory!'

!     upper-packed storage mode    
      do i = 1, n
        do j = i, n
          ap(i+j*(j-1)/2) = a(i,j)
        enddo
      enddo

!     lower-packed storage mode    
!     do i = 1, n
!       do j = 1, i
!         ap(i+(2*n-j)*(j-1)/2) = a(i,j)
!       enddo
!     enddo
     
      call zhpsv(iopt,ap,w,z,ldz,n,m,aux,naux)
      
      deallocate(ap)
      deallocate(aux)
      return
      end


!     ==================================================================
      subroutine cgevc(n,a,lda,w,z,ldz)
!     ==================================================================
!     Compute the eigenvalues and the (right) eigenvectors
!     of a complex general matrix
!     a and z must have no common elements
!     ------------------------------------------------------------------
      
      integer n, lda, ldz
      complex a(lda,n)
      complex z(ldz,n)
      complex w(n)
      
      integer, parameter :: iopt = 1
      logical select(n)
      integer naux
      real, allocatable :: aux(:)

      integer istat
            
      naux = 3*n
      allocate(aux(naux),stat=istat)
      if(istat.ne.0) stop 'cgevc 1: out of memory!'

      call cgeev(iopt,a,lda,w,z,ldz,select,n,aux,naux)
      
      deallocate(aux)
      return
      end


!     ==================================================================
      subroutine rginv(n,a,lda)
!     ==================================================================
!     invert a real general matrix
!     ------------------------------------------------------------------
      
      integer n, lda
      real a(lda,n)
      integer, parameter :: iopt = 0
      real rcond, det(2)
      integer naux
      real, allocatable :: aux(:)
      
      naux = 200*n
      allocate(aux(naux))
      call sgeicd(a,lda,n,iopt,rcond,det,aux,naux)
      deallocate(aux)
      return
      end

!     ==================================================================
      subroutine cginv(n,a,lda)
!     ==================================================================
!     invert a complex general matrix
!     ------------------------------------------------------------------

      integer n, lda
      complex  a(lda,n)

      integer ipiv(n)
      complex  work(n)
      integer info

      call cgetrf(n,n,a,lda,ipiv,info)
      if(info.ne.0) then
        print *, info
        stop 'cgetrf failed'
      endif
      call cgetri(n,a,n,ipiv,work,n,info)
      if(info.ne.0) then
        print *, info
        stop 'cgetri failed'
      endif

      return
      end



!     ==================================================================
      subroutine zginv(n,a,lda)
!     ==================================================================
!     invert a complex general matrix
!     ------------------------------------------------------------------

      integer n, lda
      complex (kind(0.0d0)) :: a(lda,n)

      integer ipiv(n)
      complex (kind(0.0d0)) :: work(n)
      integer info

      call zgetrf(n,n,a,lda,ipiv,info)
      if(info.ne.0) then
        print *, info
        stop 'cgetrf failed'
      endif
      call zgetri(n,a,n,ipiv,work,n,info)
      if(info.ne.0) then
        print *, info
        stop 'cgetri failed'
      endif

      return
      end

!     ==================================================================
      subroutine zgle(n,a,lda,b,x)
!     ==================================================================
!     solve the general complex problem A * X = B
!     where A is a complex matrix N x N and X, B are complex vectors
!     ------------------------------------------------------------------
      implicit none
      integer n, lda
      complex (kind(0.0d0)) a(lda,n), x(n), b(n)

      integer iopt
      integer ipvt(n)
      complex (kind(0.0d0)) bx(n)

      bx(:) = b(:)
      iopt = 0
      call zgef(a,lda,n,ipvt)
      call zges(a,lda,n,ipvt,bx,iopt)
      x(:) = bx(:)

      return
      end
      

!     ==================================================================
      subroutine cgle(n,a,lda,b,x)
!     ==================================================================
!     solve the general complex problem A * X = B
!     where A is a complex matrix N x N and X, B are complex vectors
!     ------------------------------------------------------------------
      implicit none
      integer n, lda
      complex a(lda,n), x(n), b(n)
      
      integer iopt
      integer ipvt(n)
      complex bx(n)
      
      bx(:) = b(:)
      iopt = 0
      call cgef(a,lda,n,ipvt)
      call cges(a,lda,n,ipvt,bx,iopt)
      x(:) = bx(:)
      
      return
      end



!     ==================================================================
!     Fourier transforms
!     ==================================================================

!     ==================================================================
      integer function allowed_fftsize(n)
!     ==================================================================
!     calculate the size of the smallest allowed FFT size that 
!     will encompass n
!     this routine is written for the ESSL routine scft, in which the
!     allowable sizes of FFT are those whose prime factors are
!       n <= 37748736
!       n = 2^h * 3^i * 5^j * 7^k * 11^m  h=1,...,25  i=0,1,2  j,k,m=0,1
!     ------------------------------------------------------------------
      implicit none
      integer n
      integer, parameter :: nn = 56
      integer nallwd(nn)
      data nallwd/
     &   2,   4,   6,   8,  10,  12,  14,  16,  18,  20,
     &  22,  24,  28,  30,  32,  36,  40,  42,  44,  48,
     &  56,  60,  64,  66,  70,  72,  80,  84,  88,  90,
     &  96, 110, 112, 120, 126, 128, 132, 140, 144, 154,
     & 160, 168, 176, 180, 192, 198, 210, 220, 224, 240,
     & 252, 256, 264, 280, 288, 308/
      integer i
      
      if(n > nallwd(nn) .or. n < 2) then
        print *, '**ERROR allowed_fftsize'
        stop 'error allowed fft_size'
      endif
      do i = nn, 1, -1
        if(nallwd(i) >= n) allowed_fftsize = nallwd(i)
      enddo
      return
      end


!     ==================================================================
      subroutine nfft(m,n)
!     ==================================================================
!     calculate N, the size of the smallest allowed FFT that will encompass
!     the 2M+1 points -M..+M.  (Ideally, N=2M+1, but this may not be
!     an allowable FFT.)
!     this routine is written for the ESSL routine scft, in which the
!     allowable sizes of FFT are those whose prime factors are
!       n <= 37748736
!       n = 2^h * 3^i * 5^j * 7^k * 11^m  h=1,...,25  i=0,1,2  j,k,m=0,1
!     ------------------------------------------------------------------
      implicit none
      integer n, m
      integer, parameter :: nn = 56
      integer nallwd(nn)
      data nallwd/
     &   2,   4,   6,   8,  10,  12,  14,  16,  18,  20,
     &  22,  24,  28,  30,  32,  36,  40,  42,  44,  48,
     &  56,  60,  64,  66,  70,  72,  80,  84,  88,  90,
     &  96, 110, 112, 120, 126, 128, 132, 140, 144, 154,
     & 160, 168, 176, 180, 192, 198, 210, 220, 224, 240,
     & 252, 256, 264, 280, 288, 308/
      integer i, nbest
      
      nbest = 2 * m + 1
      if(nbest .gt. nallwd(nn)) then
        write(*,*) '** ERROR nfft 1: '
        stop
      endif
      if(nbest .lt. 2) then
        write(*,*) '** ERROR nfft 2: '
        stop
      endif
      do i = nn, 1, -1
        if(nallwd(i) .ge. nbest) n = nallwd(i)
      enddo
      return
      end


!     ==================================================================
      subroutine n1dfft(n1,n1d)
!     ==================================================================
!     set the first dimension n1d of the FFT 3d arrays to be n1
!     ------------------------------------------------------------------
      implicit none
      integer n1, n1d
      n1d = n1
      return
      end      


!     ==================================================================
      subroutine cfft(n,x,y,isign)
!     ==================================================================
!     compute the Fourier transform of a complex periodic sequence
!     input:  x
!     output: y
!     forward:  sign = +1
!     backward: sign = -1
!     the array x and y may be the same
!     n must be acceptable:
!       n <= 37748736
!       n = 2^h * 3^i * 5^j * 7^k * 11^m  h=1,...,25  i=0,1,2  j,k,m=0,1
!     ------------------------------------------------------------------
      integer n
      complex x(n)
      complex y(n)
      
      integer isign
      integer, save :: init
      integer, parameter :: m = 1
      integer, parameter :: inc1x = 1
      integer, parameter :: inc1y = 1
      integer, parameter :: inc2x = 666
      integer, parameter :: inc2y = 666
      integer naux1
      integer, save :: naux2
      double precision, allocatable, save :: aux1(:)
      double precision, allocatable :: aux2(:)
      real, parameter :: scale = 1.0
      integer, save :: nold = -1
      integer istat
      

!     check initialization
      if(n .ne. nold) then
        init = 1
        naux1 = 20000
        if(n.gt.8192) naux1 = naux1 + 1.14 * n
        allocate(aux1(naux1),stat=istat)
        if(istat.ne.0) stop 'cfft out of memory'
        naux2 = naux1
        allocate(aux2(naux2),stat=istat)
        if(istat.ne.0) stop 'cfft out of memory'

        call scft(init,x,inc1x,inc2x,y,inc1y,inc2y,n,m,isign,scale,
     &            aux1,naux1,aux2,naux2)
     
        deallocate(aux2)
        
        nold = n
        init = 0
      endif

      allocate(aux2(naux2),stat=istat)
      if(istat.ne.0) stop 'cfft out of memory'
      
      call scft(init,x,inc1x,inc2x,y,inc1y,inc2y,n,m,isign,scale,
     &          aux1,naux1,aux2,naux2)
      
      deallocate(aux2)
      
      return
      end



!     ==================================================================
      subroutine c3dfft(a,lda,mda,hda,n1,n2,n3,isign)
!     ==================================================================
!     compute Fourier transform of a complex periodic three-dimensional array
!     input:  a
!     output: b=a
!     the matrices a and b may be the same, but in this case lda,mda>ldb,mdb
!     n1, n2, n3 must have acceptable values (see cfft)
!     ------------------------------------------------------------------
      integer n1, n2, n3
      integer lda, mda, hda
      complex a(lda,mda,*)
      integer isign
      
      real, parameter :: scale = 1.0
      integer, save :: naux = 60000
      double precision, allocatable :: aux(:)
      
!     the next lines are not required if n1<=8192, n2,n3<252
!      if(n3 < 252) then
!        if(n2 < 252) then
!          if(n1 <= 8192) then
!            naux = 60000
!          else
!            naux = 60000 + 2.28 * n1
!          endif
!        else
!          if(n1 <= 8192) then
!            naux = 60000 + (n2+256) * (min(64,n1) + 2.28)
!          else
!            naux = 60000 + 2.28 * n1 + (n2+256) * (min(64,n1) + 2.28)
!          endif
!        endif
!      else
!        if(n2 < 252) then
!          if(n1 <= 8192) then
!            naux = 60000 + (n3+256) * (min(64,n1*n2) + 2.28)
!          else
!            naux = 60000 + 2.28 * n1 + (n3+256) * (min(64,n1*n2) + 2.28)
!          endif
!        else
!          if(n1 <= 8192) then
!            naux = max( 60000 + (n2+256) * (min(64,n1) + 2.28) ,
!     &                  60000 + (n3+256) * (min(64,n1*n2) + 2.28) )
!          else
!            naux = max( 60000+2.28*n1 + (n2+256) * (min(64,n1) + 2.28) ,
!     &           60000 + 2.28 * n1 + (n3+256) * (min(64,n1*n2) + 2.28) )
!          endif
!        endif
!      endif


      allocate(aux(naux),stat=istat)
      if(istat.ne.0) stop 'cfft out of memory'
      
      call scft3(a,lda,lda*mda,a,lda,lda*mda,
     &           n1,n2,n3,isign,scale,aux,naux)
      
      deallocate(aux)
     
      return
      end


