! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

!     ==================================================================
!     interface to the vectorial GOEDECKER fft library
!     ==================================================================

!     ==================================================================
      integer function allowed_fftsize(n)
!     ==================================================================
!     calculate the size of the smallest allowed FFT size that
!     will encompass n
!     this routine is written for the GOEDECKER routine, in which the
!     allowable sizes of FFT are those whose prime factors are
!     2, 3 and 5.
!     ------------------------------------------------------------------
      implicit none
      integer n
      integer, parameter :: nn = 54
      integer nallwd(nn)
      data nallwd/
     &   2,   3,   4,   5,   6,   8,   9,  10,  12,  15,
     &  16,  18,  20,  24,  25,  27,  30,  32,  36,  40,
     &  45,  48,  50,  54,  60,  64,  72,  75,  80,  81,
     &  90,  96, 100, 108, 120, 125, 128, 135, 144, 150,
     & 160, 162, 180, 192, 200, 216, 225, 240, 243, 250,
     & 256, 270, 288, 300/
      integer i

      if(n > nallwd(nn) .or. n < 2) then
        print *, '**ERROR allowed_fftsize'
        stop 'error allowed fft_size'
      endif
      do i = nn, 1, -1
        if(nallwd(i) >= n) allowed_fftsize = nallwd(i)
      enddo
      return
      end


!     ==================================================================
      subroutine nfft(m,n)
!     ==================================================================
!     calculate N, the size of the smallest allowed FFT that will encompass
!     the 2M+1 points -M..+M.  (Ideally, N=2M+1, but this may not be
!     an allowable FFT.)
!
!     this routine is written for the GOEDECKER routine, in which the
!     allowable sizes of FFT are those whose prime factors are
!     2, 3 and 5.
!     ------------------------------------------------------------------
      PARAMETER (NN=54)
      INTEGER NALLWD(NN)
      DATA NALLWD/
     &   2,   3,   4,   5,   6,   8,   9,  10,  12,  15,
     &  16,  18,  20,  24,  25,  27,  30,  32,  36,  40,
     &  45,  48,  50,  54,  60,  64,  72,  75,  80,  81,
     &  90,  96, 100, 108, 120, 125, 128, 135, 144, 150,
     & 160, 162, 180, 192, 200, 216, 225, 240, 243, 250,
     & 256, 270, 288, 300/
      NBEST = 2 * M + 1
      IF (NBEST.GT.NALLWD(NN)) then
        write(*,*) '** ERROR nfft 1: '
        stop
      endif
      IF (NBEST.LT.2) then
        write(*,*) '** ERROR nfft 2: '
        stop
      endif
      do I = NN, 1, -1
        IF (NALLWD(I).GE.NBEST) N = NALLWD(I)
      enddo
      RETURN
      END


!     ==================================================================
      subroutine n1dfft(n1,n1d)
!     ==================================================================
      implicit none
      integer n1, n1d
      n1d = n1
      return
      end      


!     ==================================================================
      subroutine c3dfft(c,id,ie,if,nl,nm,nn,isig)
!     ==================================================================
!     == computes the forward fourier transform of a complex          ==
!     == function c. the fourier transform is                         ==
!     == returned in c in output (the input c is overwritten).        ==
!     ==   *****     single precision fourier transform      *****    ==
!     ==             isig <0  inverse                                 ==
!     ==             isig >0  direct                                  ==
!     ==--------------------------------------------------------------==

      integer id, ie, if
      integer nl, nm, nn
      integer isig
      complex c(id,ie,if)

      integer nd1, nd2, nd3
      integer inzee
      real, allocatable :: z(:,:,:,:,:)
      integer ncache
      real, allocatable :: zw(:)
!      real scale

      nd1 = nl
      nd2 = nm
      nd3 = nn
!     if even nd* = n* + 1
      if((nl/2)*2.eq.nl) nd1 = nl + 1
      if((nm/2)*2.eq.nm) nd2 = nm + 1
      if((nn/2)*2.eq.nn) nd3 = nn + 1

!     for vectorial machines ncache should be 0
      ncache = 0
!     ncache = max(nl,nm)
!     ncache = max(nn,ncache)
!     ncache = 4*ncache

      allocate(z(2,nd1,nd2,nd3,2),stat=istat)
      if(istat.ne.0) stop 'out of memory'
      allocate(zw(1),stat=istat)
!     allocate(zw(ncache),stat=istat)
!     if(istat.ne.0) stop 'out of memory'


      inzee = 1
      do k = 1, nn
        do j = 1, nm
          do i = 1, nl
            z(1,i,j,k,inzee) =  real(c(i,j,k))
            z(2,i,j,k,inzee) =  aimag(c(i,j,k))
          enddo
        enddo
      enddo

      call fftgoed(nl,nm,nn,nd1,nd2,nd3,z,isig,inzee,zw,ncache)

!      scale = 1.0/(nl*nm*nn)

      do k = 1, nn
        do j = 1, nm
          do i = 1, nl
            c(i,j,k) = cmplx(z(1,i,j,k,inzee),z(2,i,j,k,inzee))
          enddo
        enddo
      enddo

      deallocate(z)
      deallocate(zw)

      return
      end

