!     Copyright (C) 1998 Valerio Olevano
!
!     TDDFT code
!     Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
!     Calculate Dielectric Properties.

!     ==================================================================
!     interface to the Public Domain MFFT library of A. Nobile and V. Roberto
!     ==================================================================

!     ==================================================================
      integer function allowed_fftsize(n)
!     ==================================================================
!     calculate the size of the smallest allowed FFT size that
!     will encompass n
!     this routine is written for the MFFT routine C3FFT, in which the
!     allowable sizes of FFT are those whose prime factors are
!     2, 3 and 5.
!     ------------------------------------------------------------------
      implicit none
      integer n
      integer, parameter :: nn = 55
      integer nallwd(nn)
      data nallwd/
     &   1,   2,   3,   4,   5,   6,   8,   9,  10,  12,  15,
     &  16,  18,  20,  24,  25,  27,  30,  32,  36,  40,
     &  45,  48,  50,  54,  60,  64,  72,  75,  80,  81,
     &  90,  96, 100, 108, 120, 125, 128, 135, 144, 150,
     & 160, 162, 180, 192, 200, 216, 225, 240, 243, 250,
     & 256, 270, 288, 300/
      integer i

      if(n > nallwd(nn) .or. n < 1) then
        print *, '**ERROR allowed_fftsize'
        stop 'error allowed fft_size'
      endif
      do i = nn, 1, -1
        if(nallwd(i) >= n) allowed_fftsize = nallwd(i)
      enddo
      return
      end


!     ==================================================================
      subroutine c3dfft(c,id,ie,if,nl,nm,nn,isig)
!     ==================================================================
!     interface with the mffts library routines
!       id, ie, if : dimension of the complex array c
!       nl, nm, nn : size of the fft
!       isig <0  inverse fft
!       isig >0  direct fft
!       WARNING: the notation for MFFT is different wrt others
!       c : complex array to be transformed
!     ------------------------------------------------------------------
      implicit none
      integer id, ie, if
      integer nl, nm, nn
      complex c(id,ie,if)
      integer isig

      integer isigmfft
      integer, parameter :: iord = 1
      integer, save :: iopt
      integer nwl, nwm, nwn
      integer, save, allocatable :: wl(:), wm(:), wn(:)
      integer niwork
      integer, save, allocatable :: iwork(:)
      complex, allocatable :: rwork(:)
      logical, save :: init=.false.
      integer ierr
      integer istat
      integer nrwork

      if(.not.init) then
        if(ie.ne.nm.or.if.ne.nn) then
          write(*,*) '** ERROR c3dfft 1: wrong array dimensions;'
          write(*,*) '                   ie = ', ie
          write(*,*) '                   nm = ', nm
          write(*,*) '                   if = ', if
          write(*,*) '                   nn = ', nn
          stop 'wrong array dimensions'
        endif
        niwork = max0(nl,nm,nn)
        allocate(iwork(niwork),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        nwl = 4*nl+14
        if(nm.lt.32) then
          iopt = 1
          nwm = 4*nm*(id+1)+14
        else
          iopt = 0
          nwm = 4*nm+14
          stop 'check iopt'
        endif
        nwn = 4*nn+14
        allocate(wl(nwl),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        allocate(wm(nwm),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        allocate(wn(nwn),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        nrwork = id*nl*nm
        allocate(rwork(nrwork),stat=istat)
        if(istat.ne.0) stop 'out of memory'
        call c3fft(rwork,id,nl,nm,nn,wl,wm,wn,iopt,0,iord,iwork,ierr)
        if(ierr.ne.0) then
          write(*,*) '** ERROR c3dfft 2: fft initialization failed;'
          write(*,*) '                   ierr = ', ierr
          stop 'fft initialization failed'
        endif
        deallocate(rwork)
        init=.true.
      endif

      isigmfft = - isig
      call c3fft(c,id,nl,nm,nn,wl,wm,wn,iopt,isigmfft,iord,iwork,ierr)

      return
      end


