! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

 subroutine testem1(filename,fns,nq,nomega,npwvece,npwwfne,nbndse,fform)

 implicit none

 integer fns
 character (len=fns) :: filename
 integer, parameter :: iunit=12
 integer nq,nomega
 integer npwvece,npwwfne,nbndse
 integer fform,headform
 character (len=6) codvsn
 integer bantot,date,intxc,ixc,natom,ngfft(3),nkpt,&
&        nspden,nspinor,nsppol,nsym,ntype
 character (len=80) titem1(2)
 integer itype

 write(*,*) 'testing epsilon-twiddle^-1 file'
!try before em1 format
 if(fns/=0) then
   open(iunit,file=filename(1:fns),status='old',form='unformatted')
 else
   open(iunit,status='old',form='unformatted')
 endif
 read(iunit,err=100) codvsn,headform,fform
 if(fform/=1002) goto 100
 read(iunit,err=100) bantot,date,intxc,ixc,natom,ngfft(1:3),nkpt,&
&    nspden,nspinor,nsppol,nsym,ntype
 read(iunit,err=100)
 do itype=1,ntype
   read(iunit,err=100)
 enddo
 read(iunit,err=100)
 read(iunit,err=100) titem1
 read(iunit,err=100) npwvece,npwwfne,nbndse,nq,nomega
 write(*,*) 'epsilon^-1 file in the format em1'
 write(*,'(1x,a79/1x,a79)') titem1(1)(:79), titem1(2)(:79)
 write(*,*) 'dimension of epsilon^-1 matrix ',npwvece
 write(*,*) 'number of planewaves used      ',npwwfne
 write(*,*) 'number of bands used           ',nbndse
 write(*,*) 'number of q-points             ',nq
 write(*,*) 'number of omega                ',nomega
 write(*,*)
 close(iunit)
 return

 100 rewind(iunit)

!now try old epsm1 format in single and double precision
 read(iunit,err=200) titem1
 read(iunit,err=200) nq
!skip q-points
 read(iunit)
 read(iunit,err=200) npwvece,npwwfne,nbndse
 write(*,*) 'epsilon^-1 file in the format epsm1'
 write(*,'(1x,a79/1x,a79)') titem1(1)(:79), titem1(2)(:79)
 write(*,*) 'dimension of epsm1 matrix ',npwvece
 write(*,*) 'number of planewaves used ',npwwfne
 write(*,*) 'number of bands used      ',nbndse
 write(*,*) 'number of q-points        ',nq
 write(*,*)
 close(iunit)
 fform=1003
 nomega=2
 return

 200 close(iunit)
 write(*,*) 'epsilon^-1 file format unknown'
 stop 'epsilon^-1 file format unknown'
 end



 subroutine rdem1(em1file,es,npwvec,nq,nomega,q,omega,epsm1)

 implicit none

 real, parameter :: Ha_eV = 27.2113961
 integer es
 character (len=es) :: em1file
 integer, parameter :: iunit=12
 integer npwvece,npwwfne,nbndse,nqe,nomegae
 integer nq,nomega,npwvec
 real q(3,nq)
 complex omega(nomega)
 complex epsm1(npwvec,npwvec,nomega,nq)
 real (kind(0.0d0)) :: qd(3,nq)
 complex (kind(0.0d0)) :: omegad(nomega)
 complex (kind(0.0d0)), allocatable :: epsm1d(:,:)
 real dielconst
 integer fform,headform
 character (len=6) codvsn
 character (len=80) titem1(2)
 integer bantot,date,intxc,ixc,natom,ngfft(3),nkpt,&
&        nspden,nspinor,nsppol,nsym,ntype
 integer itype,ii,io,iq

 write(*,*) 'reading epsilon-twiddle^-1 file in the em1 format'

 epsm1(:,:,:,:)=(0.0,0.0)

 if(es/=0) then
   open(iunit,file=em1file(1:es),status='old',form='unformatted')
 else
   open(iunit,status='old',form='unformatted')
 endif
 read(iunit) codvsn,headform,fform
 read(iunit) bantot,date,intxc,ixc,natom,ngfft(1:3),nkpt,&
&    nspden,nspinor,nsppol,nsym,ntype
 read(iunit)
 do itype=1,ntype
   read(iunit)
 enddo
 read(iunit)
 read(iunit) titem1
 read(iunit) npwvece,npwwfne,nbndse,nqe,nomegae
 if(npwvece>npwvec) then
   write(*,*) 'dimension of epsm1 matrix         ', npwvece
   write(*,*) 'warning: reading a smaller matrix ', npwvec
 else if(npwvece<npwvec) then
   write(*,*) 'dimension of epsm1 matrix         ', npwvece
   write(*,*) 'warning: requiring a bigger matrix ', npwvec
   write(*,*) '         0-padding the rest of the matrix'
 endif
 if(nqe<nq) stop 'requiring too much q-points'
 if(nomegae<nomega) stop 'requiring too much omega-points'
 read(iunit) ! skip gvec
 read(iunit) qd(1:3,1:nq)
 read(iunit) omegad(1:nomega)
 q(:,:) = qd(:,:)
 omega(:) = omegad(:)

 write(6,'(" number of q-points ",i4)') nq
 write(6,'(a,/,(3f11.6))')&
& ' q-points [reduced coordinates]:',((q(ii,iq),ii=1,3),iq=1,nq)
 write(6,'(" number of frequencies ",i4)') nomega
 write(6,'(a,/,4(" (",f7.2,",",f7.2,")"))')&
& ' frequencies omega [eV]:',(omega(io)*Ha_eV,io=1,nomega)

!read epsilon^-1
 allocate(epsm1d(npwvece,npwvece))
 do iq=1,nq
   do io=1,nomega
     read(iunit) epsm1d(1:npwvece,1:npwvece)
     epsm1(1:npwvec,1:npwvec,io,iq) = epsm1d(1:npwvec,1:npwvec)
   enddo
   do io=nomega+1,nomegae
     read(iunit) ! skip other omegas
   enddo
 enddo

!close(iunit)
 deallocate(epsm1d)
 
 if(abs(real(omega(1)))<0.01 .and. abs(aimag(omega(1)))<0.01) then
   do iq=1,nq
     if(all(abs(q(:,iq))<0.001)) then
     dielconst = 1.0/real(epsm1(1,1,1,iq))
     write(*,'(a,f6.2)') ' dielectric constant with LF = ', dielconst
     endif
   enddo
 endif

 write(*,*) 'epsilon-twiddle^-1 read'
 write(*,*)

 return
 end


 subroutine rdepsm1(em1file,es,npwmat,nq,q,epsm1,bigomegatwsq,omegatw)

 implicit none

 integer es
 character (len=es) :: em1file
 integer, parameter :: iunit=12
 integer npwvece,npwwfne,nbndse
 integer npwmat
 integer nq, nqr
 real q(3,nq)
 real (kind(0.0d0)) qd(3,nq)
 real qs(3,nq)
 complex cdum
 complex,allocatable :: a(:,:),omegatwsq(:,:)
 character (len=80) titem1(2)
 complex bigomegatwsq(npwmat,npwmat,nq),omegatw(npwmat,npwmat,nq)
 complex epsm1(npwmat,npwmat,nq)

 integer ii,iq,istat,jj,ig,igp

 write(*,*) 'reading epsilon-twiddle^-1 file in the epsm1 format'

 allocate(a(npwmat,npwmat),stat=istat)
 if(istat/=0) stop 'out of memory'
 allocate(omegatwsq(npwmat,npwmat),stat=istat)
 if(istat/=0) stop 'out of memory'

 if(es/=0) then
   open(iunit,file=em1file(1:es),status='old',form='unformatted')
 else
   open(iunit,status='old',form='unformatted')
 endif
 rewind(iunit)
! try first double precision

 read(iunit) titem1
 write(6,'(a,/,a80)') ' title:',titem1

 read(iunit) nqr
 write(*,*) 'number of q-points ',nqr
 if(nqr/=nq) then
   write(*,*) '**error rdepsm1 1: number of q-point error'
   stop
 endif

 read(iunit,err=400) ((qd(ii,iq),ii=1,3),iq=1,nq)
 q(:,:) = qd(:,:)
 write(6,'(a,/,(i3,3f14.6))')&
& ' q-points [reciprocal lattice units]:',(iq,(q(ii,iq),ii=1,3),iq=1,nq)

 read(iunit) npwvece,npwwfne,nbndse
 write(6,'(a,3(/,a,i5))') ' parameters in epsilon^-1 calculation:',&
& ' number of plane wave vectors, NPWVEC = ',npwvece,&
& ' number of plane waves for wavefunctions, NPWWFN = ',npwwfne,&
& ' number of bands, NBNDS  = ',nbndse

 if(npwvece<npwmat) then
   write(*,*) '**error rdepsm1 2: required more plane-wave vectors'
   write(*,*) '                  than available in epsilon^-1 file'
   write(*,*) '                  npwmat = ',npwmat
   stop
 endif

!read plasmon-pole parameters (A and omega-twiddle-squared)
!and write to direct-access scratch file
 do iq=1,nq

!  skip over table of |q+G|: this will be calculated separately
   read(iunit)

!  read A parameters
   read(iunit) ((a(ii,jj),ii=1,npwmat),(cdum,ii=npwmat+1,npwvece),&
&   jj=1,npwmat)
   
   epsm1(:,:,iq) = a(:,:)     
   do ii = 1, npwmat
     epsm1(ii,ii,iq) = 1.0 + epsm1(ii,ii,iq)
   enddo
   if(all(abs(q(:,iq))<1.e-3)) then
     write(*,'(a,f5.2)') &
&     ' macroscopic dielectric constant = ',real(1.0 / epsm1(1,1,iq))
   endif

!  read omega-twiddle-squared parameters
   read(iunit) ((omegatwsq(ii,jj),ii=1,npwmat),(cdum,ii=npwmat+1,npwvece),&
&   jj=1,npwmat)
   
!  If omega-twiddle-squared is negative,
!  set omega-twiddle-squared to 1.0 (a reasonable way of treating
!  such terms, in which epsilon**-1 was originally increasing
!  along this part of the imaginary axis)
!  (note: originally these terms were ignored in Sigma; this was
!  changed on 6 March 1990.)
   do ig=1,npwmat
     do igp=1,npwmat
       if(real(omegatwsq(ig,igp))<=0.0) omegatwsq(ig,igp)=1.0
     enddo
   enddo
!  calculate omega-twiddle
!  we neglect the imag part (if one) in omega-twiddle-squared         
   omegatw(:,:,iq)=sqrt(real(omegatwsq(:,:)))

!  calculate big-omega-twiddle-squared = -omega-twiddle-squared A
   bigomegatwsq(:,:,iq)=-a(:,:)*omegatw(:,:,iq)**2

 enddo

 close(iunit)
 deallocate(a,omegatwsq)

 write(*,*) 'epsilon-twiddle^-1 read'
 write(*,*)

 return
 
 400 rewind(iunit)
! now try first single precision

 read(iunit) titem1

 read(iunit) nqr

 read(iunit,err=400) ((qs(ii,iq),ii=1,3),iq=1,nq)
 q(:,:) = qs(:,:)
 write(6,'(a,/,(i3,3f14.6))')&
& ' q-points [reciprocal lattice units]:',(iq,(q(ii,iq),ii=1,3),iq=1,nq)

 read(iunit) npwvece,npwwfne,nbndse
 write(6,'(a,3(/,a,i5))') ' parameters in epsilon^-1 calculation:',&
& ' number of plane wave vectors, NPWVEC = ',npwvece,&
& ' number of plane waves for wavefunctions, NPWWFN = ',npwwfne,&
& ' number of bands, NBNDS  = ',nbndse

 if(npwvece<npwmat) then
   write(*,*) '**error rdepsm1 2: required more plane-wave vectors'
   write(*,*) '                  than available in epsilon^-1 file'
   write(*,*) '                  npwmat = ',npwmat
   stop
 endif

!read plasmon-pole parameters (A and omega-twiddle-squared)
!and write to direct-access scratch file
 do iq=1,nq

!  skip over table of |q+G|: this will be calculated separately
   read(iunit)

!  read A parameters
   read(iunit) ((a(ii,jj),ii=1,npwmat),(cdum,ii=npwmat+1,npwvece),&
&   jj=1,npwmat)

   epsm1(:,:,iq) = a(:,:)     
   do ii = 1, npwmat
     epsm1(ii,ii,iq) = 1.0 + epsm1(ii,ii,iq)
   enddo
   if(all(abs(q(:,iq))<1.e-3)) then
     write(*,'(a,f5.2)') &
&     ' macroscopic dielectric constant = ',real(1.0 / epsm1(1,1,iq))
   endif

!  read omega-twiddle-squared parameters
   read(iunit) ((omegatwsq(ii,jj),ii=1,npwmat),(cdum,ii=npwmat+1,npwvece),&
&   jj=1,npwmat)

!  If omega-twiddle-squared is negative,
!  set omega-twiddle-squared to 1.0 (a reasonable way of treating
!  such terms, in which epsilon**-1 was originally increasing
!  along this part of the imaginary axis)
!  (note: originally these terms were ignored in Sigma; this was
!  changed on 6 March 1990.)
   do ig=1,npwmat
     do igp=1,npwmat
       if(real(omegatwsq(ig,igp))<=0.0) omegatwsq(ig,igp)=1.0
     enddo
   enddo
!  calculate omega-twiddle
!  we neglect the imag part (if one) in omega-twiddle-squared         
   omegatw(:,:,iq)=sqrt(real(omegatwsq(:,:)))

!  calculate big-omega-twiddle-squared = -omega-twiddle-squared A
   bigomegatwsq(:,:,iq)=-a(:,:)*omegatw(:,:,iq)**2

 enddo

 close(iunit)
 deallocate(a,omegatwsq)

 write(*,*) 'epsilon-twiddle^-1 read'
 write(*,*)
 end
