! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

subroutine rdgw(gwfile,gl,nkibz,nkbz,nbnds,lomo,kbz,ktab,gwsh,nkibzm)
  ! read GW-energy-parameters
  !
  ! input:
  !   gwfile
  use ggwdef
  integer gl
  character (len=gl) :: gwfile
  integer nkibzm
  integer nkibz, nkbz, nbnds, lomo
  integer ktab(nkbz)
  real kbz(3,nkbz)
  real gwen(nkibzm,nbnds)
  complex gwsh(nkibzm,nbnds)
  
  real en
  real gwr, gwi
      
  integer ik, ikibz
  integer ibcheck, ibcheckok
  integer status, filevers
     
  integer nk
  real k(3), k2bz(3)
  integer g1, g2, g3
  logical, parameter :: DEBUG=.false.

  print *, 'reading GW corrections'
   
  gwsh(:,:) = 0
  gwen(:,:) = 0

  ibcheckok = 0
  do ib = lomo, nbnds
    ibcheckok = ibcheckok + ib
  enddo

  ! determine file version
  filevers = 10
  open(13,file=gwfile,status='unknown')
  read(13,*) nk
  read(13,*) (k(i),i=1,3)
  read(13,*) nb
!  print *, nk, k, nb
  read(13,*) ib, en, gwr, gwi
  read(13,*) jb, en, gwr, gwi
  if(jb==ib+1) filevers = 11
!  print *, 'status ', ib, en, gwr, gwi, jb
  close(13)
  print *, 'gw file version ', filevers

  ! opening file
  open(13,file=gwfile,status='unknown')

  ! reading number of k-points
  read(13,*) nk
  write(*,*) 'number of k-points found ', nk
      
  if(nk.lt.nkibz) then
    write(*,*) 'warning: the number of k-points found is less than ', nkibz
  endif


  do j = 1, nk

    read(13,*) (k(i),i=1,3)
 
    ! determine the index number in bz
    ik = 0
    do jk = 1, nkbz
      if(all(abs(k-kbz(1:3,jk)) < 1.e-4)) ik = jk
    enddo
    if(ik==0) then
      write(*,'("  trying to find k =",3f14.6," in other BZ zones")') (k(i),i=1,3)
      do g1 = -1, 1
      do g2 = -1, 1
      do g3 = -1, 1
        k2bz(1) = k(1) + g1
        k2bz(2) = k(2) + g2
        k2bz(3) = k(3) + g3
        do jk = 1, nkbz
          if(all(abs(k2bz-kbz(1:3,jk)) < 1.e-4)) ik = jk
        enddo
      enddo
      enddo
      enddo
    endif
    if(ik == 0) then
      write(*,'("**ERROR, rdgw: k-point ",3f14.6," is not in set of kBZ!")') (k(i),i=1,3)
      stop '**ERROR, rdgw: k-point not in the set'
    endif

    ikibz = ktab(ik)
    write(*,100) ik, ikibz, kbz(:,ik)
100 format('  k-point, BZ number ',i3,'  IBZ number ',i3,'  r.c. ',3f7.3)

    read(13,*) nb
    write(*,*) '  number of bands found ', nb

    if(nb < (nbnds-lomo+1)) then
      write(*,*) 'warning: the number of bands found is less than ', nbnds-lomo+1
    endif
        
    if(DEBUG) write(*,'(3x,"band",3x,"gw energy",4x,"gw shift")')
    ibcheck = 0  
    do jb = 1, nb
      if(filevers==10) then
        read(13,*) ib, en, gwr
        gwi = 0
      else if(filevers==11) then
        read(13,*) ib, en, gwr, gwi
      endif
      en = en / H2eV
      gwr = gwr / H2eV
      gwi = gwi / H2eV
      if(ib <= nbnds) then
        gwsh(ikibz,ib) = cmplx(gwr,gwi)
        gwen(ikibz,ib) = en
        if(DEBUG) write(*,200) ib, gwen(ikibz,ib)*H2eV, gwsh(ikibz,ib)*H2eV
200     format(4x,i3,6x,f6.2,6x,f6.2,6x,f6.2)
        if(ib >= lomo) ibcheck = ibcheck+ib
      endif
    enddo ! jb
    if(ibcheck .ne. ibcheckok) then
      print *, 'warning: some of the gw corrections are missing'
    endif
        
  enddo ! j
  
  print *
  print *, ' k       Re GW shift [eV]'
  do ik = 1, nkibz
    write(*,'(i3,7x,10f7.2/50(10x,10f7.2/))') &
&         ik, (H2eV*real(gwsh(ik,ib)),ib=1,nbnds)
  enddo
  print *

  print *, ' k       Im GW shift [eV]'
  do ik = 1, nkibz
    write(*,'(i3,7x,10f7.2/50(10x,10f7.2/))') &
&         ik, (H2eV*aimag(gwsh(ik,ib)),ib=1,nbnds)
  enddo
  print *

  close(13)
      
  return
end
      
      
subroutine wrgw(nk,xk,maxbnd,minbnd,gwen,gwsh,nkibzm)
  ! write GW-energy-parameters
  !
  ! input:
  use ggwdef
  implicit none
  integer nkibzm
  integer nk
  real xk(3,nk)
  integer minbnd(nk), maxbnd(nk)
  real gwen(nkibzm,*)
  real gwsh(nkibzm,*)
      
  real en
  real gw
      
  integer ik, ib, nb, i

  write(*,*) 'writing GW energies and GW corrections'

  ! opening file
  open(21,file='out.gw')

  ! writing number of k-points
  write(21,'(i6)') nk
  write(*,*) 'number of k-points', nk

  do ik = 1, nk

    write(21,'(3f14.9)') (xk(i,ik),i=1,3)
    nb = maxbnd(ik) - minbnd(ik) + 1
    write(21,'(i2)') nb

    do ib = minbnd(ik), maxbnd(ik)
      en = gwen(ik,ib)*H2eV
      gw = gwsh(ik,ib)*H2eV
      write(21,'(i2,3x,f8.4,3x,f8.4)') ib, en, gw
    enddo ! ib
  enddo ! ik
  close(21)
  print *
      
  return
end

