! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

! subroutines for the i/o of files QPLDAtype
! created: 7-1998 Valerio Olevano

! DEBUG: logical constant to control output values
! VERBOSE: logical constant to control output values


!=======================================================================
subroutine testlda(kssfile,kl,nsym,nkpt,nbnd,npw,nsh,i1,i2,&
&                   ntype,natom,mpsang,nel)
!=======================================================================
! test QPLDA type file
use ggwdef
implicit none
integer kl
character (len=kl) :: kssfile
integer, parameter :: u=10
integer fform
integer nsym, nkpt, nbnd, npw, nsh, ntype, natom, mpsang, nel, nsym2
character (len=80) :: tit(2)
integer i1, i2
type(crystal) crystr
type(symmetry) symstr
type(chemistry) chemstr
type(mesh) meshstr
type(transposed_space) foustr
type(electronic_structure) elecstr
integer, external :: fileform
logical, parameter :: DEBUG = .false.

print *, 'checking Kohn-Sham Structure input file ', kssfile
fform = fileform(kssfile,kl)
if(fform == 502) then
  print *, 'kss ABINIT double precision form'
else if(fform == 602) then
  print *, 'kss ABINIT single precision form'
else if(fform==2) then
  print *, 'sta double precision form'
else if(fform==1) then
  print *, 'qplda LSI-CP, PWSCF, FHIMD single precision form'
else
  stop 'unrecognized file format'
endif

i2 = fform
if(fform < 0) stop 'unrecognized file'
if(fform == 502 .or. fform == 602) then
  open(u,form='unformatted',file=kssfile)
  call rdhdr(u,crystr,symstr,chemstr,meshstr,foustr,elecstr)
  read(u) tit(1)
  read(u) tit(2)
  if(DEBUG) print *, 'title of file:'
  if(DEBUG) write(*,'(1x,a79/1x,a79)') tit(1)(:79), tit(2)(:79)
  read(u) nsym2, nbnd, npw, nsh, mpsang
  nsym = nsym2
  ntype = chemstr%ntype
  natom = chemstr%natom
  nel = chemstr%nel
  nkpt = foustr%nk
  print *, 'number of atomic species      ', ntype
  print *, 'number of atoms               ', natom
  print *, 'number of ang mom components  ', mpsang
  print *, 'number of symmetry operations ', nsym
  print *, 'number of plane shells        ', nsh
  print *, 'number of plane waves         ', npw
  print *, 'number of k-points            ', nkpt
  print *, 'number of bands               ', nbnd
  print *, 'number of electrons           ', nel
  print *
  close(u)
  return
else if(fform==1 .or. fform==2) then
  open(u,form='unformatted',file=kssfile)
  read(u) tit(1)
  read(u) tit(2)
  print *, 'title of file:'
  write(*,'(1x,a79/1x,a79)') tit(1)(:79), tit(2)(:79)
  read(u) i1,i2
  read(u) !     skip real-space lattice vectors
  read(u) nsym
  print *, 'number of symmetry operations ', nsym
  read(u) !     skip symmetry operations
  read(u) npw
  print *, 'number of plane waves         ', npw
  read(u) !     skip g-vector
  read(u) nkpt
  print *, 'number of k-points            ', nkpt
  read(u) !     skip k-points
  read(u) nbnd
  print *, 'number of bands               ', nbnd
  close(u)
  print * 
  nel = -1
  return
endif
end subroutine


!=======================================================================
subroutine rdkss(kssfile,kl,nopx,nbx,nkx,ngx,nwx,&
&                tit,a1,a2,a3,symstr,op,gvec,kibz,en,oc,wf,&
&                ntypex,natomx,mpsangx,type,xred,&
&                vkbsigni,vkbi,vkbdi)
!=======================================================================
use ggwdef
implicit none
logical, parameter :: DEBUG = .false.
integer, parameter :: u = 10
integer kl
character (len=kl) :: kssfile
integer nopx, nbx, nkx, ngx, nwx
integer ntypex, natomx, mpsangx
!      integer nop, nb, nk, ng
character (len=80) :: tit(2)
real a1(3), a2(3), a3(3)
real op(3,3,nopx)
integer gvec(3,ngx)
real kibz(3,nkx)
real en(nkx,nbx)
real oc(nkx,nbx)
complex wf(nwx,nbx,nkx)
integer type(natomx)
double precision :: xred(3,natomx)
real vkbsigni(mpsangx,ntypex)
real vkbi(nwx,ntypex,mpsangx,nkx)
real vkbdi(nwx,ntypex,mpsangx,nkx)
type(crystal) crystr
type(symmetry) symstr
type(chemistry) chemstr
type(mesh) meshstr
type(transposed_space) foustr
type(electronic_structure) elecstr

integer mpsang

integer :: nsym2, nbndsto, ncomsto, nsh
integer, allocatable :: symrel2(:,:,:)
double precision, allocatable :: tnons2(:,:)
integer, allocatable :: shlim(:)

real, allocatable :: vkbsign(:,:)
double precision, allocatable :: vkbsignd(:,:)
double precision, allocatable :: vkb(:,:,:), vkbd(:,:,:)
double precision, allocatable :: energy(:)
complex (kind(0.0d0)), allocatable :: wfg(:)
      
real, allocatable :: vkbs(:,:,:), vkbds(:,:,:)
real, allocatable :: energys(:)
complex, allocatable :: wfgs(:)
     
integer i, j, ik, ib, ig, is, itype, isym, isymin, isymend, il,ia
integer ish, osl, ibp
integer year, month, day
real dum, sum, einf, esup, cinf, csup
real (kind(0.d0)) :: ddum
integer fform
integer, external :: fileform

print *, 'reading kss file ', kssfile
fform = fileform(kssfile,kl)
if(fform /= 502 .and. fform /= 602) stop 'not kss form file'

open(u,form='unformatted',file=kssfile)
call rdhdr(u,crystr,symstr,chemstr,meshstr,foustr,elecstr)
print *
call prhdr(crystr,symstr,chemstr,meshstr,foustr,elecstr)

a1(1:3) = crystr%rprimd(1:3,1)
a2(1:3) = crystr%rprimd(1:3,2)
a3(1:3) = crystr%rprimd(1:3,3)
if(natomx /= chemstr%natom) then
  print *, 'error: natom ', natomx, chemstr%natom
  stop 'error natom'
endif
type(1:chemstr%natom) = chemstr%type(1:chemstr%natom)
xred(1:3,1:chemstr%natom) = chemstr%xred(1:3,1:chemstr%natom)
if(nkx /= foustr%nk) stop 'error nk'
kibz(1:3,1:foustr%nk) = foustr%k(1:3,1:foustr%nk)
oc(:,:) = 0.0
do ik = 1, foustr%nk
  do ib = 1, elecstr%nb
    if(ib>nbx) cycle
    oc(ik,ib) = elecstr%oc(ib,ik)
  enddo
enddo

read(u) tit(1)
read(u) tit(2)
if(DEBUG) print *, 'title of file:'
if(DEBUG) write(*,'(1x,a79/1x,a79)') tit(1)(:79), tit(2)(:79)

read(u) nsym2, nbndsto, ncomsto, nsh, mpsang

if(DEBUG) print *, 'number of symmetries without inversion ', nsym2
if(DEBUG) print *, 'number of bands ', nbndsto
if(DEBUG) print *, 'number of plane waves ', ncomsto
if(DEBUG) print *, 'number of shells ', nsh
if(DEBUG) print *, 'mpsang ', mpsang
if(mpsang/=mpsangx) stop 'mpsang/=mpsangx'
if(nopx>nsym2) stop 'nopx>nsym2'
if(ngx>ncomsto) stop 'ngx>ncomsto'
if(nwx>ncomsto) stop 'nwx>ncomsto'
if(nbx>nbndsto) stop 'nbx>nbndsto'
      
allocate(symrel2(3,3,nsym2))
! pay attention: the indeces i,j are inverted because a transposition
! is needed to obtain symmetries in transposed k-space
! in the file symmetries in real space are stored, identical to what is
! read in the header
read(u) (((symrel2(j,i,isym),i=1,3),j=1,3),isym=1,nsym2) ! the index is inverted
op(:,:,1:nopx) = symrel2(:,:,1:nopx)

allocate(tnons2(3,nsym2))
read(u) ((tnons2(i,isym),i=1,3),isym=1,nsym2)
      
! the inversion could be taken into account
! but not the non-symmorphic
print *, 'symmetries of the system in transposed space (must be symmorphyc): '
write(*,'("  number of symmetry operations ",i2)') nopx
print *, ' symmetry operations (R t)'
do isymin=1,nopx,4
  isymend=isymin+3
  if(isymend>nopx) isymend=nopx
  do i=1,3
    write(*,'(4(3i3,f8.3,3x))') &
&   ((symrel2(i,j,isym),j=1,3),tnons2(i,isym),isym=isymin,isymend)
  enddo
  print *
enddo

if(any(abs(tnons2(:,1:nopx))>0.0001)) then
  print *, 'WARNING: non-symmorphyc symmetries found'
  print *, 'non-symmorphyc symmetries work on versions 4.6.*'
  print *, 'they do not work on versions 5.0.*'
  print *, 'in versions 5.0.* switch off them using symmorphy 0 in ABINIT'
  print *
!  print *, 'try to change the atomic position'
!  stop 'non-symmorpyc symmetries'
endif

read(u) ((gvec(i,ig),i=1,3),ig=1,ngx)

allocate(shlim(nsh))
read(u) (shlim(i),i=1,nsh)
      
write(*,'(" shell limit G-vectors [reduced coordinates]")')
osl = 1
do ish = 1, nsh
  if(shlim(ish)>ngx) osl = shlim(ish)+1
  write(*,'(i6,i6,4(i5," (",3i3,")"), &
&         20(/12x,4(i5," (",3i3,")")))') &
&    ish, shlim(ish), (ig,(gvec(i,ig),i=1,3),ig=osl,shlim(ish))
  osl = shlim(ish)+1
enddo
print *

ish = 1
do while((shlim(ish) < nwx) .and. (ish < nsh))
  ish = ish + 1
enddo
if(shlim(ish) > nwx .or. ish > nsh) then
  print *, 'WARNING: npwwfn not on a shell'
  print *, 'chose instead ', shlim(ish-1)
endif
ish = 1
do while((shlim(ish) < ngx) .and. (ish < nsh))
  ish = ish + 1
enddo
if(shlim(ish) > ngx .or. ish > nsh) then
  print *, 'WARNING: npwmat not on a shell'
  print *, 'chose instead ', shlim(ish-1)
endif

      if(fform==502) then
  allocate(vkbsignd(mpsang,chemstr%ntype))
  read(u) ((vkbsignd(il,is),il=1,mpsang),is=1,chemstr%ntype)
  vkbsigni(1:mpsang,1:chemstr%ntype) = vkbsignd(1:mpsang,1:chemstr%ntype)
else if(fform==602) then
  allocate(vkbsign(mpsang,chemstr%ntype))
  read(u) ((vkbsign(il,is),il=1,mpsang),is=1,chemstr%ntype)
  vkbsigni(1:mpsang,1:chemstr%ntype) = vkbsign(1:mpsang,1:chemstr%ntype)
endif

write(*,'(" vkbsign: ",8f4.1)') vkbsigni(:,:)
print *
if(any(vkbsigni(1:mpsang,1:chemstr%ntype)<0)) then
  print *, 'WARNING: ABINIT version before 4.0.3 and 3.4.4'
  print *, 'could give wrong result with vkbsign = -1'
  print *, 'check your ABINIT version and use a newer one'
  print *
endif
            
if(fform==502) then      
  allocate(vkb(ncomsto,chemstr%ntype,mpsang))
  allocate(vkbd(ncomsto,chemstr%ntype,mpsang))
  allocate(energy(nbndsto))
  allocate(wfg(ncomsto))
else if(fform==602) then
  allocate(vkbs(ncomsto,chemstr%ntype,mpsang))
  allocate(vkbds(ncomsto,chemstr%ntype,mpsang))
  allocate(energys(nbndsto))
  allocate(wfgs(ncomsto))
endif
      
print *, ' k       eigenvalues [eV]'
do ik = 1, foustr%nk
! print *, 'ik ', ik
  do is = 1, chemstr%ntype
!   print *, 'is ', is
    do il = 1, mpsang
!     print *, 'il ', il
      if(fform==502) then      
        read(u) vkb(:,is,il)
        read(u) vkbd(:,is,il)
        vkbi(1:nwx,is,il,ik) = vkb(1:nwx,is,il)
        vkbdi(1:nwx,is,il,ik) = vkbd(1:nwx,is,il)
      else if(fform==602) then
        read(u) vkbs(:,is,il)
        read(u) vkbds(:,is,il)
        vkbi(1:nwx,is,il,ik) = vkbs(1:nwx,is,il)
        vkbdi(1:nwx,is,il,ik) = vkbds(1:nwx,is,il)
      endif
      if(DEBUG) print *, 'vkb ', vkbi(:,:,:,ik)
      if(DEBUG) print *, 'vkbd ', vkbdi(:,:,:,ik)
    enddo
  enddo
        
! check that the reference be coherent with vkb vkbd
  do is = 1, chemstr%ntype
    if(chemstr%psa(is)%lloc /= -1) then
      il = chemstr%psa(is)%lloc + 1
      dum = sum(vkbi(:,is,il,ik))
      dum = dum + sum(vkbdi(:,is,il,ik))
      if(dum/=0.0) then
        print *, '** WARNING: lloc not coherent with vkb'
        print *, 'specie ', is
        print *, 'lloc ', chemstr%psa(is)%lloc
        print *, 'sum(vkb + vkbd) ', dum
      endif
    else
      ! found lloc on the basis of vkb
    endif
  enddo
                 
        
  if(fform==502) then      
    read(u) energy(1:nbndsto)
    en(ik,1:nbx) = energy(1:nbx)
  else if(fform==602) then
    read(u) energys(1:nbndsto)
    en(ik,1:nbx) = energys(1:nbx)
  endif
        
  write(*,'(i3,7x,10f7.2/50(10x,10f7.2/))') &
&        ik, (HARTREE*en(ik,ib),ib=1,nbx)

  if(fform==502) then      
    do ib = 1, nbx
      read(u) wfg(1:ncomsto)
      wf(1:nwx,ib,ik) = wfg(1:nwx)
    enddo
    do ib = nbx+1 , nbndsto
      read(u) wfg(1:ncomsto)
    enddo
  else if(fform==602) then
    do ib = 1, nbx
      read(u) wfgs(1:ncomsto)
      wf(1:nwx,ib,ik) = wfgs(1:nwx)
    enddo
    do ib = nbx+1 , nbndsto
      read(u) wfgs(1:ncomsto)
    enddo
  endif
enddo

if(fform==502) then
  deallocate(vkb,vkbd,energy,wfg)      
else if(fform==602) then
  deallocate(vkbs,vkbds,energys,wfgs)
endif

allocate(wfg(nwx))
print *
            
print *, 'test on the normalization of the wavefunctions'
einf = 1.0e24; esup = 0.0
cinf = 1.0e24; csup = 0.0
do ik = 1, nkx
! test on the normalization of wavefunctions
  do ib = 1, nbx
!   here it can occur a
!   forrtl: error (74): floating underflow
!   it means that the imag part of the dot_product is really small
    wfg(1:nwx) = wf(1:nwx,ib,ik)
    ddum = abs(dot_product(wfg(:),wfg(:)))
    if(ddum<einf) einf = ddum
    if(ddum>esup) esup = ddum
  enddo
! test on the orthogonalization of wavefunctions
  do ib = 1, nbx
    do ibp = ib+1, nbx
      ddum = abs(dot_product(wf(:,ib,ik),wf(:,ibp,ik)))
      if(ddum<cinf) cinf = ddum
      if(ddum>csup) csup = ddum
    enddo
  enddo
enddo
deallocate(wfg)

write(6,5693) einf, esup
5693  format(' min sum_G |a(n,k,G)| = ',f9.6/ &
     &       ' max sum_G |a(n,k,G)| = ',f9.6)
print *, 'test on the orthogonalization of the wavefunctions'
write(6,5694) cinf, csup
5694  format(' min sum_G a(n,k,G)* a(n",k,G) = ',f9.6/ &
     &       ' max sum_G a(n,k,G)* a(n",k,G) = ',f9.6)
print *
      
close(u)
return
end subroutine




!=======================================================================
subroutine rdhdr(u,crystr,symstr,chemstr,meshstr,foustr,elecstr)
!=======================================================================
! read ABINIT file header
use ggwdef
implicit none
integer u
type(crystal) crystr
type(symmetry) symstr
type(chemistry) chemstr
type(mesh) meshstr
type(transposed_space) foustr
type(electronic_structure) elecstr

logical, parameter :: DEBUG = .false.
character (len=6) :: codvsn
integer :: headform, fform
integer :: date, year, month, day

integer :: bantot, intxc, ixc, natom, ngfft(3), nkpt
integer :: nspden, nspinor, nsppol, nsym, npsp, ntype, occopt, pertcase
double precision :: zatom
integer, allocatable :: nband(:), npwarr(:), symrel(:,:,:), symafm(:)
integer, allocatable :: type(:)
integer, allocatable :: istwfk(:), so_typat(:), so_psp(:)
double precision :: acell(3), ecut, ecutsm, ecut_eff, rprimd(3,3), qptn(3)
double precision :: stmbias, tphysel, tsmear
double precision :: residm,etotal,fermie
double precision, allocatable :: xred(:,:) ! (3,natom)
double precision, allocatable :: kpt(:,:), wtk(:), occ(:), tnons(:,:)
double precision, allocatable :: znucltypat(:)
! pseudopotential
double precision, allocatable :: znuclpsp(:), zionpsp(:)
character, allocatable :: title(:,:)
integer, allocatable :: pspso(:),pspdat(:),pspcod(:),pspxc(:),lmax(:),lloc(:),mmax(:)

integer i,j,ik,ib,isym,isymin,isymend,ia,ipa,it, nel
integer mpsang, itype, iatom
integer usepaw, usewvl
integer, allocatable :: lmn_size(:)
double precision :: ecutdg


read(u) codvsn,headform,fform

if(DEBUG) write(*,'(" ABINIT unformatted file, format ",i3)') fform
write(*,'(" generated by ABINIT version ",a6," header version ",i1,".",i1)')&
&     codvsn, headform/10, headform-(headform/10)*10

if(headform /= 23 .and. headform /= 34 .and. headform /= 40&
&    .and. headform /= 41 .and. headform /= 42 .and.  headform /= 44&
&    .and. headform /= 53 .and. headform /= 56 .and. headform /= 57) then
  print *, '** WARNING: unrecognized abinit header version', headform
  if(headform>57 .and. headform<99) then
    print *, '**          The code tries to read the file using header version 5.7.'
    print *, '**          If ABINIT programmers have modified the header putting'
    print *, '**          new variables at the end of existing records, the result is OK.'
    print *, '**          If they have put new variables in the middle of records'
    print *, '**          or they have created new records, the result will be WRONG.'
    print *
  else
    stop 'unrecognized abinit header version'
  endif
endif

! 1st record
if(headform==23) then
  read(u) bantot, date, intxc, ixc, natom, ngfft(1:3), nkpt,&
&         nspden, nspinor, nsppol, nsym, ntype, occopt,&
&         acell(1:3), ecut_eff, rprimd(1:3,1:3)
else if(headform==34) then
  read(u) bantot, date, intxc, ixc, natom, ngfft(1:3), nkpt,&
&         nspden, nspinor, nsppol, nsym, npsp, ntype, occopt,&
&         ecut_eff, rprimd(1:3,1:3)
else if(headform==40) then
  read(u) bantot, date, intxc, ixc, natom, ngfft(1:3), nkpt,&
&         nspden, nspinor, nsppol, nsym, npsp, ntype, occopt,&
&         ecut, ecutsm, ecut_eff, rprimd(1:3,1:3),&
&         tphysel, tsmear
else if(headform==41) then
  read(u) bantot, date, intxc, ixc, natom, ngfft(1:3), nkpt,&
&         nspden, nspinor, nsppol, nsym, npsp, ntype, occopt, pertcase,&
&         ecut, ecutsm, ecut_eff, qptn(1:3), rprimd(1:3,1:3),&
&         tphysel, tsmear
else if(headform==42) then
  read(u) bantot, date, intxc, ixc, natom, ngfft(1:3), nkpt,&
&         nspden, nspinor, nsppol, nsym, npsp, ntype, occopt,pertcase,&
&         ecut, ecutsm, ecut_eff, qptn(1:3), rprimd(1:3,1:3),&
&         stmbias, tphysel, tsmear
else if(headform==44 .or. headform==53 .or. headform==56) then
  read(u) bantot, date, intxc, ixc, natom, ngfft(1:3), nkpt,&
&         nspden, nspinor, nsppol, nsym, npsp, ntype, occopt,pertcase,&
&         usepaw,ecut, ecutdg, ecutsm, ecut_eff,qptn(1:3),rprimd(1:3,1:3),&
&         stmbias,tphysel, tsmear
else if(headform>=57) then
  read(u) bantot, date, intxc, ixc, natom, ngfft(1:3), nkpt,&
&         nspden, nspinor, nsppol, nsym, npsp, ntype, occopt,pertcase,&
&         usepaw,ecut, ecutdg, ecutsm, ecut_eff,qptn(1:3),rprimd(1:3,1:3),&
&         stmbias,tphysel, tsmear, usewvl
endif

year = date / 10000
month = mod(date,10000) / 100
day = mod(date,100)
write(*,'(" date (yyyy mm dd)",i5,i3,i3)') year, month, day

if(DEBUG) print *, 'crystallographic data:'
if(headform==23) then
  if(DEBUG) write(*,'("  a  ",f7.2/"  b  ",f7.2/"  c  ",f7.2)') acell(1:3)
endif
if(DEBUG) write(*,'("  r1 ",3f7.2/"  r2 ",3f7.2/"  r3 ",3f7.2)') rprimd
if(DEBUG) print *
      
if(DEBUG) print *, 'atomic composition:'
if(DEBUG) write(*,'("  number of types of atom ",i3)') ntype
if(headform==23) npsp=ntype

allocate(nband(nkpt*nsppol))
allocate(npwarr(nkpt))
allocate(symrel(3,3,nsym))
allocate(symafm(nsym))
allocate(istwfk(nkpt))
allocate(kpt(3,nkpt))
allocate(occ(bantot))
allocate(tnons(3,nsym))
allocate(znucltypat(ntype))
allocate(so_typat(ntype))
allocate(so_psp(npsp))
allocate(type(natom))
allocate(wtk(nkpt))
! 2nd record
if(headform==23 .or. headform==34) then
  read(u) nband(1:nkpt*nsppol), npwarr(1:nkpt), &
&         symrel(1:3,1:3,1:nsym), type(1:natom), &
&         istwfk(1:nkpt),& 
&         kpt(1:3,1:nkpt), occ(1:bantot), tnons(1:3,1:nsym),&
&         znucltypat(1:ntype)
else if(headform==40.or.headform==41.or.headform==42.or.headform==44) then
  read(u) istwfk(1:nkpt), nband(1:nkpt*nsppol), npwarr(1:nkpt), &
&         so_typat(1:ntype),&
&         symafm(1:nsym),symrel(1:3,1:3,1:nsym), type(1:natom),&
&         kpt(1:3,1:nkpt), occ(1:bantot), tnons(1:3,1:nsym),& 
&         znucltypat(1:ntype)
else if(headform>=53) then
  read(u) istwfk(1:nkpt), nband(1:nkpt*nsppol), npwarr(1:nkpt), &
&         so_psp(1:npsp),&
&         symafm(1:nsym),symrel(1:3,1:3,1:nsym), type(1:natom),&
&         kpt(1:3,1:nkpt), occ(1:bantot), tnons(1:3,1:nsym),& 
&         znucltypat(1:ntype), wtk(1:nkpt)
endif

if(DEBUG) write(*,'("  atomic numbers          ",10f7.2)') znucltypat
if(DEBUG) write(*,'("  number of atoms         ",i3)') natom
if(DEBUG) write(*,'("  atoms type              ",10i3)') type
if(DEBUG) print *
 
if(DEBUG) print *, 'symmetries of the system:'
if(DEBUG) write(*,'("  number of symmetry operations ",i2)') nsym
if(DEBUG) print *, ' symmetry operations (R t)'
do isymin=1,nsym,4
  isymend=isymin+3
  if(isymend>nsym) isymend=nsym
  do i=1,3
    if(DEBUG) write(*,'(4(3i3,f8.3,3x))') &
&   ((symrel(i,j,isym),j=1,3),tnons(i,isym),isym=isymin,isymend)
  enddo
  if(DEBUG) print *
enddo

           
if(DEBUG) print *, 'system informations:'
if(DEBUG) write(*,'("  number of k-points ",i5)') nkpt
if(DEBUG) print *, ' k-points [reciprocal lattice units]'
if(DEBUG) write(*,'(3(f10.6,1x))') ((kpt(i,ik),i=1,3),ik=1,nkpt)

if(DEBUG) print *, ' occupation '
i = 0
do ik = 1, nkpt
  if(DEBUG) write(*,'(1x,15f5.2)') (occ(j), j=i+1,i+nband(ik))
  i = i + nband(ik)
enddo
if(i/=bantot) print *, 'WARNING: bantot =', bantot
if(DEBUG) print *
      
if(npsp/=ntype) stop 'alchemical pseudo not programmed'
! (npsp lines, 1 for each pseudopotential ; npsp=ntype, except if alchemical pseudo-atoms)
allocate(title(132,npsp))
allocate(znuclpsp(npsp),zionpsp(npsp))
allocate(pspso(npsp),pspdat(npsp),pspcod(npsp),pspxc(npsp))
allocate(lmax(npsp),lloc(npsp),mmax(npsp),lmn_size(npsp))
lmax(:) = -1
lloc(:) = -1
mmax(:) = -1

! pseudopotential record
do i=1,npsp
  if(headform==23) then
    read(u) title(1:132,i),znuclpsp(i),zionpsp(i),pspso(i),pspdat(i),pspcod(i),pspxc(i),&
&           lmax(i),lloc(i),mmax(i)
  else if(headform==34.or.headform==40.or.headform==41.or.headform==42) then
    read(u) title(1:132,i),znuclpsp(i),zionpsp(i),pspso(i),pspdat(i),pspcod(i),pspxc(i)
  else if(headform>=44) then
    read(u) title(1:132,i),znuclpsp(i),zionpsp(i),pspso(i),pspdat(i),pspcod(i),&
&    pspxc(i),lmn_size(i)
  endif
enddo

if(DEBUG) print *, 'pseudopotentials:'
do i=1,npsp
  if(DEBUG) print *, ' atom type ', i
  if(DEBUG) write(*,'(3x,77a)') title(1:77,i)
  if(DEBUG) write(*,'("   atomic number        ",6x,f6.2)') znuclpsp(i)
  if(DEBUG) write(*,'("   pseudo atomic number ",6x,f6.2)') zionpsp(i)
  if(headform==23 .and. DEBUG) then
    select case(lmax(i))
      case(0)
          if(lloc(i)==0) then
            write(*,'("   local pseudopotential")')
          else
            write(*,'("   WARNING: lmax=",i2," and lloc=",i2, &
     &            " not coherent")') lmax(i), lloc(i)
          endif
      case(1)
        select case(lloc(i))
          case(0)
            write(*,'("   p nonlocality s reference")')
          case(1)
            write(*,'("   s nonlocality p reference")')
          case default
            write(*,'("   WARNING: lmax=",i2," and lloc=",i2, &
     &            " not coherent")') lmax(i), lloc(i)
        end select
      case(2)
        select case(lloc(i))
          case(0)
            write(*,'("   pd nonlocality s reference")')
          case(1)
            write(*,'("   sd nonlocality p reference")')
          case(2)
            write(*,'("   sp nonlocality d reference")')
          case default
            write(*,'("   WARNING: lmax=",i2," and lloc=",i2, &
     &            " not coherent")') lmax(i), lloc(i)
        end select
      case(3)
        select case(lloc(i))
          case(0)
            write(*,'("   pdf nonlocality s reference")')
          case(1)
            write(*,'("   sdf nonlocality p reference")')
          case(2)
            write(*,'("   spf nonlocality d reference")')
          case(3)
            write(*,'("   spd nonlocality f reference")')
          case default
            write(*,'("   WARNING: lmax=",i2," and lloc=",i2, &
     &            " not coherent")') lmax(i), lloc(i)
        end select
    end select
    if(DEBUG) write(*,'("   mesh dimension       ",4x,i5)') mmax(i)
  endif
  if(DEBUG) write(*,'("   pseudopotential code ",6x,i3)') pspcod(i)
  if(DEBUG) write(*,'("   xc functional        ",6x,i3)') pspxc(i)
  if(pspso(i)>1) then
     print *, '**ERROR: pspso value not programmed'
     print *, 'pspso = ', pspso(i)
     stop 'pspso > 1'
  endif
enddo

allocate(xred(3,natom))
! last record
read(u) residm,xred(1:3,1:natom),etotal,fermie 

if(usepaw==1) then
  print *, '**ERROR: PAW scheme not implemented into the code'
  print *, '         The results can be completely meaningless'
  print *
  read(u) ! skip record nsel
  ! read(u) ((nsel(ispden,iatom),ispden=1,nspden),iatom=1,natom) ! 44 <= headform < 56
  ! read(u) (nsel(iatom),iatom=1,natom),cplex                    ! headform = 56
  ! read(u) (nsel(iatom),iatom=1,natom),cplex,nspden             ! headform = 57

  read(u) ! skip record buffer
  ! read(u) ibuffer(:),buffer(:)                                 ! 44 <= headform <= 57
endif

if(DEBUG) print *
if(DEBUG) print *, 'convergence data:'
if(DEBUG) write(*,'("  effective cut-off",f7.2)') ecut_eff
if(DEBUG) write(*,'("  exchange-correlation functional ",i2)') ixc
if(DEBUG) write(*,'("  total energy ",f10.4)') etotal
if(DEBUG) write(*,'("  fermi energy ",f10.4)') fermie
if(DEBUG) write(*,'("  max residual ",e10.4)') residm
if(DEBUG) print *
      
if(DEBUG) print *, 'atomic positions [reduced coordinates]:'
do ia = 1, natom
  if(DEBUG) write(*,'(3f10.6)') xred(:,ia)
enddo
if(DEBUG) print *

crystr%rprimd(:,:) = rprimd(:,:)

if(nsym > 48) stop 'internal error, increase nsymx (48) in ggwdef.F'
symstr%nsym = nsym
do i = 1, symstr%nsym
  symstr%sym(i)%r(:,:) = symrel(:,:,i)
  symstr%sym(i)%t(:)   = tnons(:,i)
enddo
call setup_symmetry_structure(symstr)

if(ntype /= npsp) stop 'cannot treat ntype /= npsp'
chemstr%ntype = ntype
chemstr%natom = natom
chemstr%npsa  = npsp
allocate(chemstr%znucl(chemstr%ntype))
allocate(chemstr%type(chemstr%natom))
allocate(chemstr%xred(3,chemstr%natom))
allocate(chemstr%psa(chemstr%npsa))
chemstr%znucl(1:chemstr%ntype) = znucltypat(1:ntype)
chemstr%type(1:chemstr%natom) = type(1:natom)
chemstr%xred(:,1:chemstr%natom) = xred(:,1:natom)
do i = 1, chemstr%npsa
  chemstr%psa(i)%znucl = znuclpsp(i)
  chemstr%psa(i)%zion  = zionpsp(i)
  chemstr%psa(i)%lmax  = lmax(i)
  chemstr%psa(i)%lloc  = lloc(i)
  chemstr%psa(i)%mmax   = mmax(i)
  chemstr%psa(i)%pspso  = pspso(i)
  chemstr%psa(i)%pspdat = pspdat(i)
  chemstr%psa(i)%pspcod = pspcod(i)
  chemstr%psa(i)%pspxc  = pspxc(i)
  chemstr%psa(i)%title(:)  = title(:,i)
enddo
chemstr%nel = 0
do ia = 1, chemstr%natom
  it = chemstr%type(ia)
  nel = chemstr%znucl(it)  ! if it's not a pseudoatom, add all electron
  do ipa = 1, chemstr%npsa
    if(chemstr%psa(ipa)%znucl == chemstr%znucl(it)) nel = chemstr%psa(ipa)%zion
  enddo
  chemstr%nel = chemstr%nel + nel
enddo

! meshstr%nrr ?

foustr%nk = nkpt
allocate(foustr%k(3,foustr%nk))
foustr%k(:,1:foustr%nk) = kpt(:,1:nkpt)

elecstr%nk = nkpt
elecstr%nb = maxval(nband)
allocate(elecstr%oc(elecstr%nb,elecstr%nk))
elecstr%oc(:,:) = zero
do ik = 1, nkpt
  do ib = 1, nband(ik)
    i = (ik-1)*nband(ik) + ib
    elecstr%oc(ib,ik) = occ(i)
  enddo
enddo
elecstr%fermie = fermie
elecstr%etotal = etotal
elecstr%residm = residm
elecstr%ecut_eff = ecut_eff
elecstr%xc = ixc

deallocate(nband,npwarr,symrel,symafm,istwfk,kpt,occ,tnons)
deallocate(znucltypat,so_typat,so_psp,type,xred)
deallocate(title,znuclpsp,zionpsp,pspso,pspdat,pspcod,pspxc)
deallocate(lmax,lloc,mmax)

return
end subroutine


!=======================================================================
subroutine prhdr(crystr,symstr,chemstr,meshstr,foustr,elecstr)
!=======================================================================
! print fundamental data
use ggwdef
implicit none
type(crystal) crystr
type(symmetry) symstr
type(chemistry) chemstr
type(mesh) meshstr
type(transposed_space) foustr
type(electronic_structure) elecstr
character (len=*), parameter :: orb = 'spdfghij'
integer i, ia, ik, j, isym, isymin, isymend


print *, 'crystallographic data:'
write(*,'("  r1 ",3f7.2/"  r2 ",3f7.2/"  r3 ",3f7.2)') crystr%rprimd
print *
      
print *, 'atomic composition:'
write(*,'("  number of types of atom ",i3)') chemstr%ntype

write(*,'("  atomic numbers          ",10f7.2)') chemstr%znucl(:)
write(*,'("  number of atoms         ",i3)') chemstr%natom
write(*,'("  atoms type              ",10i3)') chemstr%type(:)
write(*,'("  number of electrons   ",i5)') chemstr%nel
print *
print *, ' atom type    z  psz   position [reduced coordinates]'
do ia = 1, chemstr%natom
  write(*,'(i6,i5,i5,i5,2x,3f10.6)') ia,chemstr%type(ia), &
&   int(chemstr%znucl(chemstr%type(ia))),&
&   int(chemstr%psa(chemstr%type(ia))%zion), chemstr%xred(:,ia)
enddo
print *

print *, 'pseudopotentials:'
do i=1,chemstr%npsa
  print *, ' atom type ', i
  write(*,'(3x,77a)') chemstr%psa(i)%title(1:77)
  write(*,'("   atomic number        ",6x,f6.2)') chemstr%psa(i)%znucl
  write(*,'("   pseudo atomic number ",6x,f6.2)') chemstr%psa(i)%zion
  if(chemstr%psa(i)%lloc > chemstr%psa(i)%lmax) write(*,'("   WARNING: lmax=",i2," and lloc=",i2, &
     &            " not coherent")') chemstr%psa(i)%lmax, chemstr%psa(i)%lloc
  
    select case(chemstr%psa(i)%lmax)
      case(0)
          if(chemstr%psa(i)%lloc==0) then
            write(*,'("   local pseudopotential")')
          else
            write(*,'("   WARNING: lmax=",i2," and lloc=",i2, &
     &            " not coherent")') chemstr%psa(i)%lmax, chemstr%psa(i)%lloc
          endif
      case(1)
        select case(chemstr%psa(i)%lloc)
          case(0)
            write(*,'("   p nonlocality s reference")')
          case(1)
            write(*,'("   s nonlocality p reference")')
          case default
            write(*,'("   WARNING: lmax=",i2," and lloc=",i2, &
     &            " not coherent")') chemstr%psa(i)%lmax, chemstr%psa(i)%lloc
        end select
      case(2)
        select case(chemstr%psa(i)%lloc)
          case(0)
            write(*,'("   pd nonlocality s reference")')
          case(1)
            write(*,'("   sd nonlocality p reference")')
          case(2)
            write(*,'("   sp nonlocality d reference")')
          case default
            write(*,'("   WARNING: lmax=",i2," and lloc=",i2, &
     &            " not coherent")') chemstr%psa(i)%lmax, chemstr%psa(i)%lloc
        end select
      case(3)
        select case(chemstr%psa(i)%lloc)
          case(0)
            write(*,'("   pdf nonlocality s reference")')
          case(1)
            write(*,'("   sdf nonlocality p reference")')
          case(2)
            write(*,'("   spf nonlocality d reference")')
          case(3)
            write(*,'("   spd nonlocality f reference")')
          case default
            write(*,'("   WARNING: lmax=",i2," and lloc=",i2, &
     &            " not coherent")') chemstr%psa(i)%lmax, chemstr%psa(i)%lloc
        end select
    end select

  write(*,'("   mesh dimension       ",4x,i5)') chemstr%psa(i)%mmax
  write(*,'("   pseudopotential code ",6x,i3)') chemstr%psa(i)%pspcod
  write(*,'("   xc functional        ",6x,i3)') chemstr%psa(i)%pspxc
  if(chemstr%psa(i)%pspso>1) then
     print *, '**ERROR: pspso value not programmed'
     print *, 'pspso = ', chemstr%psa(i)%pspso
     stop 'pspso > 1'
  endif
enddo

print *
 
print *, 'symmetries of the system:'
write(*,'("  number of symmetry operations ",i2)') symstr%nsym
print *, ' symmetry operations (R t)'
do isymin=1,symstr%nsym,4
  isymend=isymin+3
  if(isymend>symstr%nsym) isymend=symstr%nsym
  do i=1,3
    write(*,'(4(3i3,f8.3,3x))') &
&   ((symstr%sym(isym)%r(i,j),j=1,3),symstr%sym(isym)%t(i),isym=isymin,isymend)
  enddo
  print *
enddo

           
print *, 'system informations:'
write(*,'("  number of k-points ",i5)') foustr%nk
print *, ' k-points [reduced coordinates]:'
write(*,'(3(1x,f10.6))') ((foustr%k(i,ik),i=1,3),ik=1,foustr%nk)
print *

print *, 'convergence data:'
write(*,'("  exchange-correlation functional ",i2)') elecstr%xc
write(*,'("  effective cut-off ",f6.2)') elecstr%ecut_eff
write(*,'("  total energy ",f10.4)') elecstr%etotal
write(*,'("  fermi energy ",f10.4)') elecstr%fermie
write(*,'("  max residual ",e10.4)') elecstr%residm
print *
      
return
end subroutine


!=======================================================================
integer function fileform(file,l)
!=======================================================================
! check file type
implicit none
integer l
character (len=l) :: file
integer, parameter :: u=10
integer status, status1, status2, status3
character (len=6) :: codvsn
integer headform, fform
character (len=80) :: title
integer i1,i2,i3,i4,i5,i6,i7,i8,i9,i10,i11,i12,i13,i14,i15,i16
logical, parameter :: DEBUG = .false.

if(DEBUG) print *, 'checking Kohn-Sham Structure input file ', file

if(DEBUG) print *, 'trying kss ABINIT form'
open(u,status='unknown',form='unformatted',file=file)
read(u,iostat=status) codvsn, headform, fform
close(u)
fileform = fform
if(fform==502 .and. status==0) then
  if(DEBUG) print *, 'kss ABINIT double precision form'
  return
else if(fform==602 .and. status==0) then
  if(DEBUG) print *, 'kss ABINIT single precision form'
  return
endif
  
if(DEBUG) print *, 'non kss form, status = ', status
if(DEBUG) print *, 'codvsn, headform, fform = ', codvsn, headform, fform

if(DEBUG) print *, 'trying sta and qplda forms'
open(u,status='unknown',form='unformatted',file=file)
read(u,iostat=status1) title
if(DEBUG) print *, title
read(u,iostat=status2) title
if(DEBUG) print *, title
read(u,iostat=status3) i1,i2,i3,i4,i5,i6,i7,i8,i9,i10,i11,i12,i13,i14,i15,i16
close(u)
! flags used so far:
! i1 = 0 if QPLDA file is formatted, 1 if unformatted
! i2 = 0 if RWG format, 1 if BF format, 2 STA format
! i3 = 1 if non-symmorphic operations (+vectors) included, otherwise 0
if(DEBUG) print *, i1,i2,i3
if(DEBUG) print *, status1, status2, status3
if(status1==0.and.status2==0.and.status3==0) then
  if(i2==2) then
    fileform = 2
    if(DEBUG) print *, 'sta double precision form'
    return
  else if(i2==0.or.i2==1) then
    fileform = 1
    if(DEBUG) print *, 'qplda single precision form'
    return
  endif
endif

print *, 'unrecognized file format'
print *
fileform = -1
return
end function






!     ==================================================================
      subroutine rdlda(qpldafile,kl,nopx,nbx,nkx,ngx,nwx,nop,nb,nk,ng, &
     &                 tit,i1,i2,i3,a1,a2,a3,op,gvec,kibz,en,occ,wf)
!     ==================================================================
!     read QPLDA type file
      implicit none
      integer kl
      character (len=kl) :: qpldafile
      integer, parameter :: iunit = 10
      integer nopx, nbx, nkx, ngx, nwx
      integer nop, nb, nk, ng
      character (len=80) :: tit(2)
      integer i1,i2,i3,i4,i5,i6,i7,i8,i9,i10,i11,i12,i13,i14,i15,i16
      real a1(3), a2(3), a3(3)
      real op(3,3,nopx)
      integer gvec(3,ngx)
      real kibz(3,nkx)
      real en(nkx,nbx)
      real occ(nkx,nbx)
      complex wf(nwx,nbx,nkx)

      real b1(3), b2(3), b3(3)
      real ucvol, bzvol, PI

!     nopx: maximum number of symmetry operations, i.e. dimension of op
!     nbx: maximum number of bands, i.e. dimension of en, occ, wf
!     nkx: maximum number of k-points in IBZ, i.e. dim. of kvec, en, occ, wf
!     ngx: maximum number of G-vectors in calculation, i.e. dim. of gvec, wf
!     nwx: maximum number of plane-waves to be used for wave-functions
!     nop: number of symmetry operations contained in file
!     nb: number of bands contained in file
!     nk: number of k-points in IBZ contained in file
!     ng: number of G-vectors in calculation contained in file
!     tit: 2*80-character description of LDA input file
!     i*: flags
!     a1, a2, a3: real-space lattice vectors in au
!     b1, b2, b3: reciprocal-space lattice vectors in au**-1
!     ucvol: unit cell volume in au**3
!     bzvol: reciprocal cell volume
!     op: symmetry operation matrices
!     gvec: RL vectors G
!     kibz: k-points in IBZ
!     en: LDA energies (Hartrees)
!     occ: occupations of bands (2.0=full)
!     wf: LDA wave functions (G) in plane wave order [Norm:|vector|=1]


      integer nopr, nbr, nkr, ngr, nwr
      integer nbw

!     n*r: number of * read from file
!     if nb < nbx then nbr = nb else nbr = nbx
!     if ng < ngx then ngr = ng else ngr = ngx
!     if ng < nwx then nwr = ng else nwr = nwx
!     if nop < nopx then nopr = nop else nopr = nopx
!     ** if nk < nkx then nkr = nk else ERROR **:
!      due to the structure of LDA file with regard of energies and
!      occupations, (all energies/occupations in one record) it is
!      impossible to read nkx less than nk elements; implicit do in
!      those sections is not synctactically correct and explicit do
!      doesn't go as look for EOR at each loop. 
!     check the return value of nop, nb, ng to see how many elements
!     were loaded


      integer ib, ik, ig, i, j, k, ibp
      integer isymin, isymend, isym
      real einf, esup, cinf, csup, sum
      integer idum
      real dum
      complex cdum


      logical DEBUG
      parameter(DEBUG=.FALSE.)
      logical VERBOSE
      parameter(VERBOSE=.TRUE.)
      logical FORM
      
      real, parameter :: HARTREE = 27.2116
      

!     i1=0 -> file formatted -> FORM=.TRUE.
!     i1!=0 -> file unformatted -> FORM=.FALSE.
      if (i1.eq.0) then
        FORM = .TRUE.
      else
        FORM = .FALSE.
      endif


      PI = 4.0 * atan(1.0)


      if (FORM) then
        write (*,*) 'opening LDA input file formatted unit ',iunit
        open(iunit,status='unknown',form='FORMATTED',file=qpldafile)
      else
        write (*,*) 'opening LDA input file unformatted unit ',iunit
        open(iunit,status='unknown',form='UNFORMATTED',file=qpldafile)
!  ****** TEMP Add recl for FPS unformatted file
!       open (iunit, status='unknown',form='UNFORMATTED',RECL=500000)
!  ******
      endif


      write (*,*) 'reading title of file:'
      if (FORM) then
        read (iunit,10) tit(1), tit(2)
10      format (A80/A80)
      else
        read (iunit) tit(1)
        read (iunit) tit(2)
      endif

      write (6,2974) tit(1), tit(2)
2974  format(A80/A80)
      write (*,*)


!     Flags used so far:
!     i1 = 0 if QPLDA file is formatted, 1 if unformatted
!     i2 = 0 if RWG format, 1 if BF format
!     i3 = 1 if non-symmorphic operations (+vectors) included, otherwise 0

      write (*,*) 'reading flags'

      if (FORM) then
        read (iunit,20) i1,i2,i3,i4,i5,i6,i7,i8,i9,i10,&
     &               i11,i12,i13,i14,i15,i16
20      format (16I5)
      else
        read (iunit) i1,i2,i3,i4,i5,i6,i7,i8,i9,i10,&
     &               i11,i12,i13,i14,i15,i16
      endif

!     check that flag i1 is consistent with value of FORM assumed
      if ((FORM).and.(i1.ne.0)) then
        write (*,*) &
     &  'ERROR rdlda 1: input file unformatted read as formatted.'
        stop
      endif
      if ((.not.FORM).and.(i1.ne.1)) then
        write (*,*) &
     &  'ERROR rdlda 2: input file formatted read as unformatted.'
        stop
      endif

      if (DEBUG) write (6,2755) i1,i2,i3,i4,i5,i6,i7,i8,i9,i10, &
     &               i11,i12,i13,i14,i15,i16
2755  format (16I4)
      write (*,*)


      write (*,*) 'reading real-space lattice vectors'

      if (FORM) then
        read (iunit,30) a1, a2, a3
30      format (3E26.18)
      else
        read (iunit) a1, a2, a3
      endif

      if (VERBOSE) then
!       calculate Brillouin zone and Unit cell volumes
        ucvol = a1(1)* (a2(2)*a3(3)-a2(3)*a3(2)) &
     &        + a1(2)* (a2(3)*a3(1)-a2(1)*a3(3)) &
     &        + a1(3)* (a2(1)*a3(2)-a2(2)*a3(1))
        bzvol = 8*PI*PI*PI / ucvol

!       calculate reciprocal-space lattice vectors b1-b3
        b1(1) = 2.0 * PI * (a2(2)*a3(3)-a2(3)*a3(2)) / ucvol
        b1(2) = 2.0 * PI * (a2(3)*a3(1)-a2(1)*a3(3)) / ucvol
        b1(3) = 2.0 * PI * (a2(1)*a3(2)-a2(2)*a3(1)) / ucvol
        b2(1) = 2.0 * PI * (a3(2)*a1(3)-a3(3)*a1(2)) / ucvol
        b2(2) = 2.0 * PI * (a3(3)*a1(1)-a3(1)*a1(3)) / ucvol
        b2(3) = 2.0 * PI * (a3(1)*a1(2)-a3(2)*a1(1)) / ucvol
        b3(1) = 2.0 * PI * (a1(2)*a2(3)-a1(3)*a2(2)) / ucvol
        b3(2) = 2.0 * PI * (a1(3)*a2(1)-a1(1)*a2(3)) / ucvol
        b3(3) = 2.0 * PI * (a1(1)*a2(2)-a1(2)*a2(1)) / ucvol

        write (6,8723) a1, a2, a3
8723    format (' real-space lattice vectors in cartesians [au]:' &
     &          /('     ',3F12.6))
        write (6,1022) b1, b2, b3
1022    format (/' calculated reciprocal lattice vectors', &
     &          ' in cartesians [au]:'/ &
     &          ('     ',3F12.6) )
        write (6,1021) bzvol, ucvol
1021    format (/' brillouin zone volume = ', F12.6, ' [au]'/ &
     &           ' unit cell volume      = ', F12.6, ' [au]')
      endif
      write (*,*) 


!     read symmetry operations (real-space-lattice units) and CONVERT
!     to reciprocal-lattice units   [nop+1 lines]
!     [Note: if R is the matrix in lattice units, then the corresponding
!     matrix in RL units is transpose(R**-1).  But since the set of
!     operations contains inverse pairs, we only need to tranpose
!     the matrices.  Thus op(J,I) rather than op(I,J).]

      write (*,*) 'reading symmetry operations'

      if (FORM) then
        read (iunit,40) nop
40      format (I12)
      else
        read (iunit) nop
      endif

      write (*,*) 'number of translationless symmetry operations', nop

      if (nop.le.nopx) then
        nopr = nop
      else
        write (*,*) 'WARNING rdlda 3: reading only a number of ', &
     &  'symmetry operations ', nopx
        nopr = nopx
      endif

      if (nop.le.nopx) then
        if (FORM) then
          read (iunit,50) (((op(j,i,k),i=1,3),j=1,3),k=1,nop)
50        format (3E26.18)
        else
          read (iunit) (((op(j,i,k),i=1,3),j=1,3),k=1,nop)
        endif
      else
        if (FORM) then
          read (iunit,50) (((op(j,i,k),i=1,3),j=1,3),k=1,nopx), &
     &                      (((dum,i=1,3),j=1,3),k=nopx+1,nop)
        else
          read (iunit) (((op(j,i,k),i=1,3),j=1,3),k=1,nopx), &
     &                 (((dum,i=1,3),j=1,3),k=nopx+1,nop)
        endif
      endif


      if (VERBOSE) then
        write(*,*) 'symmetry operations [reciprocal lattice units]:'
        do isymin=1,nopr,3
          isymend=isymin+2
          if(isymend.gt.nopr) isymend=nopr
          do i=1,3
            write(6,2995) ((op(i,j,isym),j=1,3),isym=isymin,isymend)
2995        format(2(3f8.4,4x),3f8.4)
          enddo
          write(*,*)
        enddo
        write(*,*)
      endif


      if (I3.eq.1) then
!       read (iunit,1020) ((voffset(i,j),i=1,3),j=1,nop)
        write (*,*) 'ERROR rdlda 4: program is not able to handle &
     & non-symorphic translation vectors; run CNVNSY first or check I3'
        stop
      endif


      write (*,*) 'reading reciprocal lattice vectors'
!     ie in the basis of the reciprocal lattice basis vectors

      if (FORM) then
        read (iunit,60) ng
60      format (I12)
      else
        read (iunit) ng
      endif

      write (*,*) 'number of G vectors found, i.e. of plane waves ', ng

      if (ng.le.ngx) then
        ngr = ng
      else
        ngr = ngx
      endif
      write (*,*) 'reading a number of G vectors ', ngr

      if (ng.le.ngx) then
        if (FORM) then
          read (iunit,70) ((gvec(i,ig),i=1,3),ig=1,ng)
70        format (3I5)
        else
          read (iunit) ((gvec(i,ig),i=1,3),ig=1,ng)
        endif
      else
        if (FORM) then
          read (iunit,70) ((gvec(i,ig),i=1,3),ig=1,ngx), &
     &                      ((idum,i=1,3),ig=ngx+1,ng)
        else
          read (iunit) ((gvec(i,ig),i=1,3),ig=1,ngx), &
     &                 ((idum,i=1,3),ig=ngx+1,ng)
        endif
      endif

      if (VERBOSE) write (6,3995) (ig,(gvec(i,ig),i=1,3),ig=1,ngr)
3995  format (' G vectors [reciprocal lattice units]:'/ &
     &           (5(I5,':',3I3)))
      write (*,*)


      write (*,*) 'reading k-points'

      if (FORM) then
        read (iunit,80) nk
80      format (I12)
      else
        read (iunit) nk
      endif

      write (*,*) 'number of k-points ', nk

      if (nk.le.nkx) then
        nkr = nk
      else
        nkr = nkx
        write (*,*)
        write (*,*) '** ERROR rdlda 5: ', &
     &  'k dimension of vectors initialized at ',nkx
        write (*,*) '                  increase the dimension at ',nk
        stop
      endif

      if (nk.le.nkx) then
        if (FORM) then
          read (iunit,90) ((kibz(i,ik),i=1,3),ik=1,nk)
90        format (3E26.18)
        else
          read (iunit) ((kibz(i,ik),i=1,3),ik=1,nk)
        endif
      else
        if (FORM) then
          read (iunit,90) ((kibz(i,ik),i=1,3),ik=1,nkx), &
     &                      ((dum,i=1,3),ik=nkx+1,nk)
        else
          read (iunit) ((kibz(i,ik),i=1,3),ik=1,nkx), &
     &                 ((dum,i=1,3),ik=nkx+1,nk)
        endif
      endif

      if (VERBOSE) write (6,2862) (ik,(kibz(i,ik),i=1,3),ik=1,nkr)
2862  format (' k-points [reciprocal lattice units]:'/ &
     &          (I5,3F12.6))
      write (*,*)


      write (*,*) 'reading energies'

      if (FORM) then
        read (iunit,1110) nb
1110    format (I12)
      else
        read (iunit) nb
      endif

      write (*,*) 'number of bands found ', nb

      if (nb.le.nbx) then
        nbr = nb
      else
        nbr = nbx
      endif
      write (*,*) 'reading a number of bands ', nbr

      if (nb.le.nbx) then
        if (nk.le.nkx) then
          if (FORM) then
            read (iunit,91) ((en(ik,ib),ik=1,nk),ib=1,nb)
91          format (3E26.18)
          else
            read (iunit) ((en(ik,ib),ik=1,nk),ib=1,nb)
          endif
        else
          if (FORM) then
            do ib = 1, nb
              read (iunit,91) (en(ik,ib),ik=1,nkx),(dum,ik=nkx+1,nk)
            enddo
          else
            do ib = 1, nb
              read (iunit) (en(ik,ib),ik=1,nkx),(dum,ik=nkx+1,nk)
            enddo
          endif
        endif
      else
        if (nk.le.nkx) then
          if (FORM) then
            read (iunit,91) ((en(ik,ib),ik=1,nk),ib=1,nbx), &
     &                        ((dum,ik=1,nk),ib=nbx+1,nb)
          else
            read (iunit) ((en(ik,ib),ik=1,nk),ib=1,nbx), &
     &                   ((dum,ik=1,nk),ib=nbx+1,nb)
          endif
        else
          if (FORM) then
            do ib = 1, nbx
              read (iunit,91) (en(ik,ib),ik=1,nkx), &
     &                          (dum,ik=nkx+1,nk)
            enddo
            do ib = nbx+1, nb
              read (iunit,91) (dum,ik=1,nk)
            enddo
          else
            do ib = 1, nbx
              read (iunit) (en(ik,ib),ik=1,nkx), (dum,ik=nkx+1,nk)
            enddo
            do ib = nbx+1, nb
              read (iunit) (dum,ik=1,nk)
            enddo
          endif
        endif
      endif

      write (*,*)


      write (*,*) 'reading numbers of occupation'
      if (nb.le.nbx) then
        if (nk.le.nkx) then
          if (FORM) then
            read (iunit,92) ((occ(ik,ib),ik=1,nk),ib=1,nb)
92          format (3E26.18)
          else
            read (iunit) ((occ(ik,ib),ik=1,nk),ib=1,nb)
          endif
        else
          if (FORM) then
            do ib = 1, nb
              read (iunit,92) (occ(ik,ib),ik=1,nkx),(dum,ik=nkx+1,nk)
            enddo
          else
            do ib = 1, nb
              read (iunit) (occ(ik,ib),ik=1,nkx),(dum,ik=nkx+1,nk)
            enddo
          endif
        endif
      else
        if (nk.le.nkx) then
          if (FORM) then
            read (iunit,92) ((occ(ik,ib),ik=1,nk),ib=1,nbx), &
     &                        ((dum,ik=1,nk),ib=nbx+1,nb)
          else
            read (iunit) ((occ(ik,ib),ik=1,nk),ib=1,nbx), &
     &                   ((dum,ik=1,nk),ib=nbx+1,nb)
          endif
        else
          if (FORM) then
            do ib = 1, nbx
              read (iunit,92) (occ(ik,ib),ik=1,nkx), &
     &                          (dum,ik=nkx+1,nk)
            enddo
            do ib = nbx+1, nb
              read (iunit,92) (dum,ik=1,nk)
            enddo
          else
            do ib = 1, nbx
              read (iunit) (occ(ik,ib),ik=1,nkx), (dum,ik=nkx+1,nk)
            enddo
            do ib = nbx+1, nb
              read (iunit) (dum,ik=1,nk)
            enddo
          endif
        endif
      endif

      write (*,*)

      if (VERBOSE) then
        nbw = 15
        if(nbr.lt.nbw) nbw = nbr
        do ik = 1, nkr
!         write (6,3324) ik, (en(ik,ib),ib=1,nbw)
          write (6,3325) ik, (HARTREE*en(ik,ib),ib=1,nbw)
          write (6,3329) ik, (occ(ik,ib),ib=1,nbw)
        enddo
        write (*,*)
      endif
3324    format (/' eigenvalues for k-point',I5,' [Hartrees]:'/ &
     &            (5F12.6))
3325    format (/' eigenvalues for k-point',I5,' [eV]:'/ &
     &            (5F12.6))
3329    format (/' occupations for k-point',I5,':'/ &
     &            (5F12.6))


!     reading wavefunctions
!     (a(G), normalised to 1) and write to scratch file
!     test on the normalization and orthgonalization
      einf = 10e+24
      esup = -1.0
      cinf = 10e+24
      csup = -1.0

      write (*,*) 'reading wavefunctions'

      if (ng.le.nwx) then
        nwr = ng
      else
        nwr = nwx
      endif
      write(*,*) 'reading a number of plane-waves ', nwr

      do ik=1,nkr
        if (nb.le.nbx) then
          do ib = 1, nb
            if (ng.le.nwx) then
              if (FORM) then
                read (iunit,95) (wf(ig,ib,ik),ig=1,ng)
95              format (3E26.18)
              else
                read (iunit) (wf(ig,ib,ik),ig=1,ng)
              endif
            else
              if (FORM) then
                read (iunit,95) (wf(ig,ib,ik),ig=1,nwx), &
     &                            (cdum,ig=nwx+1,ng)
              else
                read (iunit) (wf(ig,ib,ik),ig=1,nwx), &
     &                          (cdum,ig=nwx+1,ng)
              endif
            endif
          enddo
        else
          do ib = 1, nbx
            if (ng.le.nwx) then
              if (FORM) then
                read (iunit,95) (wf(ig,ib,ik),ig=1,ng)
              else
                read (iunit) (wf(ig,ib,ik),ig=1,ng)
              endif
            else
              if (FORM) then
                read (iunit,95) (wf(ig,ib,ik),ig=1,nwx), &
     &                            (cdum,ig=nwx+1,ng)
              else
                read (iunit) (wf(ig,ib,ik),ig=1,nwx), &
     &                          (cdum,ig=nwx+1,ng)
              endif
            endif
          enddo
          do ib = nbx+1, nb
            if (FORM) then
              read (iunit,95) (cdum,ig=1,ng)
            else
              read (iunit) (cdum,ig=1,ng)
            endif
          enddo
        endif
      enddo


      if (DEBUG) then
        write (6,5692)  wf(1,1,1), wf(2,1,1)
        write (6,5692)  wf(1,2,1), wf(2,2,1)
5692    format (' wavefunction sample ',5F12.6)
      endif

      write(*,*) 'test on the normalization of the wavefunctions'

      do ik = 1, nkr
        do ib = 1, nbr
          sum = 0.0
          do ig = 1, nwr
            sum = sum + conjg(wf(ig,ib,ik))*wf(ig,ib,ik)
          enddo
          if(sum.lt.einf) einf = sum
          if(sum.gt.esup) esup = sum
        enddo
!       test on the orthogonalization of wavefunctions
        do ib = 1, nbr
          do ibp = ib+1, nbr
            cdum = 0.0
            do ig = 1, nwx
              cdum = cdum + conjg(wf(ig,ib,ik))*wf(ig,ibp,ik)
            enddo
            if(abs(cdum).lt.cinf) cinf = abs(cdum)
            if(abs(cdum).gt.csup) csup = abs(cdum)
          enddo
        enddo
      enddo

      write(6,5693) einf, esup
5693  format(' min sum_G |a(n,k,G)| = ',f9.6/ &
     &       ' max sum_G |a(n,k,G)| = ',f9.6)
      write(*,*) 'test on the orthogonalization of the wavefunctions'
      write(6,5694) cinf, csup
5694  format(' min sum_G a(n,k,G)* a(n",k,G) = ',f9.6/ &
     &       ' max sum_G a(n,k,G)* a(n",k,G) = ',f9.6)
      write (*,*) 


      write (*,*) 'closing file'
      close (iunit)
      write (*,*) 

      end



!     ==================================================================
      subroutine rdldaabinit(stafile,kl,nopx,nbx,nkx,ngx,nwx, &
     &                 tit,a1,a2,a3,op,gvec,kibz,en,occ,wfg)
!     ==================================================================
      use ggwdef
      implicit none
      integer kl
      character (len=kl) :: stafile
      integer, parameter :: iunit = 10
      integer nopx, nbx, nkx, ngx, nwx
      integer nop, nb, nk, ng
      character (len=80) :: tit(2)
      real a1(3), a2(3), a3(3)
      real op(3,3,nopx)
      integer gvec(3,ngx)
      real kibz(3,nkx)
      real en(nkx,nbx)
      real occ(nkx,nbx)
      complex wfg(nwx,nbx,nkx)

      real b1(3), b2(3), b3(3)
      real ucvol, bzvol

!     nopx: maximum number of symmetry operations, i.e. dimension of op
!     nbx: maximum number of bands, i.e. dimension of en, occ, wf
!     nkx: maximum number of k-points in IBZ, i.e. dim. of kvec, en, occ, wf
!     ngx: maximum number of G-vectors in calculation, i.e. dim. of gvec, wf
!     nwx: maximum number of plane-waves to be used for wave-functions
!     nop: number of symmetry operations contained in file
!     nb: number of bands contained in file
!     nk: number of k-points in IBZ contained in file
!     ng: number of G-vectors in calculation contained in file
!     tit: 2*80-character description of LDA input file
!     i*: flags
!     a1, a2, a3: real-space lattice vectors in au
!     b1, b2, b3: reciprocal-space lattice vectors in au**-1
!     ucvol: unit cell volume in au**3
!     bzvol: reciprocal cell volume
!     op: symmetry operation matrices
!     gvec: RL vectors G
!     kibz: k-points in IBZ
!     en: LDA energies (Hartrees)
!     occ: occupations of bands (2.0=full)
!     wf: LDA wave functions (G) in plane wave order [Norm:|vector|=1]


      integer nopr, nbr, nkr, ngr

!     n*r: number of * read from file
!     if nb < nbx then nbr = nb else nbr = nbx
!     if ng < ngx then ngr = ng else ngr = ngx
!     if ng < nwx then nwr = ng else nwr = nwx
!     if nop < nopx then nopr = nop else nopr = nopx
!     ** if nk < nkx then nkr = nk else ERROR **:
!      due to the structure of LDA file with regard of energies and
!      occupations, (all energies/occupations in one record) it is
!      impossible to read nkx less than nk elements; implicit do in
!      those sections is not synctactically correct and explicit do
!      doesn't go as look for EOR at each loop. 
!     check the return value of nop, nb, ng to see how many elements
!     were loaded


      integer ib, ik, ig, i, j, k, ibp
      integer isymin, isymend, isym
      real einf, esup, cinf, csup, sum
      complex cdum


      logical DEBUG
      parameter(DEBUG=.true.)
      logical VERBOSE
      parameter(VERBOSE=.false.)
      
      double precision :: a1d(3), a2d(3), a3d(3)
      integer symrel(3,3,nopx)
      double precision :: kpt(3,nkx)
      double precision :: fermie
      double precision :: end(nkx,nbx)
      double precision :: occd(nkx,nbx)
      double precision :: wfd1(nwx), wfd2(nwx)
      
      
      print *, 'opening qplda input file (abinit unformatted form)'
      open(iunit,status='unknown',form='unformatted',file=stafile)

      print *, 'title:'
      read(iunit) tit(1)
      read(iunit) tit(2)
      write(6,2974) tit(1), tit(2)
2974  format(A80/A80)
      print *

!     skipping flags
      read(iunit)

      print *, 'real-space lattice primitive vectors, ', &
     &         'cartesian coordinates a.u. [Bohr]:'

      read(iunit) a1d, a2d, a3d
      a1(:) = a1d(:)
      a2(:) = a2d(:)
      a3(:) = a3d(:)
      write(*,1026) a1, a2, a3
1026  format(' a1 = ',3f7.2/' a2 = ',3f7.2/' a3 = ',3f7.2)

      if(VERBOSE) then
!       calculate Brillouin zone and Unit cell volumes
        ucvol = a1(1)* (a2(2)*a3(3)-a2(3)*a3(2)) &
     &        + a1(2)* (a2(3)*a3(1)-a2(1)*a3(3)) &
     &        + a1(3)* (a2(1)*a3(2)-a2(2)*a3(1))
        bzvol = 8*PI*PI*PI / ucvol

!       calculate reciprocal-space lattice vectors b1-b3
        b1(1) = 2.0 * PI * (a2(2)*a3(3)-a2(3)*a3(2)) / ucvol
        b1(2) = 2.0 * PI * (a2(3)*a3(1)-a2(1)*a3(3)) / ucvol
        b1(3) = 2.0 * PI * (a2(1)*a3(2)-a2(2)*a3(1)) / ucvol
        b2(1) = 2.0 * PI * (a3(2)*a1(3)-a3(3)*a1(2)) / ucvol
        b2(2) = 2.0 * PI * (a3(3)*a1(1)-a3(1)*a1(3)) / ucvol
        b2(3) = 2.0 * PI * (a3(1)*a1(2)-a3(2)*a1(1)) / ucvol
        b3(1) = 2.0 * PI * (a1(2)*a2(3)-a1(3)*a2(2)) / ucvol
        b3(2) = 2.0 * PI * (a1(3)*a2(1)-a1(1)*a2(3)) / ucvol
        b3(3) = 2.0 * PI * (a1(1)*a2(2)-a1(2)*a2(1)) / ucvol

        write(6,1022) b1, b2, b3
1022    format(/' calculated reciprocal lattice vectors', &
     &          ' in cartesians [au]:'/ &
     &          ('     ',3F12.6) )
        write(6,1021) bzvol, ucvol
1021    format(/' brillouin zone volume = ', F12.6, ' [au]'/ &
     &           ' unit cell volume      = ', F12.6, ' [au]')
      endif
      print *


!     reading symmetry operations
!     read symmetry operations (real-space-lattice units) and CONVERT
!     to reciprocal-lattice units   [nop+1 lines]
!     [Note: if R is the matrix in lattice units, then the corresponding
!     matrix in RL units is transpose(R**-1).  But since the set of
!     operations contains inverse pairs, we only need to tranpose
!     the matrices.  Thus op(J,I) rather than op(I,J).]

      print *, 'symmetry operations'

      read(iunit) nop

      print *, 'number of symmetry operations', nop

      if(nop.le.nopx) then
        nopr = nop
      else
        print *, 'WARNING rdlda 3: reading only a number of ', &
     &  'symmetry operations ', nopx
        nopr = nopx
      endif

      read(iunit) (((symrel(j,i,k),i=1,3),j=1,3),k=1,nopx)
      op(:,:,:) = symrel(:,:,:)
      
      print *, 'symmetry operations [reciprocal lattice units]:'
      do isymin=1,nopr,8
        isymend=isymin+7
        if(isymend.gt.nopr) isymend=nopr
        do i=1,3
          write(6,2995) ((symrel(i,j,isym),j=1,3),isym=isymin,isymend)
2995      format(7(3i3,1x),3i3)
        enddo
        print *
      enddo



!     reading reciprocal lattice vectors (in reciprocal lattice units)
!     ie in the basis of the reciprocal lattice basis vectors

      print *, 'reciprocal lattice vectors'

      read(iunit) ng

      print *, 'number of G vectors found, i.e. of plane waves ', ng

      if(ng.le.ngx) then
        ngr = ng
      else
        ngr = ngx
      endif
      print *, 'reading a number of G vectors ', ngr

      read(iunit) ((gvec(i,ig),i=1,3),ig=1,ngx)

      write(6,3995) (ig,(gvec(i,ig),i=1,3),ig=1,ngr)
3995  format(' G vectors [reciprocal lattice units]:'/ &
     &           (5(I5,':',3I3)))
      print *


!     reading k-points in r.l. units

      print *, 'k-points'

      read(iunit) nk

      write(*,*) 'number of k-points ', nk

      if(nk.le.nkx) then
        nkr = nk
      else
        nkr = nkx
        write (*,*)
        write (*,*) '** ERROR rdlda 5: ', &
     &  'k dimension of vectors initialized at ',nkx
        write (*,*) '                  increase the dimension at ',nk
        stop
      endif
      if(nk/=nkx) stop 'nk read not equal nk input'

      read(iunit) ((kpt(i,ik),i=1,3),ik=1,nkx)
      kibz(:,:) = kpt(:,:)
      
      write(6,2862) (ik,(kibz(i,ik),i=1,3),ik=1,nkr)
2862  format(' k-points [reciprocal lattice units]:'/ &
     &          (I5,3F12.6))
      print *


!     reading energies [Hartrees] (in ascending order)

      print *, 'reading energies'

      read(iunit) nb

      print *, 'number of eigenvalues found ', nb

      if(nb.le.nbx) then
        nbr = nb
      else
        nbr = nbx
      endif
      print *, 'reading a number of eigenvalues ', nbr
      
      read(iunit) fermie

      do ik = 1, nkx
        read(iunit) (end(ik,ib),ib=1,nbx)
        read(iunit) (occd(ik,ib),ib=1,nbx)
        do ib = 1, nb
          read(iunit) (wfd1(ig),wfd2(ig),ig=1,nwx)
          if(ib<=nbx) then
            wfg(1:nwx,ib,ik) = cmplx(wfd1(:),wfd2(:))
          endif
        enddo
      enddo
      en(:,:) = end(:,:)
      occ(:,:) = occd(:,:)
      
      
      print *, ' k       eigenvalues [eV]'
      do ik = 1, nkr
        write(6,3325) ik, (HARTREE*en(ik,ib),ib=1,nbr)
3325    format(i3,7x,10f7.2/50(10x,10f7.2/))
      enddo
      print *

      if(DEBUG) then
      print *, ' k       eigenvalues [Hartree]'
      do ik = 1, nkr
        write(6,3326) ik, (en(ik,ib),ib=1,nbr)
3326    format(i3,7x,10f7.4/50(10x,10f7.4/))
      enddo
      print *
      endif

      write(*,'("Fermi energy ",f7.2)') fermie
      
      print *, ' k       occupation'
      do ik = 1, nkr
        write(6,3325) ik, (occ(ik,ib),ib=1,nbr)
      enddo
      print *


!     reading wavefunctions
!     (a(G), normalised to 1) and write to scratch file
!     test on the normalization and orthgonalization
      einf = 10e+24
      esup = -1.0
      cinf = 10e+24
      csup = -1.0

      print *, 'wavefunctions'


      do ik = 1, nkx
!       test on the normalization of wavefunctions
        do ib = 1, nbx
          sum = 0.0
          do ig = 1, nwx
            sum = sum + conjg(wfg(ig,ib,ik))*wfg(ig,ib,ik)
          enddo
          if(sum.lt.einf) einf = sum
          if(sum.gt.esup) esup = sum
        enddo
!       test on the orthogonalization of wavefunctions
        do ib = 1, nbx
          do ibp = ib+1, nbx
            cdum = 0.0
            do ig = 1, nwx
              cdum = cdum + conjg(wfg(ig,ib,ik))*wfg(ig,ibp,ik)
            enddo
            if(abs(cdum).lt.cinf) cinf = abs(cdum)
            if(abs(cdum).gt.csup) csup = abs(cdum)
          enddo
        enddo
      enddo


      print *, 'test on the normalization of the wavefunctions'
      write(6,5693) einf, esup
5693  format(' min sum_G |a(n,k,G)| = ',f9.6/ &
     &       ' max sum_G |a(n,k,G)| = ',f9.6)
      print *, 'test on the orthogonalization of the wavefunctions'
      write(6,5694) cinf, csup
5694  format(' min sum_G a(n,k,G)* a(n",k,G) = ',f9.6/ &
     &       ' max sum_G a(n,k,G)* a(n",k,G) = ',f9.6)
      print * 

      close (iunit)
      return
      end subroutine
      
      
!     ==================================================================
      subroutine testvkb(vkbfile,vl,npwwfn,nkibz,ntype,natom,mpsang)
!     ==================================================================
!     test vkb file and return parameter of it
      implicit none
      integer vl
      character (len=vl) :: vkbfile
      integer npwwfn, nkibz
      integer ntype, natom, mpsang
      integer nspnl, nkibzr, ngvkb
      integer is, ia, il, ik
      integer, allocatable :: na(:), nl(:)
      
      print *, 'testing file *.vkb'
      open(unit=11,form='unformatted',status='unknown',file=vkbfile)
      read(11) nkibzr,nspnl
      if(nkibzr/=nkibz) then
        print *, '** ERROR: number of k in file *.vkb not right!'
        stop '*.vkb nkpt'
      endif
      ntype = nspnl
      print *, 'number of atomic species ', nspnl
      print *, 'number of k-points ', nkibzr
      allocate(na(nspnl),nl(nspnl))
      natom = 0
      mpsang = 0
      do is = 1, nspnl
        read(11) na(is), nl(is)
        write(*,*) ' specie ',is
        write(*,*) '  number of atoms ',na(is)
        write(*,*) '  number of angular momentum components ',nl(is)
        natom = natom + na(is)
        if(nl(is) > mpsang) mpsang = nl(is)
        do ia = 1, na(is)
          read(11) ! skip atomic position
        enddo
        do il = 1, nl(is)
          read(11) ! skip vkbsign
        enddo
      enddo
      print *, 'total number of atoms ', natom
      print *, 'maximum number of angular momentum components ', mpsang
      do ik = 1, nkibz
        read(11) ngvkb
        write(*,*) 'k-point ', ik, 'number of G ', ngvkb
        if(ngvkb<npwwfn) then
          write(*,*) 'number of g in out.vnl not enough', ngvkb
          stop 'ngvkb in *.vkb'
        endif
        do is = 1, nspnl
          do il = 1, nl(is)
            read(11) ! skip vkb
            read(11) ! skip vkbd
          enddo
        enddo
      enddo
      close(11)
      print *
      return
      end subroutine
      
      
!     ==================================================================
      subroutine rdvkb(vkbfile,vl,npwwfn,nkibz,ntype,natom,mpsang, &
     &                 type,xcart,vkbsign,vkb,vkbd)
!     ==================================================================
!     read vkb file
      implicit none
      integer vl
      character (len=vl) :: vkbfile
      integer npwwfn, nkibz
      integer ntype, natom, mpsang
      integer type(natom)
      double precision :: xcart(3,natom)
      real vkbsign(mpsang,ntype)
      real vkb(npwwfn,ntype,mpsang,nkibz)
      real vkbd(npwwfn,ntype,mpsang,nkibz)
      integer nspnl, nkibzr, ngvkb
      integer, allocatable :: na(:), nl(:)
      real tau0(3)
      integer is, ia, iar, il, i, ik, ig
      real dum
      logical, parameter :: DEBUG = .false.
      
      print *, 'reading Kleinman-Bylander form factors ', &
     &         'and atomic structure factors (file *.vkb)'
      open(unit=11,form='unformatted',status='unknown',file=vkbfile)
      read(11) nkibzr,nspnl
      print *, 'number of atomic species ', nspnl
      print *, 'number of k-points ', nkibzr
      if(nkibzr/=nkibz) stop '*.vkb, nkpt'
      if(nspnl/=ntype) stop '*.vkb, nspnl'
      allocate(na(nspnl),nl(nspnl))
      vkbsign(:,:) = 0.0
      ia = 0
      do is = 1, nspnl
        read(11) na(is), nl(is)
        write(*,*) ' specie ',is
        write(*,*) '  number of atoms ',na(is)
        write(*,*) '  number of angular momentum components ',nl(is)
        if(nl(is)>mpsang) stop '*.vkb mpsang'
        if(ia+na(is)>natom) stop '*.vkb natom'
        do iar = 1, na(is)
          ia = ia + 1
          type(ia) = is
          read(11) (tau0(i),i=1,3)
          xcart(:,ia) = tau0(:)
          write(*,3845) iar, (xcart(i,ia),i=1,3)
3845      format('   atom ',i2,' cell shift factor xcart = ',3f10.6)
        enddo
        do il = 1, nl(is)
          read(11) vkbsign(il,is)
          write(*,'("  vkbsign ",5f4.1)') vkbsign(1:nl(is),is)
        enddo
      enddo
      
      vkb(:,:,:,:) = 0.0
      vkbd(:,:,:,:) = 0.0
      do ik = 1, nkibz
        read(11) ngvkb
        print *, 'k-point ', ik, 'number of G ', ngvkb
        if(ngvkb<npwwfn) stop 'ngvkb in *.vkb'
        do is = 1, nspnl
          if(DEBUG) print *, 'is = ', is
          do il = 1, nl(is)
            if(DEBUG) print *, 'il = ', il
            read(11) (vkb(ig,is,il,ik),ig=1,npwwfn), &
     &                (dum,ig=npwwfn+1,ngvkb)
            read(11) (vkbd(ig,is,il,ik),ig=1,npwwfn), &
     &                (dum,ig=npwwfn+1,ngvkb)
            if(DEBUG) then
              write(*,'("vkb = ",15f5.2)') vkb(:,is,il,ik)
              write(*,'("vkbd = ",15f5.2)') vkbd(:,is,il,ik)
            endif
            if(ik == 1) then
              dum = 0.0
              do ig = 1, npwwfn
                dum = dum + vkb(ig,is,il,ik) + vkbd(ig,is,il,ik)
              enddo
              if(dum == 0.0) then
                if(il == 1) then
                  print *, 'specie ', is, ' reference s'
                else if(il == 2) then
                  print *, 'specie ', is, ' reference p'
                else if(il == 3) then
                  print *, 'specie ', is, ' reference d'
                endif
              endif
            endif
          enddo ! il
        enddo ! is
      enddo ! ik
      close(11)
      print *
      return
      end subroutine
