! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

!=======================================================================
!     subroutine for the evaluation of lattice values
!     last changes: 11-1997 Valerio Olevano
!=======================================================================




!=======================================================================
      subroutine lattice(a1,a2,a3,b1,b2,b3,ucvol,bzvol)
!=======================================================================
!     calculate reciprocal lattice vectors and cell volumes
!-----------------------------------------------------------------------

      real a1(3), a2(3), a3(3)
      real b1(3), b2(3), b3(3)
      real ucvol, bzvol, PI
      integer lattype

!     a1, a2, a3: real-space lattice vectors in au
!     b1, b2, b3: reciprocal-space lattice vectors in au**-1
!     ucvol: unit cell volume in au**3
!     bzvol: reciprocal cell volume

      real aa(3,3)
      real a


      PI = 4.0 * atan(1.0)

!     calculate Brillouin zone and Unit cell volumes
      ucvol = a1(1)* (a2(2)*a3(3)-a2(3)*a3(2))
     &      + a1(2)* (a2(3)*a3(1)-a2(1)*a3(3))
     &      + a1(3)* (a2(1)*a3(2)-a2(2)*a3(1))
      bzvol = 8*PI*PI*PI / ucvol

!     calculate reciprocal-space lattice vectors b1-b3
      b1(1) = 2.0 * PI * (a2(2)*a3(3)-a2(3)*a3(2)) / ucvol
      b1(2) = 2.0 * PI * (a2(3)*a3(1)-a2(1)*a3(3)) / ucvol
      b1(3) = 2.0 * PI * (a2(1)*a3(2)-a2(2)*a3(1)) / ucvol
      b2(1) = 2.0 * PI * (a3(2)*a1(3)-a3(3)*a1(2)) / ucvol
      b2(2) = 2.0 * PI * (a3(3)*a1(1)-a3(1)*a1(3)) / ucvol
      b2(3) = 2.0 * PI * (a3(1)*a1(2)-a3(2)*a1(1)) / ucvol
      b3(1) = 2.0 * PI * (a1(2)*a2(3)-a1(3)*a2(2)) / ucvol
      b3(2) = 2.0 * PI * (a1(3)*a2(1)-a1(1)*a2(3)) / ucvol
      b3(3) = 2.0 * PI * (a1(1)*a2(2)-a1(2)*a2(1)) / ucvol

!     find Bravais lattice type
       lattype = 1
!     fcc
      a = (ucvol*4.0)**(1.0/3.0)
      do i = 1, 3
        aa(1,i) = a1(i) / (a/2)
        aa(2,i) = a2(i) / (a/2)
        aa(3,i) = a3(i) / (a/2)
        if(aa(1,i)<=0 .and. aa(2,i)<=0 .and. aa(3,i)<=0) then
          aa(:,i) = - aa(:,i)
        endif
      enddo
      if(aa(1,1)==1 .and. aa(1,2)==0 .and. aa(1,3)==1 .and.
     &   aa(2,1)==0 .and. aa(2,2)==1 .and. aa(2,3)==1 .and.
     &   aa(3,1)==1 .and. aa(3,2)==1 .and. aa(3,3)==0) then
        lattype = 1
      endif
      if(lattype == 1) then
!        print *, 'Bravais lattice: fcc'
        write(*,50) a, a * 0.529177
50      format(' a = ',f5.2,' [au]   ',f5.2,' Angstrom')
        print *
      endif

      write(*,100) a1, a2, a3
100   format(' real-space lattice vectors in cartesians [au]:'
     &       /('     ',3F8.2))
      write(*,200) b1, b2, b3
200   format(/' calculated reciprocal lattice vectors',
     &       ' in cartesians [au]:'/
     &       ('     ',3F8.4))
      write(*,300) bzvol, ucvol
300   format (/' brillouin zone volume = ', F8.4, ' [au]'/
     &         ' unit cell volume      = ', F8.2, ' [au]')
      write(*,*) 

      return
      end



c=======================================================================
      SUBROUTINE SUROT(OP,NOP,NFFT1,NFFT1A,NFFT2,NFFT3,NR,NPWVEC,
     &             GVEC, GROTTB, IROTTB, grottbm1)
c=======================================================================
C     Set up tables indicating rotations of r-points and G-vectors
C     IROTTB(r,R) contains the index no. of R**-1 r [Note inverse!]
C     GROTTB(G,I,R) contains the index no. of R I G
C     Input:
C       OP
C       NOP
C       NFFT1, NFFT1A, NFFT2, NFFT3
C     Output:
C     IROTTB
C     GROTTB
C----------------------------------------------------------------------

      integer nr
      REAL OP(3,3,NOP)
      INTEGER GVEC(3,NPWVEC), IROTTB(NR,NOP), GROTTB(NPWVEC,2,NOP)
      integer grottbm1(NPWVEC,2,nop)

      INTEGER NFFT(3)
      REAL R(3), RROT(3), GX(3), GXX(3)

      WRITE (6,1034)
1034  FORMAT (' setting up r- and G-rotation tables')

      NFFT(1) = NFFT1
      NFFT(2) = NFFT2
      NFFT(3) = NFFT3

      DO 100 IX = 0, NFFT1-1
      DO 110 IY = 0, NFFT2-1
      DO 120 IZ = 0, NFFT3-1

        R(1) = IX
        R(2) = IY
        R(3) = IZ
        IR = 1 + IX + IY*NFFT1A + IZ*NFFT1A*NFFT2

        DO 200 IOP = 1, NOP
C         Form R**-1 r
          CALL DOSYMR (OP(1,1,IOP), 1, R, NFFT, RROT)
          JX = MODX(NINT(RROT(1)),NFFT1)
          JY = MODX(NINT(RROT(2)),NFFT2)
          JZ = MODX(NINT(RROT(3)),NFFT3)
          IROTTB(IR,IOP) = 1 + JX + JY*NFFT1A + JZ*NFFT1A*NFFT2
200     CONTINUE

120   CONTINUE
110   CONTINUE
100   CONTINUE

C     Ensure that any spare space in the FFT array (because of
C     NFFT1A possible being one greater than NFFT1) is always
C     rotated onto itself so that no harm is done.  Note that the
C     do-loop is not executed if NFFT1A=NFFT1
      DO 300 IX = NFFT1, NFFT1A-1
      DO 310 IY = 0, NFFT2-1
      DO 320 IZ = 0, NFFT3-1
        IR = 1 + IX + IY*NFFT1A + IZ*NFFT1A*NFFT2
        DO 400 IOP = 1, NOP
          IROTTB(IR,IOP) = IR
400     CONTINUE
320   CONTINUE
310   CONTINUE
300   CONTINUE

      WRITE (6,1035)
1035  FORMAT (' r-rotation tables set up')

C     Set up G-rotation table
      DO 520 IG = 1, NPWVEC
        GX(1) = GVEC(1,IG)
        GX(2) = GVEC(2,IG)
        GX(3) = GVEC(3,IG)
        DO 500 IINV = 1, 2
          DO 510 IOP = 1, NOP
            CALL DOSYM(OP(1,1,IOP),IINV,GX,GXX)
              IFOUND = 0
              DO 530 IGP = 1, NPWVEC
                IF (ABS(GXX(1)-GVEC(1,IGP)).LT.1.0E-3) THEN
                IF (ABS(GXX(2)-GVEC(2,IGP)).LT.1.0E-3) THEN
                IF (ABS(GXX(3)-GVEC(3,IGP)).LT.1.0E-3) THEN
                  IFOUND = 1
                  GROTTB(IG,IINV,IOP) = IGP
                  grottbm1(igp,iinv,iop) = ig
                ENDIF
                ENDIF
                ENDIF
530           CONTINUE
              IF (IFOUND.EQ.0) THEN
                write(*,*) '**ERROR surot 10: G-shell not closed:'
                write(*,*) IG, NPWVEC, IOP, IINV, GXX, GX
c               write(*,*) ((GVEC(i,j), i=1,3), j=1,NPWVEC)
                stop
              ENDIF
510       CONTINUE
500     CONTINUE
520   CONTINUE

      WRITE (6,1135)
1135  FORMAT (' G-rotation tables set up'/)

      RETURN
      END







c=======================================================================
      subroutine identk(KIBZ,NKIBZ,NKBZMX,
     &                  NR,NOP,NINV,IROTTB,
     &                  OP,KBZ,KTAB,KTABR,KTABI,KTABO,
     &                  NKBZ,WTK)
c=======================================================================
C     Identify k-points in whole BZ and also the q-points by which they
C     differ
C     Input:
C       KIBZ: k-points in IBZ
c       nop: number of symmetry operations
C       OP: symmetry operation matrices
C       IROTTB: rotation table for r-points
c       ninv: = 2 inversion is considered; = 1 not considered
C     Output:
C       KBZ: k-points in whole BZ, and
C       KTAB: a table giving for each the k-points
C             in the IBZ to which it is related, and
C       KTABR: the r-point to look at for each required r (unless NR=0,
C              in which case KTABR is untouched)
C       KTABI: whether the wfn has to be complex-conjugated (-1 if so, 1 if not)
C       KTABO: the symmetry operation that takes kIBZ to each kBZ
C       NKBZ: no. of such k-points
C       WTK: weight for each k-point in IBZ for symmetric quantities:
C            no. of distinct k's in whole BZ / (ninv*NOP)
C----------------------------------------------------------------------

      integer nr
      integer nkibz, nkbzmx
      integer nop, ninv
      REAL KIBZ(3,NKIBZ), KBZ(3,NKBZMX), OP(3,3,NOP), WTK(NKIBZ)
      INTEGER KTAB(NKBZMX), KTABR(nr,NKBZMX), KTABI(NKBZMX)
      INTEGER KTABO(NKBZMX), IROTTB(nr,NOP)

      REAL KNEW(3)

      write(*,*) 'identifying k-points'

c     zero no. of k-points found
      NKBZ = 0

c     loop over k-points in IBZ
      DO 100 IKIBZ = 1, NKIBZ
        WTK(IKIBZ) = 0.0

c       loop over symmetry operations R and inversions I
        DO 200 IINV = 1, ninv
          DO 300 IOP = 1, NOP
c           form RIk
            CALL DOSYM(OP(1,1,IOP), IINV, KIBZ(1,IKIBZ), KNEW(1))
c           check whether it has already been found (to within a RL vector)
            IOLD = 0
            DO 400 IKBZ = 1, NKBZ
              IOLD = IOLD + ICMPK2(KNEW,KBZ(1,IKBZ))
400         CONTINUE
            IF (IOLD.EQ.0) THEN
              NKBZ = NKBZ + 1
              WTK(IKIBZ) = WTK(IKIBZ) + 1.0
              IF (NKBZ.GT.NKBZMX) then
                write(*,*) '**ERROR identk 1: NKBZMX too small;'
                write(*,*) '                  NKBZMX = ', NKBZMX
                write(*,*) '                  increase NKBZMX!'
                stop
              endif
              KBZ(1,NKBZ) = KNEW(1)
              KBZ(2,NKBZ) = KNEW(2)
              KBZ(3,NKBZ) = KNEW(3)
              KTAB(NKBZ) = IKIBZ
              KTABO(NKBZ) = IOP
              KTABI(NKBZ) = 3 - 2 * IINV
              DO 500 IR = 1, NR
c               we want u(R**-1 r).  IROTTB contains the r-point-no. of R**-1 r.
                KTABR(IR,NKBZ) = IROTTB(IR,IOP)
500           CONTINUE
            ENDIF

300       CONTINUE
200     CONTINUE
      WTK(IKIBZ) = WTK(IKIBZ) / (ninv*NOP)
100   CONTINUE


!     print out results
      write(*,'("  number of k-points in the IBZ ",i5)') nkibz
      print *, ' k-points [reduced coordinates]    weights'
      write(*,'(3(1x,f10.6),2x,f6.3)') 
     &  ((kibz(i,ik),i=1,3),wtk(ik),ik=1,nkibz)
      write(*,'("  number of k-points in the full BZ ",i5)') nkbz
      print *, ' k-points [reduced coordinates]:'
      write(*,'(3(1x,f10.6))') 
     &  ((kbz(i,ik),i=1,3),ik=1,nkbz)
      print *
      return
      end


!     ==================================================================
      subroutine findk(nkcalc,nkbz,xkcalc,kbz,kcalc)
!     ==================================================================
!     check if the k-point is in the set of the kbz
!     ------------------------------------------------------------------

      integer nkcalc, nkbz
      real xkcalc(3,nkcalc)
      real kbz(3,nkbz)
      integer kcalc(nkcalc)

      write(*,*) 'check if the k-points for sigma are in the set of BZ'
      do j = 1, nkcalc
        kcalc(j) = 0
        do ik = 1, nkbz
          if(all(abs(xkcalc(:,j)-kbz(:,ik))<1.e-3)) kcalc(j) = ik
        enddo
        if (kcalc(j).eq.0) then
          write(*,*) '** ERROR findk 1: k-point not in the set of kbz;'
          stop
        endif
      enddo

      write(*,*) 'check ok!'
      write(*,*)
      return
      end


!     ==================================================================
      logical function findkp(nkbz,kbz,ik,k,g0)
!     ==================================================================
!     check if the k-point is in the set of the kbz
!     (subtracting the reciprocal vector g0)
!     and return the index number and the reciprocal vector g0
!     ------------------------------------------------------------------
      implicit none
      integer nkbz
      real kbz(3,nkbz)
      real k(3)
      integer ik
      integer g0(3)
      integer ikp
      real kp(3), kpg0(3)
      integer ig1, ig2, ig3
      logical, external :: lcompv
      logical yetfound
      
      yetfound = .false.
      do ikp = 1, nkbz
        kp(:) = kbz(:,ikp)
!       form k' + G0
        do ig1 = -2, 2
          kpg0(1) = kp(1) + ig1
          do ig2 = -2, 2
            kpg0(2) = kp(2) + ig2
            do ig3 = -2, 2
              kpg0(3) = kp(3) + ig3
!             compare k' + G0 with k given              
              if(lcompv(kpg0,k)) then
!               check if k has been yet found
                if(yetfound) then
                  print *, '** ERROR, findkp 1: kp yet found'
                  print *, k
                  print *, ik, kbz(:,ik), g0
                  print *, ikp, kbz(:,ikp), ig1, ig2, ig3
                  stop 'kp yet found'
                endif
                yetfound = .true.
                ik = ikp
                g0(1) = ig1
                g0(2) = ig2
                g0(3) = ig3
              endif
            enddo
          enddo
        enddo
      enddo
      findkp = yetfound
      return
      end


!     ==================================================================
      subroutine checkq(nkbz,kbz,nop,op,nq,q,b1,b2,b3,tpula)
!     ==================================================================
!     identify the q-points by which the k-points in BZ differ
!     not used in sigma
!     check if q-points given as input are good
!     ------------------------------------------------------------------

      integer nkbz
      integer nop
      integer nqallm
      integer nqall
      integer nq
      real op(3,3,nop)
      real kbz(3,nkbz)
      real q(3,nq)
      real b1(3), b2(3), b3(3)
      real tpula
      real, allocatable :: qall(:,:)

      real qrot(3), qposs(3), qtemp(3)
      integer gtemp(3)
      logical, external :: litst0

      print *, 'check if q-points given are allowed, ',
     &           'must be q = k - k1 + G0'

!      print *, 'find q-points q = k - k1'

      nqallm = 1000

710   allocate(qall(3,nqallm),stat=istat)
      if(istat.ne.0) stop 'out of memory'

!     zero no. of q-points found
      nqall = 0

!     loop over all k-points in BZ, forming k - k(1)
      do ik = 1, nkbz
          qposs(1) = kbz(1,ik) - kbz(1,1)
          qposs(2) = kbz(2,ik) - kbz(2,1)
          qposs(3) = kbz(3,ik) - kbz(3,1)

!         check whether this q (or its equivalent) has already been found
          ifound = 0
          do iq = 1, nqall
            do iinv = 1, 2
              do iop = 1, nop
                call dosym(op(1,1,iop),iinv,qall(1,iq),qrot)
                if (icmpk2(qrot,qposs).eq.1) ifound = ifound + 1
              enddo
            enddo
          enddo
          if (ifound.eq.0) then
            nqall = nqall + 1
            if (nqall.gt.nqallm) then
              deallocate(qall)
              nqallm = nqallm * 2
              goto 710
            endif
            qall(1,nqall) = qposs(1)
            qall(2,nqall) = qposs(2)
            qall(3,nqall) = qposs(3)
          endif
      enddo

!      print *, 'reduce to 1 BZ'
      do iq = 1, nqall
        call reduceto1bz(qall(:,iq),qtemp,gtemp,b1,b2,b3)
        qall(:,iq) = qtemp(:)
      enddo

!      print *, 'check if q-points given as input are good'
      ierr = 0
      do iq = 1, nq
        if(.not. litst0(q(:,iq))) then
          ifound = 0
          do iqp = 1, nqall
            if(icmpk2(q(1,iq),qall(1,iqp)).eq.1) ifound = 1
          enddo
          if(ifound.eq.0) then
            ierr = 1
            write(*,*) '** ERROR findq 1: q-point not allowed'
            write(6,2150) (q(i,iq),i=1,3)
2150        format('                   q = ',3f12.6)
          endif
        endif
      enddo

      if(ierr.eq.1.or.nq.eq.0) then
      write(*,*) 'number of q-points found ', nqall
      write(6,2106) tpula
2106  format(' 2pi/a = ',f10.6,' a.u.')
      write(*,*) 'q-points (in the first brillouin zone):'
      write(*,*) '[reciprocal lattice units b1 b2 b3]    ',
     &           '[cartesian coordinates units of 2pi/a]'
      do j = 1, nqall
!     and also calculate and print in Cartesian coordinates for RJN
!     note that ULA is not guaranteed to be the same as that used by RJN;
!     it is chosen to be sqrt(2) * |a1|
        qx = (qall(1,J)*b1(1) + qall(2,J)*b2(1) + qall(3,J)*b3(1))/tpula
        qy = (qall(1,J)*b1(2) + qall(2,J)*b2(2) + qall(3,J)*b3(2))/tpula
        qz = (qall(1,J)*b1(3) + qall(2,J)*b2(3) + qall(3,J)*b3(3))/tpula
        write(6,2105) (qall(i,j),i=1,3), qx, qy, qz
2105    format(3f12.6,4x,3f12.6)
      enddo
      write(*,*)

!      open(unit=67,file='chi.qpt',status='unknown',form='formatted')
!      write(67,2108) nqall
!2108  format(i3)
!      do j = 1, nqall
!       in the case of q=0 add a small amount in a direction of no symmetry
!        if(litst0(qall(:,j))) then
!          qall(1,j) = 0.000010
!          qall(2,j) = 0.000020
!          qall(3,j) = 0.000030
!        endif
!        write(67,2107) (qall(i,j),i=1,3)
!2107    format(f9.6,3x,f9.6,3x,f9.6)
!      enddo
!      close(67)

!      write(*,*) 'q-points allowed written in chi.qpt'
!      write(*,*) '** warning: a small amount has been added at q=0'

      if(ierr.eq.1) stop 'q-point not allowed'
      stop ''

      endif


      write(*,*) 'check ok'
      write(*,*)
      deallocate(qall)
      return
      end


!     ==================================================================
      subroutine findnq(nkbz,kbz,nop,op,nq)
!     ==================================================================
!     identify the number of q-points by which the k-points in BZ differ
      implicit none
      integer nkbz
      integer nop
      integer nqallm
      integer nqall
      integer nq
      real op(3,3,nop)
      real kbz(3,nkbz)
      real, allocatable :: qall(:,:)

      real qrot(3), qposs(3)
      integer istat, ik, iq, iop, iinv, ifound
      logical, external :: litst0
      integer, external :: icmpk2

      nqallm = 1000

710   allocate(qall(3,nqallm),stat=istat)
      if(istat.ne.0) stop 'out of memory'

      nqall = 0

!     loop over all k-points in BZ, forming k - k1
      do ik = 1, nkbz
          qposs(1) = kbz(1,ik) - kbz(1,1)
          qposs(2) = kbz(2,ik) - kbz(2,1)
          qposs(3) = kbz(3,ik) - kbz(3,1)

!         check whether this q (or its equivalent) has already been found
          ifound = 0
          do iq = 1, nqall
            do iinv = 1, 2
              do iop = 1, nop
                call dosym(op(1,1,iop),iinv,qall(1,iq),qrot)
                if (icmpk2(qrot,qposs).eq.1) ifound = ifound + 1
              enddo
            enddo
          enddo
          if (ifound.eq.0) then
            nqall = nqall + 1
            if (nqall.gt.nqallm) then
              deallocate(qall)
              nqallm = nqallm * 2
              goto 710
            endif
            qall(1,nqall) = qposs(1)
            qall(2,nqall) = qposs(2)
            qall(3,nqall) = qposs(3)
          endif
      enddo

      deallocate(qall)
      nq = nqall
      write(*,*) 'number of q-points found ', nq
      return
      end


!     ==================================================================
      subroutine findq(nkbz,kbz,nop,op,nq,q,b1,b2,b3)
!     ==================================================================
!     identify the q-points by which the k-points in BZ differ
      implicit none
      integer nkbz
      integer nop
      integer nq
      real op(3,3,nop)
      real kbz(3,nkbz)
      real q(3,nq)
      real b1(3), b2(3), b3(3)

      real qrot(3), qposs(3)
      integer gtemp(3)
      integer ik, iq, iop, iinv, iqp, j, i
      logical found
      logical, external :: litst0
      integer, external :: icmpk2


      write(*,*) 'find q-points q = k - k1 and translate in first BZ'

!     zero no. of q-points found
      iq = 0

!     loop over all k-points in BZ, forming k - k1
      do ik = 1, nkbz
          qposs(1) = kbz(1,ik) - kbz(1,1)
          qposs(2) = kbz(2,ik) - kbz(2,1)
          qposs(3) = kbz(3,ik) - kbz(3,1)

!         check whether this q (or its equivalent) has already been found
          found = .false.
          do iqp = 1, iq
            do iinv = 1, 2
              do iop = 1, nop
                call dosym(op(1,1,iop),iinv,q(1,iqp),qrot)
                if(icmpk2(qrot,qposs).eq.1) found = .true.
              enddo
            enddo
          enddo
          if(.not.found) then
            iq = iq + 1
            if(iq>nq) stop '** BUG: iq > nq'
            q(1,iq) = qposs(1)
            q(2,iq) = qposs(2)
            q(3,iq) = qposs(3)
          endif
      enddo

      if(iq .ne. nq) stop '** BUG: iq != nq'

!     translate q-points (calculated for info) to 1st BZ 
      do iq = 1, nq
        call bz1(q(:,iq),gtemp,b1,b2,b3)
      enddo

      write(*,*) 'q-points [reduced coordinates]'
      do j = 1, nq
        if(litst0(q(:,j))) then
          q(1,j) = 0.000010
          q(2,j) = 0.000020
          q(3,j) = 0.000030
        endif
        write(6,2105) (q(i,j),i=1,3)
2105    format(3f12.6)
      enddo
      write(*,*)

      return
      end



!     ==================================================================
      logical function lcompv(va,vb)
!     ==================================================================
!     compare two 3d vectors, and return .true. if same, .false. if diff
!     ------------------------------------------------------------------
      real va(3), vb(3)
      lcompv = .false.
      if(abs(va(1)-vb(1)) < 1.0e-4) then
        if(abs(va(2)-vb(2)) < 1.0e-4) then
          if(abs(va(3)-vb(3)) < 1.0e-4) then
            lcompv = .true.
          endif
        endif
      endif
      return
      end
      
      

c=======================================================================
      FUNCTION ICMPK2(K1,K2)
c=======================================================================
C  Compare two k-points.  Return 1 if same WITHIN A RL VECTOR,
C    0 if different.
C----------------------------------------------------------------------
      INTEGER ICMPK2
      REAL K1(3), K2(3), F
C
C  Statement function definition
      F(X) = ABS(X - NINT(X))
C
      ICMPK2 = 0
      IF (F(K1(1)-K2(1)).LT.1.0E-4) THEN
        IF (F(K1(2)-K2(2)).LT.1.0E-4) THEN
          IF (F(K1(3)-K2(3)).LT.1.0E-4) THEN
            ICMPK2 = 1
          ENDIF
        ENDIF
      ENDIF
      RETURN
      END



c=======================================================================
      SUBROUTINE DOSYM(OP, IINV, K1, K2)
c=======================================================================
C     Perform a symmetry operation k2 = RI k1
C     If the OP matrices are in reciprocal-lattice units then the vectors
C     must be too.
C     IINV is 1 for non-inversion, 2 for inversion
C----------------------------------------------------------------------
      REAL K1(3), K2(3), OP(3,3)

      DO 100 I = 1, 3
        K2(I) = 0.0
100   CONTINUE

      IMULT = 3 - 2 * IINV
      DO 200 J = 1, 3
        DO 300 I = 1, 3
          K2(I) = K2(I) + IMULT * OP(I,J) * K1(J)
300     CONTINUE
200   CONTINUE

      RETURN
      END





c=======================================================================
      SUBROUTINE DOSYMR(OP, IINV, R1, NFFT, R2)
c=======================================================================
C     Perform an inverse symmetry operation on a real-space vector in
C       FFT units
C     Rationale:
C       If the OP matrices are in reciprocal-lattice units then
C       since the inverse-transpose of the OP matrix is the
C       corresponding matrix in real-space-lattice units, NFFT(I)/NFFT(J)
C       times the transpose is the matrix corresponding to the inverse
C       operation in FFT units.  Thus calling DOSYMR with OP and r1 performs
C       the operation r2 = R**-1 r1 if OP corresponds to R.
C       IINV is 1 for non-inversion, 2 for inversion
C----------------------------------------------------------------------
      REAL R1(3), R2(3), OP(3,3)
      INTEGER NFFT(3)

      DO 100 I = 1, 3
        R2(I) = 0.0
100   CONTINUE

      IMULT = 3 - 2 * IINV
      DO 200 J = 1, 3
        DO 300 I = 1, 3
          R2(I) = R2(I) + IMULT * OP(J,I) * NFFT(I) *
     &      R1(J) / FLOAT(NFFT(J))
300     CONTINUE
200   CONTINUE

      RETURN
      END








c=======================================================================
      subroutine identq(Q,NQ,NQBZMX,OP,NOP,ninv,QBZ,QTAB,
     &                  QTABI,QTABO,NQBZ)
c=======================================================================
C     Identify q-points in whole BZ
c     NOT USED in chi!
C     Note: for q close to zero only one q-point is counted (ie not the
C       rotations and reflections of the little q, nor other little
C       q-points in the qIBZ set)
C     Input:
C       Q: q-points in IBZ
C       OP: symmetry operation matrices
C     Output:
C       QBZ: q-points in whole BZ, and
C       QTAB: a table giving for each the q-points
C             in the IBZ to which it is related, and
C       QTABI: whether the wfn has to be complex-conjugated (-1 if so, 1 if not)
C       QTABO: the symmetry operation that takes kIBZ to each kBZ
C       NQBZ: no. of such q-points
C----------------------------------------------------------------------
      integer nq, nop, nqbzmx
      real Q(3,NQ), QBZ(3,NQBZMX), OP(3,3,NOP)
      integer QTAB(NQBZMX), QTABI(NQBZMX), QTABO(NQBZMX)

      real qnew(3)


      write(*,*) 'identifying q-points'

c     Zero no. of q-points found
      NQBZ = 0

      NQEQ0 = 0

c     Loop over q-points in IBZ
      DO 100 IQ = 1, NQ

c       If q is close to zero, treat it differently as described above
        IF (itst0(Q(1,IQ)).EQ.1) THEN
          IF (NQEQ0.EQ.0) THEN
            NQEQ0 = 1
            NQBZ = NQBZ + 1
            IF (NQBZ.GT.NQBZMX) then
                write(*,*) '**ERROR identq 1: NQBZMX too small;'
                write(*,*) '                  increase NQBZMX!'
                stop
              endif
            QBZ(1,NQBZ) = Q(1,IQ)
            QBZ(2,NQBZ) = Q(2,IQ)
            QBZ(3,NQBZ) = Q(3,IQ)
            QTAB(NQBZ) = IQ
            QTABO(NQBZ) = 1
            QTABI(NQBZ) = 1
          ENDIF
        ELSE

C       Loop over symmetry operations R and inversions I
        DO 200 IOP = 1, NOP
          DO 300 IINV = 1, ninv
C           Form RIk
            CALL DOSYM(OP(1,1,IOP),IINV,Q(1,IQ),qnew(1))
C           Check whether it has already been found (to within a RL vector)
            IOLD = 0
            DO 400 IQBZ = 1, NQBZ
              IOLD = IOLD + ICMPK2(qnew,QBZ(1,IQBZ))
400         CONTINUE
            IF (IOLD.EQ.0) THEN
              NQBZ = NQBZ + 1
              IF (NQBZ.GT.NQBZMX) then
                write(*,*) '**ERROR identq 2: NQBZMX too small;'
                write(*,*) '                  increase NQBZMX!'
                stop
              endif
              QBZ(1,NQBZ) = qnew(1)
              QBZ(2,NQBZ) = qnew(2)
              QBZ(3,NQBZ) = qnew(3)
              QTAB(NQBZ) = IQ
              QTABO(NQBZ) = IOP
              QTABI(NQBZ) = 3 - 2 * IINV
            ENDIF

300       CONTINUE
200     CONTINUE

        ENDIF

100   CONTINUE

c     Print out results
      WRITE(6,1000) NQ, (J,(Q(I,J),I=1,3),J=1,NQ)
1000  FORMAT (' q-points in irreducible wedge (IBZ) ',I3,/
     &        ' q-points [reciprocal lattice units]:'/
     &        (I5,3F12.6))
      WRITE(6,2000) NOP, NQBZ, (J,(QBZ(I,J),I=1,3),J=1,NQBZ)
2000  FORMAT ( ' together with the',I5,' symmetry operations',
     &                ' and inversion'/
     &         ' have yielded',I7,' q-points in Brillouin Zone (BZ):'/
     &          (4(I5,3F5.2)))
      write(*,*)

      RETURN
      END



      subroutine findqg0(iq,g0,kmkp,nqbz,qbz)
       implicit none
 
!      Arguments ------------------------------------
       integer :: iq,nqbz
       integer :: g0(3)
       real :: kmkp(3),qbz(3,nqbz)
 
!      Local variables-------------------------------
       integer :: jg01,jg02,jg03,iqbz
       real :: qpg0(3),kp(3)
       real, parameter :: biggest_q0=1.0e-3
       real, parameter :: smallest_positive_real=1.0e-9

! *************************************************************************
       iq=0
       if(all(abs(kmkp(:))<smallest_positive_real)) then
!      find q close to 0
        do iqbz=1,nqbz
         if(all(abs(qbz(:,iqbz))<biggest_q0)) iq=iqbz
        enddo
        if(iq==0) stop 'wrong em1 file: q=0 not existing'
        g0(:)=0
        return
       else
        do iqbz=1,nqbz
         do jg01=-2,2
          do jg02=-2,2
           do jg03=-2,2
!           form q-G0 and check if it is the one
            qpg0(1)=qbz(1,iqbz)+jg01
            qpg0(2)=qbz(2,iqbz)+jg02
            qpg0(3)=qbz(3,iqbz)+jg03
            if(all(abs(qpg0(:)-kmkp(:))<1.e-5)) then
             if(iq/=0) then
              print *, '**error: duplicated q+g0.'
              print *, iqbz,qbz(:,iqbz),jg01,jg02,jg03
              print *, iq,qbz(:,iq),g0
              stop 'duplicated q+g0'
             endif
             iq=iqbz
             g0(1)=jg01
             g0(2)=jg02
             g0(3)=jg03
            endif
           enddo
          enddo
         enddo
        enddo
        if(iq==0) then
         print *, '**error: q = k - kp + g0 not found.'
         print *, 'kmkp  = ',kmkp(:)
         stop 'q = k - kp + g0 not found'
        endif
       endif 
       return
       end




c=======================================================================
      SUBROUTINE CLCQPG ( NPWVEC, GVEC, Q, NQ, B1,B2,B3, QPG)
c=======================================================================
C     Calculate |q+G| for each q and G
C     Input:
C       GVEC: G-vectors
C       Q: q-points
C       B1,B2,B3: RL vectors
C     Output:
C       QPG: |q+G| for each q and G
C----------------------------------------------------------------------
      REAL QPG(NPWVEC,NQ), Q(3,NQ), B1(3),B2(3),B3(3)
      INTEGER GVEC(3,NPWVEC)

      REAL GPQ(3)

      DO 100 IQ = 1, NQ
        DO 200 IG = 1, NPWVEC
          GPQ(1) = GVEC(1,IG) + Q(1,IQ)
          GPQ(2) = GVEC(2,IG) + Q(2,IQ)
          GPQ(3) = GVEC(3,IG) + Q(3,IQ)
          QPG(IG,IQ) = VLNGTH(GPQ,B1,B2,B3)
200     CONTINUE
100   CONTINUE

      RETURN
      END










!     ==================================================================
      function modx(i,j)
!     ==================================================================
!     calculate a function similar to MOD:
!     MODX(i,j) = MOD (i,j)             i>=0
!               = MOD ( MOD(i,j)+j , j) i<0
!     thus MODX(3,2)=1; MODX(-1,2)=1.
!     this is different from MOD(-1,2)=-1 for i negative.
!     rewritten by Nathalie Vast on August the 28th 2000
!     to allow vectoriz. (inline function)
!     checked on November 14th 2000
!      IF (I.GE.0) THEN
!        MODX = MOD(I,J)
!      ELSE
!        MODX = MOD( MOD(I,J) + J , J)
!      ENDIF
!     ------------------------------------------------------------------
!      IF (I.GE.0) THEN
!        MODX = MOD(I,J)
!      ELSE
!        MODX = MOD( MOD(I,J) + J , J)
!      ENDIF
      implicit none
      integer i, j, modx
      modx = mod( mod(i,j) + ((1-sign(1,i))/2)*j , j)
      return
      end



c=======================================================================
      function itst0(q)
c=======================================================================
c     check whether a q-vector is nominally zero (itst0=1 if so)
c     the criterion is that each component (in RL units) should be < epsilon
c     originally epsilon = 0.001
c     ----------------------------------------------------------------------
      real q(3)
      parameter(epsilon=0.001)
      itst0 = 0
      if(abs(q(1)).lt.epsilon) then
        if(abs(q(2)).lt.epsilon) then
          if(abs(q(3)).lt.epsilon) then
            itst0 = 1
          endif
        endif
      endif
      return
      end


!     ==================================================================
      logical function litst0(q)
!     ==================================================================
!     check whether a q-vector is nominally zero (litst0=.true. if so)
!     the criterion is that each component (in RL units) should be < e
!     originally epsilon = 0.001
!     ------------------------------------------------------------------
      implicit none
      real q(3)
      real, parameter :: e = 0.001
      litst0 = .false.
      if( all(abs(q(:))<e) ) litst0 = .true.
      return
      end


!     ==================================================================
      subroutine reduceto1bz(kpg,k,g,b1,b2,b3)
!     ==================================================================
!     given a k+g-point not necessarily in the 1st Brillouin Zone
!     find k and G where k is in the first BZ and G
!     a RL vector such that kpg=k+G. 
!     k, G and kpg are expressed in RL coords.
!     ------------------------------------------------------------------
      implicit none
      real kpg(3), k(3), b1(3), b2(3), b3(3)
      integer g(3)

      real kt(3)
      integer gt(3)
      real xmin, x
      integer gt1, gt2, gt3
      real, external :: vlngth

      xmin = 99999999.99
      do gt1 = -10, 10
        gt(1) = gt1
        do gt2 = -10, 10
          gt(2) = gt2
          do gt3 = -10, 10
            gt(3) = gt3
            kt(:) = kpg(:) - gt(:) 
            x = vlngth(kt,b1,b2,b3)
!           Heuristic method for choosing the right G when two Gs will do
!           (avoid the G such that kBZ has all components with same sign
!           otherwise k1BZ+k2BZ may need a G0 with components other than
!           -1,0,1 in RL coordinates to translate it back to the 1st BZ)
!           so add a small "penalty" to X if all 3 components of G have
!           the same sign
            x = x + 0.001 *
     &         abs(sign(1.,kt(1))+sign(1.,kt(2))+sign(1.,kt(3)))
            if(x < xmin) then
              g(:) = gt(:)
              k(:) = kt(:)
              xmin = x
            endif
          enddo
        enddo
      enddo
      return
      end





c=======================================================================
      SUBROUTINE BZ1(K,G,B1,B2,B3)
c=======================================================================
C     Given a k-point not necessarily in the 1st Brillouin Zone (but quite
C     close to it) find kBZ and G where kBZ is in the first BZ and G
C     a RL vector such that k=kBZ+G.  G is assumed to have all components
C     either -1,0,1, though in fact -2 to 2 are checked and if -2 or 2
C     is found as a component of G an error is flagged.
C     K, G and kBZ are expressed in RL coords.
C     Input:
C       K; k
C       B1-3: RL basis vectors
C     Output:
C       K: kBZ
C       G: G
C       (k is overwritten by kBZ (ie G is subtracted from k))
C-----------------------------------------------------------------------
      REAL K(3), B1(3), B2(3), B3(3)
      INTEGER G(3)

      REAL KMG(3)

      XMIN = 99999999.99
      DO 100 IX = -2, 2
        DO 110 IY = -2, 2
          DO 120 IZ = -2, 2
            KMG(1) = K(1) - IX
            KMG(2) = K(2) - IY
            KMG(3) = K(3) - IZ
            X = VLNGTH(KMG,B1,B2,B3)

C  Heuristic method for choosing the right G when two Gs will do
C   (avoid the G such that kBZ has all components with same sign
C    otherwise k1BZ+k2BZ may need a G0 with components other than
C    -1,0,1 in RL coordinates to translate it back to the 1st BZ)
C   so add a small "penalty" to X if all 3 components of G have
C   the same sign
            X = X + 0.001 *
     &         ABS(SIGN(1.,KMG(1))+SIGN(1.,KMG(2))+SIGN(1.,KMG(3)))

            IF (X.LT.XMIN) THEN
              G(1) = IX
              G(2) = IY
              G(3) = IZ
              XMIN = X
            ENDIF
120       CONTINUE
110     CONTINUE
100   CONTINUE

C     Check that G does not have a component of -2 or 2
      IF (ABS(G(1)).EQ.2) print *, 'WARNING: q is in 2nd BZ'
      IF (ABS(G(2)).EQ.2) print *, 'WARNING: q is in 2nd BZ'
      IF (ABS(G(3)).EQ.2) print *, 'WARNING: q is in 2nd BZ'

C     Subtract G from k
      K(1) = K(1) - G(1)
      K(2) = K(2) - G(2)
      K(3) = K(3) - G(3)

      RETURN
      END






!     ==================================================================
      FUNCTION VLNGTH(V,B1,B2,B3)
!     ==================================================================
!     calculate the length of a vector expressed in terms of the
!       basis vectors b1-b3
!     ------------------------------------------------------------------
      REAL V(3), B1(3), B2(3), B3(3)

      VLNGTH = 0.0
      do I = 1, 3
        VLNGTH = VLNGTH + (V(1)*B1(I)+V(2)*B2(I)+V(3)*B3(I))**2
      enddo
      VLNGTH = SQRT(VLNGTH)
      RETURN
      END


      COMPLEX FUNCTION PDTQRC(R,C,B1,B2,B3)
!     Calculate the dot product of a real vector with a complex
!     vector, where each is in terms of b1-b3
      COMPLEX C(3)
      REAL R(3), B1(3), B2(3), B3(3)
      PDTQRC = 0.0
      DO I = 1, 3
      PDTQRC = PDTQRC +
     &         (R(1)*B1(I)+R(2)*B2(I)+R(3)*B3(I)) *
     &         (C(1)*B1(I)+C(2)*B2(I)+C(3)*B3(I))
      enddo
      RETURN
      END



!     ==================================================================
      subroutine findshells(b1,b2,b3,ng,ngs,gvec)
!     ==================================================================
!     find the g-vector numbers that close a shell
!     check if ngs closes a shell
!     ------------------------------------------------------------------
      implicit none
      real b1(3), b2(3), b3(3)
      integer ng, ngs
      integer gvec(3,ng)

      real g(3)
      integer s, ig
      real l, lold
      logical :: shellclosed = .false.
      real, external :: vlngth

      write(*,*) 'shell  g-vector     radius [au]'
      s = 0
      g(:) = gvec(:,1)
      lold = vlngth(g,b1,b2,b3)
      do ig = 2, ng
        g(:) = gvec(:,ig)
        l = vlngth(g,b1,b2,b3)
        if((l-lold).gt.0.000001) then
          write(6,10) s, ig-1, lold
10        format(i6,i10,f11.6)
          lold = l
          s = s + 1
          if(ngs == ig-1) shellclosed = .true.            
        endif
      enddo
      if(shellclosed) then
        print *, 'shell closed at ', ngs
      else if(ngs == ng) then
        print *, 'shell probably closed at ', ngs
      else
        print *, 'ERROR, findshell 1: shell not closed'
        print *, 'NPWMAT = ', ngs
        stop 'shell not closed'
      endif
      print *

      return
      end


!     ==================================================================
      subroutine identifyg(g,ig,gvec,npwvec)
!     ==================================================================
!     find ig, index of gvec, corresponding to g
      implicit none
      integer npwvec, ig
      integer g(3), gvec(3,npwvec)
      ig = 1
      do while(g(1) /= gvec(1,ig) .or. 
     &         g(2) /= gvec(2,ig) .or.
     &         g(3) /= gvec(3,ig))
        ig = ig+1
        if(ig > npwvec) exit
      enddo
      return
      end
      
      
c=======================================================================
      subroutine findgtab(gvec, NPWVEC, gtab, NGTABM, ngtab)
c=======================================================================
c     it builds gtab that gives the number of gvec corresponding
c     to the three coordinates
c-------------------------------------------------------------------
      integer NPWVEC, NGTABM, ngtab
      integer gvec(3,NPWVEC)
      integer gtab(NGTABM,NGTABM,NGTABM)

      integer s

c     zeroes gtab
      do i = 1, NGTABM
        do j = 1, NGTABM
          do k = 1, NGTABM
            gtab(i,j,k) = 0
          enddo
        enddo
      enddo

c     find ngtab as the maximum absolute value taken by gvec in any direction
      ngtab = 0
      do i = 1, NPWVEC
        do j = 1, 3
          if (ngtab.lt.iabs(gvec(j,i))) then
            ngtab = iabs(gvec(j,i))
          endif
        enddo
      enddo

c     check that NGTABM is enough big to contain the [-ngtab,ngtab]
c     interval of values
      if (NGTABM.lt.ngtab*2+1) then
        write(*,*) '** ERROR identg 1: NGTABM too small;'
        write(*,*) '                   increase at ', ngtab*2+1
        stop
      endif

c     obtain gtab (s is the shift to not have negative indexes)
      s = ngtab+1
      do i = 1, NPWVEC
        gtab(gvec(1,i)+s,gvec(2,i)+s,gvec(3,i)+s) = i
      enddo

c     print gtab
c     do i = 1, NGTABM
c       do j = 1, NGTABM
c         do k = 1, NGTABM
c           write(*,6729) i, j, k, gtab(i,j,k)
c6729        format((3i3,i4))
c         enddo
c       enddo
c     enddo

      return
      end


!=======================================================================
      subroutine qtabg0(nqbz, qbz, QDENOMMX, QDENOM, KMKPTAB)
!=======================================================================
!  Find the denominator of the q-vectors QDENOM
!  and create the table which relates
!   q = (k - k') - G0 = (q1, q2, q3) - G0
!  to the q in the QBZ and the rec. lattice vector G0
!  by
!
!   IND = KMKPTAB(q1*QDENOM, q2*QDENOM, q3*QDENOM) 
!
!   IQBZ = IND / 1000 ; IND = IND - 1000*IQBZ
!   IG01 = IND / 100 ; IND = IND - 100*IG01 ; IG01 = IG01 - 2
!   IG02 = IND / 10  ; IND = IND - 10*IG02  ; IG02 = IG02 - 2
!   IG03 = IND - 2
!
!  Input:
!    NQBZ, QBZ, QDENOMMX
!  Ouput:
!    QDENOM, KMKPTAB
!----------------------------------------------------------------------

      implicit none
!      INTEGER, INTENT(IN)  :: NQBZ, QDENOMMX
       integer nqbz, QDENOMMX
!      REAL, DIMENSION(3,NQBZ), INTENT(IN)  :: QBZ
       real, dimension(3,nqbz) :: qbz
!      INTEGER, INTENT(OUT) :: QDENOM  
       integer qdenom
!      INTEGER, DIMENSION(-3*QDENOMMX:3*QDENOMMX,-3*QDENOMMX:3*QDENOMMX,
!     &             -3*QDENOMMX:3*QDENOMMX), INTENT(OUT) :: KMKPTAB
       integer, dimension(-3*QDENOMMX:3*QDENOMMX,-3*QDENOMMX:3*QDENOMMX,
     &             -3*QDENOMMX:3*QDENOMMX) :: KMKPTAB

       integer :: I, IQ, ITEMP, IND, IG01, IG02, IG03
       REAL    :: RTEMP
       REAL, DIMENSION(3) :: QPG0
       integer, EXTERNAL  :: ITST0


!     Find the denominator of the q-vectors QDENOM
       QDENOM = 1
!      IQ_LOOP: DO IQ = 1, NQBZ
       DO IQ = 1, NQBZ
         IF (ITST0(QBZ(1,IQ)) == 1) CYCLE
         DO I = 1, 3
           RTEMP = QBZ(I,IQ)
           IF (ABS(RTEMP) < 1E-3) CYCLE
           ITEMP = NINT(1. / RTEMP)
           IF (ITEMP > QDENOM) QDENOM = ITEMP
         ENDDO
!      ENDDO IQ_LOOP
      ENDDO
      IF (QDENOM > QDENOMMX) stop 'qtabg0 1: qdenom > qdenommx'

      KMKPTAB(:,:,:) = 0
      DO IQ = 1, NQBZ
        DO IG01 = -2, 2
        DO IG02 = -2, 2
        DO IG03 = -2, 2
          QPG0(1) = QBZ(1,IQ) + IG01
          QPG0(2) = QBZ(2,IQ) + IG02
          QPG0(3) = QBZ(3,IQ) + IG03
          QPG0(:) = QPG0(:) * QDENOM 
          IND = 1000*IQ + 100*(IG01+2) + 10*(IG02+2) + (IG03+2)
          KMKPTAB(NINT(QPG0(1)), NINT(QPG0(2)), NINT(QPG0(3))) = IND
        ENDDO
        ENDDO
        ENDDO
      ENDDO

      WRITE(*,'(/" The denominator of the q-vectors QDENOM =",I5)')
     &      QDENOM

      END SUBROUTINE QTABG0
      
     
!     ================================================================== 
      subroutine hermitianize(m,n)
!     ==================================================================
!     perform hermitianization of a matrix
!     i.e. force a matrix to be hermitian
!     added modifications to vectorialize the code by N. Vast 
!     ------------------------------------------------------------------
      implicit none
      integer n
      complex m(n,n)
      
      integer i, j
      complex t(n)
      
      do i = 1, n
        do j = i, n
          t(j) = (m(i,j) + conjg(m(j,i))) / 2.0
        enddo
        do j = i, n
          m(i,j) = t(j)
        enddo
        do j = i, n
          m(j,i) = conjg(t(j))
        enddo
      enddo
      return
      end subroutine hermitianize
      
      complex function dotproductqrc(r,c,b1,b2,b3)
      implicit none
      complex c(3)
      real r(3), b1(3), b2(3), b3(3)
      integer i
      dotproductqrc = 0.0
      do i = 1, 3
        dotproductqrc = dotproductqrc + 
     &    (r(1)*b1(i)+r(2)*b2(i)+r(3)*b3(i)) * 
     &    (c(1)*b1(i)+c(2)*b2(i)+c(3)*b3(i))
      enddo
      return
      end

  
       subroutine iqg0(k,qdenom,qdenommx,kmkptab,iq,g0)

       implicit none
       integer  qdenom,iq,i,qdenommx
       integer g0(3),kmkptab(-3*qdenommx:3*qdenommx,
     &           -3*qdenommx:3*qdenommx,-3*qdenommx:3*qdenommx)
       real k(3)

       k(:) = k(:) * qdenom
       i = kmkptab(nint(k(1)), nint(k(2)), nint(k(3)))
       if (i == 0) stop 'excheig 2: q = k-kp-G0 not found'
       iq = i / 1000 ; i = i - 1000*iq
       g0(1) = i / 100 ; i = i - 100*g0(1) ; g0(1) = g0(1) - 2
       g0(2) = i / 10  ; i = i - 10*g0(2)  ; g0(2) = g0(2) - 2
       g0(3) = i - 2

       end

