! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

subroutine rho_tw_g(ng,nr,nrb,nfft1a,nfft1,nfft2,nfft3,igfftg0,&
     &              wfn1,i1,ktabr1,ktabp1,wfn2,i2,ktabr2,ktabp2,rhotwg)
! calculates rhotwg(G) = < wfn1 | exp(-i*(q+G)*r) | wfn2 >
  implicit none
  integer ng, nr, nrb, nfft1,nfft1a,nfft2,nfft3
  integer i1, i2
  integer igfftg0(ng)
  integer ktabr1(nr), ktabr2(nr) ! tables R^-1 r for that k points
  complex ktabp1, ktabp2         ! phase factors for non-simmorphic symmetries
  complex wfn1(nr), wfn2(nr)
  complex rhotwg(ng)
  complex rhotw(nr)
  integer ir
  ! to account for symmetries
  ! phi_Rk(r) = phi_k(R^-1 r)
  ! phi_Ik(r) = phi_k*(r)
  ! form rho-twiddle(r)
  if(i1 == 1) then
    if(i2 == 1) then
      do ir = 1, nr
        rhotw(ir) = conjg(ktabp1 * wfn1(ktabr1(ir)))  *  ktabp2 * wfn2(ktabr2(ir))
      enddo
    else
      do ir = 1, nr
        rhotw(ir) = conjg(ktabp1 * wfn1(ktabr1(ir))  *  ktabp2 * wfn2(ktabr2(ir)))
      enddo
    endif
  else
    if(i2 == 1) then
      do ir = 1, nr
        rhotw(ir) = ktabp1 * wfn1(ktabr1(ir))  *  ktabp2 * wfn2(ktabr2(ir))
      enddo
    else
      do ir = 1, nr
        rhotw(ir) = ktabp1 * wfn1(ktabr1(ir))  *  conjg(ktabp2 * wfn2(ktabr2(ir)))
      enddo
    endif
  endif
  ! FFT to rho-twiddle (G)
  call c3dfft(rhotw,nfft1a,nfft2,nfft3,nfft1,nfft2,nfft3,-1)
  ! Set up rho-twiddle(G-G0) for matrix in PW order, normalising for FFT
  rhotwg(:) = rhotw(igfftg0(:)) / nrb
  return
end


subroutine rhotwg0q0(ng,gvec,ik,wfn1,wfn2,kb,e1,e2,iinv,op,q,b1,b2,b3,rhotw)
! calculates rhotwg(q=0,G=0) = < wfn1 | exp(-i*(q+G)*r) | wfn2 >
  use ggwdef
  implicit none
  type(kb_potential) :: kb
  integer ng
  integer gvec(3,ng)
  integer iinv, ik
  real e1, e2
  real q(3)
  real b1(3), b2(3), b3(3)
  real op(3,3)
  complex :: rhotw
  complex :: wfn1(ng), wfn2(ng)
  complex :: ct(3)
  real qrot(3), opinv(3,3), ediff
  integer ig, igp, is, ilm, i
  complex cta1, cta4, cta2(3), cta3(3)
  logical :: inclvkb = .true.
  complex :: fnlkslm(ng),fnldkslm(3,ng)
  complex, external :: pdtqrc

  ediff = e1 - e2
! first term <c,k|-iq*grad|v,k>
  ct(:) = 0
  do ig = 1, ng
    ct(:) = ct(:) + gvec(:,ig) * conjg(wfn1(ig)) * wfn2(ig)
  enddo
  
! second term <c,k|[Vnl,iqr]|v,k>
  if(inclvkb) then
!   do ig = 1, ng
!     do igp = 1, ng
!       ctemp = conjg(wfn1(ig)) * wfn2(igp)
!       must multiply by ucvol because yet gradvnl contain 1/ucvol
!       ct(:) = ct(:) + ctemp * gradvnl(:,ig,igp,ik)
!     enddo
!   enddo
    do is = 1, kb%natom
      do ilm = 1, kb%mpsang*kb%mpsang
        fnlkslm(:) = kb%fnl(:,ilm,is,ik)
        do i = 1, 3
          fnldkslm(i,:) = kb%fnld(i,:,ilm,is,ik)
        enddo
        cta1 = 0
        cta2(:) = 0
        cta3(:) = 0
        cta4 = 0
        do ig = 1, ng
          cta1 = cta1 + wfn1(ig)*fnlkslm(ig)
          cta2(:) = cta2(:) + wfn2(ig) * fnldkslm(:,ig)
          cta3(:) = cta3(:) + wfn1(ig) * fnldkslm(:,ig)
          cta4 = cta4 + wfn2(ig)*fnlkslm(ig)
        enddo
        ct(:) = ct(:) + conjg(cta1)*cta2(:)+conjg(cta3(:))*cta4
      enddo
    enddo
  endif
  ct(:) = - ct(:) / ediff ! check the sign
!       Forming e.Mcv(k)
!       [which is Re.Mcv(k) (or -Re.conjg(Mcv(k)) for inversion)
!       where kBZ = R**-1 In (kIBZ).]
!       First part: Re (or -Re  for inversion)
!           Forming e.Mcv(k)
!           Second part: e.Mcv(k) (or e.conjg(Mcv(k)) for inversion)
  opinv(:,:) = op(:,:)
  call rginv(3,opinv,3)
  call dosym(opinv,iinv,q,qrot)
  rhotw = pdtqrc(qrot,ct,b1,b2,b3)
  if(iinv == 2) rhotw = conjg(rhotw) ! check the sign
  
end


subroutine crhotwiddleg(ng,nr,nrb,nfft1a,nfft1,nfft2,nfft3,&
    &                   igfftg0,wfn1,wfn2,rhotwg)
! calculates rhotwg(G) = < wfn1 | exp(-i*(q+G)*r) | wfn2 >
! without symmetry operations and inversion
  implicit none
  integer nr, ng
  integer nrb
  integer nfft1,nfft1a,nfft2,nfft3
  integer igfftg0(ng)
  complex wfn1(nr), wfn2(nr)
  complex rhotwg(ng)
  complex rhotw(nr)
  ! form rho-twiddle(r)
  rhotw(:) = conjg(wfn1(:)) * wfn2(:)
  ! FFT to rho-twiddle(G)
  call c3dfft(rhotw,nfft1a,nfft2,nfft3,nfft1,nfft2,nfft3,-1)
  ! Set up rho-twiddle(G-G0) for matrix in PW order, normalising for FFT
  rhotwg(:) = rhotw(igfftg0(:)) / nrb
  return
end

