! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.

! symmetry: library which deals with symmetry (all hidden here)
! created june 2004 Valerio Olevano


subroutine setup_symmetry_structure(symstr)
  use ggwdef
  implicit none
  type(symmetry) symstr
  integer i, j, nsym
  logical, parameter :: onlysim = .false.
  logical, parameter :: intro_time_rev = .false.

  if(onlysim) then  ! remove non-simmorphic
    nsym = symstr%nsym
    i = 1
    do while(i <= nsym)
      if(any(abs(symstr%sym(i)%t(:)) > 0.0001)) then
        do j = i+1, nsym
          symstr%sym(j-1)%t(:) = symstr%sym(j)%t(:)
          symstr%sym(j-1)%r(:,:) = symstr%sym(j)%r(:,:)
        enddo
        nsym = nsym - 1
        i = i - 1
      endif
      i = i + 1
    enddo
    symstr%nsym = nsym
  endif  ! remove non-simmorphic

  if(intro_time_rev) then! introduce time reversal
    do i = symstr%nsym, 1, -1
      symstr%sym(2*i)%r(:,:) = symstr%sym(i)%r(:,:)
      symstr%sym(2*i)%t(:) = symstr%sym(i)%t(:)
      symstr%sym(2*i)%tr = -1
      symstr%sym(2*i-1)%r(:,:) = symstr%sym(i)%r(:,:)
      symstr%sym(2*i-1)%t(:) = symstr%sym(i)%t(:)
      symstr%sym(2*i-1)%tr = +1
    enddo
    symstr%nsym = symstr%nsym * 2
  endif
  
  do i = 1, symstr%nsym
    ! the inverse of { R , t} is { R^-1 , - R^-1 t }
    call mtriinv(symstr%isym(i)%r,symstr%sym(i)%r)
  ! BUG: the next should be negative but for a strange reason does not work???
  ! probably it is due to the fact that the sign of the FFT is reversed
  ! symstr%isym(i)%t = - matmul(symstr%isym(i)%r,symstr%sym(i)%t)
    symstr%isym(i)%t = matmul(symstr%isym(i)%r,symstr%sym(i)%t)
    symstr%isym(i)%tr = symstr%sym(i)%tr
    ! the inverse transpose
    symstr%itsym(i)%r = transpose(symstr%isym(i)%r)
    symstr%itsym(i)%t(:) = symstr%isym(i)%t(:)
    symstr%itsym(i)%tr = symstr%isym(i)%tr
  enddo
  call print_symmetry_structure(symstr)
  return
end

subroutine print_symmetry_structure(symstr)
  use ggwdef
  implicit none
  type(symmetry) symstr
  integer i, j, isym, isymin, isymend
  print *, 'symmetries of the system:'
  write(*,'("  number of symmetry operations ",i2)') symstr%nsym
  print *, ' symmetry operations (R t)'
  do isymin=1,symstr%nsym,4
    isymend=isymin+3
    if(isymend>symstr%nsym) isymend=symstr%nsym
    do i=1,3
      write(*,'(4(3i3,f8.3,3x))') &
&      ((symstr%sym(isym)%r(i,j),j=1,3),symstr%sym(isym)%t(i),isym=isymin,isymend)
    enddo
    print *
  enddo
  print *, ' inverse symmetry operations (R^-1 , -R^-1 t)'
  do isymin=1,symstr%nsym,4
    isymend=isymin+3
    if(isymend>symstr%nsym) isymend=symstr%nsym
    do i=1,3
      write(*,'(4(3i3,f8.3,3x))') &
&      ((symstr%isym(isym)%r(i,j),j=1,3),symstr%isym(isym)%t(i),isym=isymin,isymend)
    enddo
    print *
  enddo
  print *, ' inverse transpose symmetry operations (R^-1^t , -R^-1 t)'
  do isymin=1,symstr%nsym,4
    isymend=isymin+3
    if(isymend>symstr%nsym) isymend=symstr%nsym
    do i=1,3
      write(*,'(4(3i3,f8.3,3x))') &
&      ((symstr%itsym(isym)%r(i,j),j=1,3),symstr%itsym(isym)%t(i),isym=isymin,isymend)
    enddo
    print *
  enddo
  return
end


integer function mtridet(a)
  implicit none
  integer a(3,3)

  mtridet =   a(1,1) * a(2,2) * a(3,3) &
  &         - a(1,1) * a(2,3) * a(3,2) &
  &         - a(1,2) * a(2,1) * a(3,3) &
  &         + a(1,2) * a(2,3) * a(3,1) &
  &         + a(1,3) * a(2,1) * a(3,2) &
  &         - a(1,3) * a(2,2) * a(3,1)
  return
end


subroutine mtriinv(a,b)
  implicit none
  integer a(0:2,0:2), b(0:2,0:2), c(0:2,0:2)
  integer det
  integer, external :: mtridet
  det = mtridet(b)
  a(0,0) = (  b(1,1) * b(2,2) - b(1,2) * b(2,1)) / det
  a(1,0) = (- b(1,0) * b(2,2) + b(1,2) * b(2,0)) / det 
  a(2,0) = (  b(1,0) * b(2,1) - b(1,1) * b(2,0)) / det
  a(0,1) = (- b(0,1) * b(2,2) + b(0,2) * b(2,1)) / det
  a(1,1) = (  b(0,0) * b(2,2) - b(0,2) * b(2,0)) / det 
  a(2,1) = (- b(0,0) * b(2,1) + b(0,1) * b(2,0)) / det
  a(0,2) = (  b(0,1) * b(1,2) - b(0,2) * b(1,1)) / det
  a(1,2) = (- b(0,0) * b(1,2) + b(0,2) * b(1,0)) / det 
  a(2,2) = (  b(0,0) * b(1,1) - b(0,1) * b(1,0)) / det
  
  c = matmul(a,b)
  if(c(0,0)/=1 .or. c(1,1)/=1 .or. c(2,2)/=1) stop 'wrong inverse'
  if(c(0,1)/=0 .or. c(0,2)/=0 .or. c(1,2)/=0) stop 'wrong inverse'
  if(c(1,0)/=0 .or. c(2,0)/=0 .or. c(2,1)/=0) stop 'wrong inverse'
  return
end


subroutine setup_rrotation_table(symstr,fftmesh,rrottab)
  use ggwdef
  implicit none
  type(symmetry) symstr
  type(mesh) fftmesh
  integer rrottab(fftmesh%nr,symstr%nsym)
  integer i1, i2, i3, ir, j1, j2, j3
  integer is, n(3)
  double precision r(3), rp(3)
  integer, external :: modx
  
  print *, 'setting up r-rotation table'
  n(:) = fftmesh%nrr(:)
  do i1 = 0, n(1) - 1
    do i2 = 0, n(2) - 1
      do i3 = 0, n(3) - 1
        ir = 1 + i1 + i2 * n(1) + i3 * n(1) * n(2)
        r(1) = dble(i1) / n(1)
        r(2) = dble(i2) / n(2)
        r(3) = dble(i3) / n(3)
        do is = 1, symstr%nsym
          rp(:) = matmul(symstr%isym(is)%r(:,:),r(:)) + symstr%isym(is)%t(:)
          j1 = nint(rp(1) * n(1))
          j2 = nint(rp(2) * n(2))
          j3 = nint(rp(3) * n(3))
          j1 = modx(j1,n(1))
          j2 = modx(j2,n(2))
          j3 = modx(j3,n(3))
          rrottab(ir,is) = 1 + j1 + j2 * n(1) + j3 * n(1) * n(2)
        enddo
      enddo
    enddo
  enddo
  print *, 'r-rotation table setup'
  print *
  return
end


subroutine setup_grotation_table(symstr,ng,gvec,grottab)
  use ggwdef
  implicit none
  type(symmetry) symstr
  integer ng
  integer gvec(3,ng)
  ! f95 form integer grottab(ng,1:-1:-2,symstr%nsym)
  integer grottab(ng,2,symstr%nsym)
  integer ig, gp(3), igp, is, inv, iinv
  
  print *, 'setting up G-rotation table'
  grottab(:,:,:) = -1
  do ig = 1, ng
    do inv = 1, -1, -2
      iinv = 1
      if(inv == -1) iinv = 2
      do is = 1, symstr%nsym
        gp(:) = inv * matmul(symstr%itsym(is)%r(:,:),gvec(:,ig))
        do igp = 1, ng
          if(all(gp(:) == gvec(:,igp))) grottab(ig,iinv,is) = igp
        enddo
      enddo
    enddo
  enddo
  if(any(grottab(:,:,:) == -1)) stop 'G-shell not closed'
  print *, 'G-rotation table setup'
  print *
  return
end

subroutine setup_ibz(nkibz,kibz,k)
  use ggwdef
  implicit none
  type(bzstructure) k
  integer nkibz
  real kibz(3,nkibz)
  integer ik, istat
  
  k%nibz = nkibz
  allocate(k%ibz(k%nibz),stat=istat)
  do ik = 1, k%nibz
    k%ibz(ik)%rc(:) = kibz(:,ik)
  enddo
  return
end


subroutine expand_ibz(symstr,k,ninv,ktabi)
  use ggwdef
  implicit none
  type(symmetry) symstr
  type(bzstructure) k
  integer ktabi(*)
  integer ikibz, is, ikbz, ik, i
  integer ninv, iinv, einv
  logical, external :: cmpkbz
  
  allocate(k%bz(k%nibz*symstr%nsym*ninv))
  einv = 1
  if(ninv == 2) einv = -1
  print *, 'expanding IBZ to BZ'
  k%nbz = 0
  do ikibz = 1, k%nibz
    k%ibz(ikibz)%w = 0
    do iinv = 1, einv, -2
      do is = 1, symstr%nsym
        k%nbz = k%nbz + 1
        k%bz(k%nbz)%rc(:) = iinv * matmul(symstr%itsym(is)%r(:,:),k%ibz(ikibz)%rc(:))
        k%bz(k%nbz)%ibz = ikibz
        k%bz(k%nbz)%isym = is
        ktabi(k%nbz) = iinv  ! to do: include inversion in symmetries
        k%ibz(ikibz)%w = k%ibz(ikibz)%w + 1
        do ikbz = 1, k%nbz - 1
          if(cmpkbz(k%bz(k%nbz)%rc(:),k%bz(ikbz)%rc(:))) then
            k%nbz = k%nbz - 1
            k%ibz(ikibz)%w = k%ibz(ikibz)%w - 1
            exit
          endif
        enddo
      enddo
    enddo
    k%ibz(ikibz)%w = k%ibz(ikibz)%w / (symstr%nsym * ninv)
  enddo
! print out results
  write(*,'("  number of k-points in the IBZ ",i5)') k%nibz
  print *, ' k-points [reduced coordinates]    weights'
  write(*,'(3(1x,f10.6),2x,f6.3)') &
      &  ((k%ibz(ik)%rc(i),i=1,3),k%ibz(ik)%w,ik=1,k%nibz)
  write(*,'("  number of k-points in the full BZ ",i5)') k%nbz
  print *, ' k-points [reduced coordinates]:   ibz   sym   timer'
  write(*,'(3(1x,f10.6),3i6)') ((k%bz(ik)%rc(i),i=1,3), &
    &  k%bz(ik)%ibz, k%bz(ik)%isym, ktabi(ik), ik=1,k%nbz)
  print *
  return
end

logical function cmpkbz(k1,k2)
  double precision :: k1(3), k2(3), kd(3)
  cmpkbz = .false.
  kd(:) = k1(:) - k2(:)
  if(all(abs(kd(:) - nint(kd(:))) < 1.e-4)) cmpkbz = .true.
  return
end
