! Copyright (C) 1998-2007 Valerio Olevano, Lucia Reining, Francesco Sottile, CNRS.
! All rights reserved.
!
! DP code
! Linear Response TDDFT code in Frequency-Reciprocal space on a Plane Waves basis set.
! Calculate Dielectric Properties.


!=======================================================================
      subroutine cmevxclda(nbcmax,
     &  nbnds, NKIBZM, nkibz, nkbz, nop, ninv, nr, nrb,
     &  ucvol, irottb, wtk, ktab,
     &  nkcalc, kcalc, minbnd, maxbnd,
     &  occ, wfr, mevxclda)
!=======================================================================
c     calculate <VxcLDA> for this band (only on first trip)
c     [actually do it for the kIBZ corresponding to this kBZ, but this
c     should be the same since Vxc(r) has the full symmetry]
!-----------------------------------------------------------------------
      implicit none
      integer NBCMAX, NKIBZM, nkibz, nkbz, nop, ninv, nr, nrb, nbnds
      integer irottb(nr,nop)
      real wtk(nkibz)
      integer ktab(nkbz)
      real ucvol
      integer nkcalc, kcalc(nkcalc), minbnd(nkcalc), maxbnd(nkcalc)
      real occ(NKIBZM,*)
      complex wfr(nr,nbnds,NKIBZM)

      complex mevxclda(NBCMAX,nkcalc)

      complex ctemp
      real, allocatable :: vxclda(:)

      integer ik, ib, ibr, ikibz, ir
      integer istat
      logical, parameter :: VERBOSE = .true.


      allocate(vxclda(nr),stat=istat)
      if(istat.ne.0) stop 'out of memory'

      call cvxclda(
     &  nbnds,NKIBZM,nkibz,nkbz,nop,ninv,nr,nrb,
     &  ucvol,irottb,wtk,
     &  occ,wfr,
     &  vxclda)


      do ik = 1, nkcalc
        ikibz = ktab(kcalc(ik))
        do ib = minbnd(ik), maxbnd(ik)
          ibr = ib - minbnd(ik) + 1
          ctemp = 0.0
          do ir = 1, nr
            ctemp = ctemp + conjg(wfr(ir,ib,ikibz)) *
     &        vxclda(ir) * wfr(ir,ib,ikibz)
          enddo
          mevxclda(ibr,ik) = ctemp / nrb
        enddo
      enddo

      if (VERBOSE) then
        do ik = 1, nkcalc
          write(6,5) ik
5         format(' LDA exchange energies for k-point ', i3/
     &           '  n          <VxcLDA>')
          do ib = minbnd(ik), maxbnd(ik)
            ibr = ib - minbnd(ik) + 1
            write(6,10) ib, mevxclda(ibr,ik) * 27.2116
10          format(i3,2e14.6)
          enddo
        enddo
      endif

      write(*,*)

      deallocate(vxclda)
      return
      end


!=======================================================================
      subroutine cvxclda(
     &  nbnds,NKIBZM,nkibz,nkbz,nop,ninv,nr,nrb,
     &  ucvol,irottb,wtk,
     &  occ,wfr,
     &  vxclda)
!=======================================================================
c     Calculate charge density and VxcLDA
c     Input:
c      wfrs
c     Output:
c      VXCLDA: VxcLDA on FFT grid
!-----------------------------------------------------------------------
      implicit none
      integer NKIBZM, nbnds
      integer nkibz, nkbz, nop, ninv, nr, nrb
      real ucvol
      REAL OCC(NKIBZM,*)
      REAL WTK(NKIBZ), VXCLDA(NR)
      INTEGER IROTTB(nr,NOP)

      complex wfr(nr,nbnds,NKIBZM)

      real rho(nr), rho2(nr)
      real enpuc

      integer ik, ir, ib, iop
      real fact
c      real, external :: vxcca


c     print info
      write(*,*) 'calculating charge density and VxcLDA'


c     zero charge density
      do ir = 1, nr
        rho(ir) = 0.0
        rho2(ir) = 0.0
      enddo

c     loop over k-points in IBZ
      do ik = 1, NKIBZ
        do ib = 1, nbnds
c         skip the higher bands if occupation is less than 1E-5
          if (occ(ik,ib).gt.1.0e-5) then
            do ir = 1, nr
              rho(ir) = rho(ir) +
     &          occ(ik,ib)*conjg(wfr(ir,ib,ik))*wfr(ir,ib,ik)*wtk(ik)
            enddo
          endif
        enddo
      enddo

c     loop over symmetry operations, symmetrising rho
c     factor of 2 is for inversion
      fact = real(ninv) / (NKBZ*UCVOL)
      do IOP = 1, NOP
        do IR = 1, NR
          rho2(IR) = rho2(IR) + fact * rho(IROTTB(IR,IOP))
        enddo
      enddo

c     calculate total no. of electrons as a check
      enpuc = 0.0
      do ir = 1, nr
        enpuc = enpuc + rho2(ir)
      enddo
      enpuc = enpuc * ucvol / nrb
      write (6,1113) enpuc
1113  format(' total number of electrons per unit cell = ',f7.4)
      
c     form VxcLDA (in Hartrees)
      do IR = 1, NR
c        VXCLDA(IR) = VXCCA(rho2(IR))
      enddo

      RETURN
      END


!=======================================================================
      subroutine ckxclda(nfft1,nfft1a,nfft2,nfft3,
     &                   nr,rho2,Kxclda,NPWMAT,gvec)
!=======================================================================
!     calculate exchange-correlation kernel Kxclda in GG' space
!-----------------------------------------------------------------------
      implicit none
      integer nfft1,nfft1a,nfft2,nfft3
      integer nr, NPWMAT
      real rho2(nr)
      complex Kxclda(NPWMAT,NPWMAT)
      integer gvec(3,NPWMAT)
      complex Kxcldagmg(nr)
      integer ig2,ig3,ig4,ig4x,ig4y,ig4z
!     the same, modx and modx2
      integer :: modx2, i, j
      modx2(i,j) = mod( mod(i,j) + ((1-sign(1,i))/2)*j , j)

!     calculate kernel in G-G' form
      call ckxcldag(nfft1,nfft1a,nfft2,nfft3,nr,rho2,Kxcldagmg)
      do ig2 = 1, NPWMAT
        do ig3 = 1, NPWMAT
          ig4x = modx2(gvec(1,ig3)-gvec(1,ig2),nfft1)
          ig4y = modx2(gvec(2,ig3)-gvec(2,ig2),nfft2)
          ig4z = modx2(gvec(3,ig3)-gvec(3,ig2),nfft3)
          ig4 = 1 + ig4x + ig4y*nfft1a + ig4z*nfft1a*nfft2
          Kxclda(ig3,ig2) = Kxcldagmg(ig4)
        enddo
      enddo
      return
      end

!=======================================================================
      subroutine ckxcldag(nfft1,nfft1a,nfft2,nfft3,
     &                    nr,rho2,Kxclda)
!=======================================================================
!     calculate exchange-correlation kernel Kxclda in G-space
!     put it into Kxclda in igfft form
!-----------------------------------------------------------------------

      integer nfft1,nfft1a,nfft2,nfft3
      integer nr
      real rho2(nr)
      complex Kxclda(nr)
      real fftfct

      call ckxcldar(nr,rho2,Kxclda)

      write(*,*) 'fft Kxclda(r) -> Kxclda(G)'
      write(*,*)

      call c3dfft(Kxclda,nfft1a,nfft2,nfft3,nfft1,nfft2,nfft3,-1)

      fftfct = 1 / real(nfft1*nfft2*nfft3)
!     do ig = 1, NPWVEC
!       kxcg(ig) = Kxclda(igfft(ig,3,3,3))*fftfct
!     enddo

!     renormalization after fft
      Kxclda(:) = Kxclda(:)*fftfct

      return
      end


c=======================================================================
      subroutine ckxcldar(nr,rho2,Kxclda)
c=======================================================================
c     calculate exchange-correlation kernel Kxclda
c-----------------------------------------------------------------------
      use xc_f90_lib_m
      implicit none

      integer nr, ir
      real rho2(nr)
      complex Kxclda(nr)
      double precision :: rho, f_x, f_c
      type(xc_f90_func_t) :: px, pc
      type(xc_f90_info_t) :: ix, ic

      call xc_f90_lda_init(px, ix, XC_LDA_X, XC_UNPOLARIZED,
     &                     3, XC_RELATIVISTIC)
      call xc_f90_lda_init(pc, ic, XC_LDA_C_PZ, XC_UNPOLARIZED)

      write(*,*) 'calculating KxcLDA'
      do ir = 1, nr
        rho = rho2(ir)
        call xc_f90_lda_fxc(px, rho, f_x)
        call xc_f90_lda_fxc(pc, rho, f_c)
        Kxclda(ir) = f_x + f_c
      enddo
      call xc_f90_lda_end(px)
      call xc_f90_lda_end(pc)
      write(*,*)
      return
      end



